#include <windows.h>
#include <stdio.h>
#include "commands.h"
#include "osbdm.h"
#include "Resource.h"
#include "Configure.h"
#include "common_usb.h"
#include "log.h"

LRESULT CALLBACK Configure(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

int enableTargetVdd            = 0;
enum { vdd5V, vdd3V3 } targetVdd = vdd3V3;
int enableCycleVddOnReset      = 1;
int enableCycleVddOnConnect    = 1;
int enableLeaveTargetPowered   = 0;
int enableAutomaticReconnect   = 0;
int enableGuessSpeed           = 1;
int enableUseAltBDMTargetClock = 0;
int enableUseResetSignal       = 1;

int capabilities               = 0;

unsigned char usb_data[10];

void OSBDM_API bdm_set_option(unsigned char option, unsigned char value) {
   usb_data[0] = 2;  // receive up to 2 bytes
   usb_data[1] = CMD_OPTION;
   usb_data[2] = (unsigned char)option;
   usb_data[3] = (unsigned char)value;
   bdm_usb_recv_ep0(usb_data);
}

unsigned int OSBDM_API bdm_get_capabilities(void) {
   usb_data[0] = 3;  // receive up to 3 bytes
   usb_data[1] = CMD_OPTION;
   usb_data[2] = BDM_GET_CAPABILITIES;
   bdm_usb_recv_ep0(usb_data);
   return ((usb_data[2]<<8)+usb_data[1]);
}

void configureBDM() {
   if (!enableTargetVdd) {
      bdm_set_option(BDM_OPT_SET_TARGET_VDD,                BDM_TARGET_VDD_OFF);
      bdm_set_option(BDM_OPT_CYCLE_TARGET_VDD_ON_RESET,     0);
      bdm_set_option(BDM_OPT_CYCLE_TARGET_VDD_ON_CONNECT,   0);
      bdm_set_option(BDM_OPT_LEAVE_TARGET_POWERED,          0);
   }
   else {
      if (targetVdd == vdd3V3)
         bdm_set_option(BDM_OPT_SET_TARGET_VDD,  BDM_TARGET_VDD_3V3);
      else if (targetVdd == vdd5V)
         bdm_set_option(BDM_OPT_SET_TARGET_VDD,  BDM_TARGET_VDD_5V);
      bdm_set_option(BDM_OPT_CYCLE_TARGET_VDD_ON_RESET,     enableCycleVddOnReset);
      bdm_set_option(BDM_OPT_CYCLE_TARGET_VDD_ON_CONNECT,   enableCycleVddOnConnect);
      bdm_set_option(BDM_OPT_LEAVE_TARGET_POWERED,          enableLeaveTargetPowered);
   }
   bdm_set_option(BDM_OPT_AUTOMATIC_RECONNECT,        enableAutomaticReconnect);
   bdm_set_option(BDM_OPT_GUESS_SPEED,                enableGuessSpeed);
   bdm_set_option(BDM_OPT_USE_ALT_BDM_CLOCK,          enableUseAltBDMTargetClock);
   bdm_set_option(BDM_OPT_USE_RESET_SIGNAL,           enableUseResetSignal);
}

void setToDefault(void) {
   enableTargetVdd            = 0;
   targetVdd                  = vdd3V3;
   enableCycleVddOnReset      = 0;
   enableCycleVddOnConnect    = 1;
   enableLeaveTargetPowered   = 0;
   enableAutomaticReconnect   = 1;
   enableGuessSpeed           = 1;
   enableUseAltBDMTargetClock = 0;
   enableUseResetSignal       = 0;
}

void updateDialog(HWND hDlg) {
   HWND VddEnableEn              = GetDlgItem(hDlg, IDC_VDD_CHECKBOX);
   HWND radio3V3En               = GetDlgItem(hDlg, IDC_3V3_RADIO);
   HWND radio5VEn                = GetDlgItem(hDlg, IDC_5V_RADIO);
   HWND checkVddCycleOnReset     = GetDlgItem(hDlg, IDC_CYCLE_VDD_ON_RESET_CHECKBOX);
   HWND checkVddCycleOnConnect   = GetDlgItem(hDlg, IDC_CYCLE_VDD_ON_CONNECTION_CHECKBOX);
   HWND checkVddLeaveOn          = GetDlgItem(hDlg, IDC_LEAVE_TARGET_ON_CHECKBOX);
   HWND checkUseResetSignal      = GetDlgItem(hDlg, IDC_USE_RESET_CHECKBOX);

   if (!(capabilities & BDM_CAP_VDDCONTROL)) {
      // Vdd control options disabled
      enableTargetVdd          = 0;
      enableCycleVddOnReset    = 0;
      enableLeaveTargetPowered = 0;
      enableCycleVddOnConnect  = 0;
      EnableWindow(VddEnableEn, 0 );
   }
   if (!(capabilities & BDM_CAP_RESET)) {
      // Reset control options disabled
      enableCycleVddOnConnect  = 0;
      EnableWindow(checkUseResetSignal, 0 );
   }
   CheckDlgButton(hDlg, IDC_VDD_CHECKBOX, enableTargetVdd);
   EnableWindow(radio3V3En,               enableTargetVdd );
   EnableWindow(radio5VEn,                enableTargetVdd );
   EnableWindow(checkVddCycleOnReset,     enableTargetVdd );
   EnableWindow(checkVddCycleOnConnect,   enableTargetVdd );
   EnableWindow(checkVddLeaveOn,          enableTargetVdd );

//   BDM_CAP_RESET        = (1<<0),   // RESET can be driven
//   BDM_CAP_FLASH        = (1<<1),   // 12 V Flash programming supply available (RS08)
//   BDM_CAP_VDDSENSE     = (1<<3),   // Sensing of target Vdd
   
   switch(targetVdd){
      case vdd5V :
         CheckRadioButton(hDlg, IDC_3V3_RADIO, IDC_5V_RADIO, IDC_5V_RADIO);
         break;
      case vdd3V3 :
         CheckRadioButton(hDlg, IDC_3V3_RADIO, IDC_5V_RADIO, IDC_3V3_RADIO);
         break;
   }
   
   CheckDlgButton(hDlg, IDC_CYCLE_VDD_ON_RESET_CHECKBOX,       enableCycleVddOnReset);
   CheckDlgButton(hDlg, IDC_CYCLE_VDD_ON_CONNECTION_CHECKBOX,  enableCycleVddOnConnect);
   CheckDlgButton(hDlg, IDC_LEAVE_TARGET_ON_CHECKBOX,          enableLeaveTargetPowered);

   CheckDlgButton(hDlg, IDC_RECONNECT_CHECKBOX,                enableAutomaticReconnect);
   CheckDlgButton(hDlg, IDC_GUESS_SPEED_CHECKBOX,              enableGuessSpeed);
   CheckDlgButton(hDlg, IDC_USE_ALT_BDM_CLOCK_CHECKBOX,        enableUseAltBDMTargetClock);
   CheckDlgButton(hDlg, IDC_USE_RESET_CHECKBOX,                enableUseResetSignal);
}

LRESULT CALLBACK Configure(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  // WM_COMMAND wNotifyCode = HIWORD(wParam); 
  // wID = LOWORD(wParam); 
  // hwndCtl = (HWND) lParam;
     
   switch (message) {
      case WM_INITDIALOG:
         setToDefault();
         updateDialog(hDlg);
         return TRUE;

      case WM_COMMAND:
         switch (LOWORD(wParam)) {
            case IDOK :
            case IDCANCEL :
               EndDialog(hDlg, LOWORD(wParam));
               return TRUE;
            case IDC_VDD_CHECKBOX :
               enableTargetVdd     = !enableTargetVdd;
               updateDialog(hDlg);
               break;
            case IDC_LEAVE_TARGET_ON_CHECKBOX :
               enableLeaveTargetPowered  = !enableLeaveTargetPowered;
               updateDialog(hDlg);
               break;
            case IDC_3V3_RADIO :
               targetVdd                 = vdd3V3;
               updateDialog(hDlg);
               break;
            case IDC_5V_RADIO :
               targetVdd                 = vdd5V;
               updateDialog(hDlg);
               break;
            case IDC_CYCLE_VDD_ON_RESET_CHECKBOX :
               enableCycleVddOnReset     = !enableCycleVddOnReset;
               updateDialog(hDlg);
               break;
            case IDC_CYCLE_VDD_ON_CONNECTION_CHECKBOX :
               enableCycleVddOnConnect   = !enableCycleVddOnConnect;
               updateDialog(hDlg);
               break;
            case IDC_RECONNECT_CHECKBOX :
               enableAutomaticReconnect  = !enableAutomaticReconnect;
               updateDialog(hDlg);
               break;
//            case IDC_RESET_ON_PROBLEM_CHECKBOX :
//               enableProblemReset        = !enableProblemReset;
//               updateDialog(hDlg);
//               break;
            case IDC_GUESS_SPEED_CHECKBOX :
               enableGuessSpeed            = !enableGuessSpeed;
               updateDialog(hDlg);
               break;
            case IDC_USE_ALT_BDM_CLOCK_CHECKBOX :
               enableUseAltBDMTargetClock  = !enableUseAltBDMTargetClock;
               updateDialog(hDlg);
               break;
            case IDC_USE_RESET_CHECKBOX :
               enableUseResetSignal  = !enableUseResetSignal;
               updateDialog(hDlg);
               break;
            case IDDEFAULT :
               setToDefault();
               updateDialog(hDlg);
               break;
               
            default :
               return FALSE;
         }
   }
   return FALSE;
}

void configureDialog(HINSTANCE hDLLInst){
int rc;

   print("configureDialog: hDLLInst = %d\r\n", hDLLInst);

   capabilities = bdm_get_capabilities();
   
   rc = DialogBox(hDLLInst, (LPCTSTR)IDD_USBDM_CONFIGURE, 0, (DLGPROC)Configure);
   
   print("configureDialog: rc = %d\r\n", rc);
   if (rc != 0) {
      LPVOID lpMsgBuf;
      DWORD dw = GetLastError(); 

      FormatMessage(
          FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |FORMAT_MESSAGE_IGNORE_INSERTS,
          NULL,
          dw,
          MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
          (LPTSTR) &lpMsgBuf,
          0, NULL );
      print("configureDialog: Error %s", lpMsgBuf);
   }
}

