#ifndef _OSBDM_H_
#define _OSBDM_H_

/* header file for TBDML.DLL */
#if defined(DLL)
#define OSBDM_API WINAPI __declspec(dllexport)
#elif defined (COMMANDLINE)
#define OSBDM_API
#else
#define OSBDM_API WINAPI __declspec(dllimport)
#endif

typedef enum {
   HC12  = 0,
   HCS08 = 1,
   RS08  = 2,
   CFV1  = 3,
   T_OFF = 0xFF,
} target_type_e;

typedef enum {	/* type of reset action required */
	SPECIAL_MODE=0,
	NORMAL_MODE=1
} target_mode_e;

typedef enum {	/* state of ACKN feature (supported or not) */
	WAIT=0,
	ACKN=1
} ackn_state_e;

typedef enum {	/* target reset detection state */
	RESET_INACTIVE=0,
	RESET_DETECTED=1
} reset_state_e;

typedef enum {	/* how was the target speed detected */
	NO_CONNECTION=0,
	SYNC=1,
	RESERVED=2,
	MANUAL_SETUP=3
} connection_state_e;

typedef struct { /* state of BDM communication */
	ackn_state_e        ackn_state;
	reset_state_e       reset_state;
	connection_state_e  connection_state;
} bdm_status_t;

typedef union {
	struct {
		unsigned int pc;
		unsigned int sp;
		unsigned int ix;
		unsigned int iy;
		unsigned int d;
		unsigned int ccr;
	} hc12;
	struct {
		unsigned int pc;
		unsigned int sp;
		unsigned int hx;
		unsigned int a;
		unsigned int ccr;
	} hcs08;
} registers_t;

// Options
enum {
   BDM_GET_CAPABILITIES                = 0, // Returns capability vector for hardware
   BDM_OPT_SET_TARGET_VDD              = 1, // Set target Vdd (Off, 3V3 or 5V)
   BDM_OPT_CYCLE_TARGET_VDD_ON_CONNECT = 2, // Enable/disable Cycling Target Vdd if connection problems
   BDM_OPT_AUTOMATIC_RECONNECT         = 3, // Enable/disable automatic reconnect
   BDM_OPT_GUESS_SPEED                 = 4, // Enable/disable guessing speed for target w/o ACKN feature (HCS12)
   BDM_OPT_LEAVE_TARGET_POWERED        = 5, // Enable/disable leaving target powered when exiting
   BDM_OPT_USE_ALT_BDM_CLOCK           = 6, // Enable/disable using alternative BDM clock source in target (HCS08)
   BDM_OPT_CYCLE_TARGET_VDD_ON_RESET   = 7, // Enable/disable Cycling Target Vdd  on Reset
   BDM_OPT_USE_RESET_SIGNAL            = 8, // Enable/disable use of RESET signal on BDM interface
};

// Vdd options
enum {
   BDM_TARGET_VDD_OFF      = 0,   
   BDM_TARGET_VDD_3V3      = 1,   
   BDM_TARGET_VDD_5V       = 2,
};   
   
unsigned char  OSBDM_API _opensourcebdm_dll_version(void); 				         /* returns DLL version in BCD */
unsigned char  OSBDM_API _opensourcebdm_init(void);						         /* initialises USB and returns number of devices found */
unsigned char  OSBDM_API _opensourcebdm_open(unsigned char device_no); 	      /* opens a device with given number (0...), returns 0 on success and 1 on error */
void           OSBDM_API _opensourcebdm_close(void);								   /* closes currently open device */
unsigned int   OSBDM_API _opensourcebdm_get_version(void);				         /* returns version of HW (MSB) and SW (LSB) of the interface in BCD format */
unsigned char  OSBDM_API _opensourcebdm_get_last_sts(void);				         /* returns status of the last command: 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_set_target_type(target_type_e target_type); /* sets target MCU type, returns 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_target_sync(void); 				         /* measures BDM frequency of the target using the SYNC feature and connects to it, returns 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_target_reset(target_mode_e target_mode); /* resets the target to normal or special mode, returns 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_bdm_sts(bdm_status_t *bdm_status);    /* fills user supplied structure with actual state of BDM communication channel, returns 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_read_bd(unsigned int address); 	      /* reads one byte from the BDM area of memory */
unsigned char  OSBDM_API _opensourcebdm_write_bd(unsigned int address, unsigned char data); /* writes one byte to the BDM area of memory, returns 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_target_go(void);                      /* starts target execution from current PC address, returns 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_target_step(void); 				         /* steps over a single target instruction, returns 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_target_halt(void); 				         /* brings the target into active background mode, returns 0 on success and non-zero on failure */
unsigned char  OSBDM_API _opensourcebdm_set_speed(float crystal_frequency);   /* sets the communication speed, parameter is crystal frequency in MHz, returns 0 on success and non-zero on failure */
float          OSBDM_API _opensourcebdm_get_speed(void); 						   /* returns crystal frequency of the target in MHz */
unsigned char  OSBDM_API _opensourcebdm_read_byte(unsigned int address);      /* reads one byte from memory */
void           OSBDM_API _opensourcebdm_write_byte(unsigned int address, unsigned char data); /* writes one byte to memory */
unsigned int   OSBDM_API _opensourcebdm_read_word(unsigned int address); 	   /* reads one word from memory (address must be aligned) */
void           OSBDM_API _opensourcebdm_write_word(unsigned int address, unsigned int data); /* writes one word to memory (address must be aligned) */
void           OSBDM_API _opensourcebdm_read_block(unsigned int address, unsigned int count, unsigned char *data); /* reads block from memory, data is returned to the user supplied buffer */
void           OSBDM_API _opensourcebdm_write_block(unsigned int address, unsigned int count, unsigned char *data); /* writes block to memory */
unsigned char  OSBDM_API _opensourcebdm_read_regs(registers_t *registers);    /* reads contents of core registers, returns 0 on success and non-zero on failure */
void           OSBDM_API _opensourcebdm_write_reg_pc(unsigned int value); 		/* writes PC */
void           OSBDM_API _opensourcebdm_write_reg_sp(unsigned int value); 		/* writes SP */
void           OSBDM_API _opensourcebdm_write_reg_x(unsigned int value);	 	/* writes H:X (HCS08) or IX (HC12) */
void           OSBDM_API _opensourcebdm_write_reg_y(unsigned int value); 		/* writes IY (HC12 only) */
void           OSBDM_API _opensourcebdm_write_reg_d(unsigned int value); 		/* writes B:A (HC12) or A (HCS08) */
void           OSBDM_API _opensourcebdm_write_reg_ccr(unsigned int value); 	/* writes CCR */

void           OSBDM_API _opensourcebdm_mem_dlstart(void);
void           OSBDM_API _opensourcebdm_mem_dlend(void);
unsigned char  OSBDM_API _opensourcebdm_target_readyfor_datald(void);
unsigned int   OSBDM_API _opensourcebdm_read_bkpt(void);
void           OSBDM_API _opensourcebdm_write_bkpt(unsigned int value);
unsigned char  OSBDM_API _opensourcebdm_read_status(void);
void           OSBDM_API _opensourcebdm_write_control(unsigned char value);


unsigned int   OSBDM_API _opensourcebdm_read_creg(unsigned char regNo);
unsigned int   OSBDM_API _opensourcebdm_read_dreg(unsigned char regNo);
BOOL           OSBDM_API _opensourcebdm_read_mem(unsigned int addr,     unsigned int size,
                                                 unsigned int memSpace, unsigned char *data);
unsigned int   OSBDM_API _opensourcebdm_read_reg(unsigned char regNo);
unsigned int   OSBDM_API _opensourcebdm_read_xcsr_byte(void);
unsigned char  OSBDM_API _opensourcebdm_set_derivative_type(unsigned char derivative);
BOOL           OSBDM_API _opensourcebdm_write_mem(unsigned int addr, unsigned int count, 
                                                  unsigned int memSpace, unsigned char *data);
void           OSBDM_API _opensourcebdm_write_creg(unsigned char regNo, unsigned int value);
void           OSBDM_API _opensourcebdm_write_dreg(unsigned char regNo, unsigned int value);
void           OSBDM_API _opensourcebdm_write_reg(unsigned char regNo, unsigned int value);

#if defined (COMMANDLINE)
void _opensourcebdm_initialize(HINSTANCE);
void _opensourcebdm_uninitialize(void);
void _opensourcebdm_debug(char usb_data[]);
#endif

#endif // _OSBDM_H_
