/*
    Opensource BDM  - interface DLL
    Based on TBDML interface DLL by
    Copyright (C) 2005  Daniel Malik

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <windows.h>
#include <stdio.h>
#include "Configure.h"
#include "log.h"
#include "version.h"

//#define TBDML_API  // Mark this file as exporting TBDML functiosn

#include "osbdm.h"
#include "common_usb.h"
#include "commands.h"
#include "Common.h"

/* define symbol "LOG" in "log.h" during compilation to produce a log file tbdml_dll.log */

static unsigned char usb_data[MAX_PACKET_SIZE+1]; // Extra +1 for debug byte
static int initialised = 0;
static HINSTANCE hDLLInst;

/* returns version of the DLL in BCD format */
OSBDM_API unsigned char _opensourcebdm_dll_version(void) {
	return (TBDML_DLL_VERSION);
}

/* initialises USB and returns number of devices found */
OSBDM_API unsigned char _opensourcebdm_init(void) {
	unsigned char i;
	bdm_usb_find_devices();	/* look for devices on all USB busses */
	i=bdm_usb_cnt();
	print("OSBDM_INIT: Usb initialised, found %d device(s)\r\n",i);
	return (i);					/* count the devices found and return the number */
}

/* opens a device with given number (0...) */
/* returns 0 on success and 1 on error */
OSBDM_API unsigned char _opensourcebdm_open(unsigned char device_no) {
int rc = 0;

   print("OSBDM_OPEN: Trying to open device #%d\r\n",device_no);
   rc = bdm_usb_open(device_no);
   print("OSBDM_OPEN: rc = #%d\r\n", rc);

   if (rc == 0)
      configureDialog(hDLLInst);

   return (rc);
}

/* closes currently open device */
OSBDM_API void _opensourcebdm_close(void) {
	print("OSBDM_CLOSE: Trying to close the device\r\n");
   usb_data[0]=2;  /* receive 2 byte */
   usb_data[1]=CMD_SET_TARGET;
   usb_data[2]=0xFF;
   bdm_usb_recv_ep0(usb_data);
	bdm_usb_close();
}

/* gets version of the interface (HW and SW) in BCD format */
OSBDM_API unsigned int  _opensourcebdm_get_version(void) {
   usb_data[0] = 3;  // receive up to 3 bytes
	usb_data[1] = CMD_GET_VER;
	bdm_usb_recv_ep0(usb_data);
	return ((*((U16 *)(usb_data+1))));
}

/* returns status of the last command (either the command number or CMD_FAILED or CMD_UNKNOWN */
OSBDM_API unsigned char _opensourcebdm_get_last_sts(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_GET_LAST_STATUS;
	bdm_usb_recv_ep0(usb_data);
	return ((usb_data[0]==CMD_FAILED)||(usb_data[0]==CMD_UNKNOWN));
}

/* sets target MCU type */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_set_target_type(target_type_e target_type) {
   
   configureBDM();

   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_SET_TARGET;
	usb_data[2] = target_type;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_SET_TARGET);
}

/* sets target derivative type */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_set_derivative_type(unsigned char derivative) {
   // ToDo _opensourcebdm_set_derivative_type
   print("_opensourcebdm_set_derivative_type(%d)\r\n", derivative);
// ToDO - CHECK!
   return 0;
}
/* measures BDM frequency of the target using the SYNC feature and connects to it */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_target_sync(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_CONNECT;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_CONNECT);
}

/* resets the target to normal or special mode */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_target_reset(target_mode_e target_mode) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_RESET;
	usb_data[2] = target_mode;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_RESET);
}

/* fills user supplied structure with actual state of BDM communication channel */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_bdm_sts(bdm_status_t *bdm_status) {
   usb_data[0] = 2;  // receive up to 2 bytes
	usb_data[1] = CMD_GET_STATUS;
	bdm_usb_recv_ep0(usb_data);
	if (usb_data[0]!=CMD_GET_STATUS)
	   return (1);
	bdm_status->ackn_state = (ackn_state_e)(usb_data[1]&0x01);
	bdm_status->reset_state = (reset_state_e)((usb_data[1]&0x02)>>1);
	bdm_status->connection_state = (connection_state_e)((usb_data[1]&0x18)>>3);
	return (0);
}

/* starts target execution from current PC address */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_target_go(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_GO1;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_GO1);
}

/* steps over a single target instruction */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_target_step(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_STEP1;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_STEP1);
}

/* brings the target into active background mode */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_target_halt(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_HALT;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_HALT);
}

/* returns crystal frequency of the target in MHz */
OSBDM_API float _opensourcebdm_get_speed(void) {
   usb_data[0] = 3;  // receive up to 3 bytes
	usb_data[1] = CMD_READ_SPEED1;
	bdm_usb_recv_ep0(usb_data);
	return (60.0 * 2 * 128 / (*(U16 *)(usb_data+1)));
}

/* sets the communication speed, parameter is crystal frequency in MHz */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_set_speed(float crystal_frequency) {
	/* the command parameter in this case is expected length of SYNC pulse expressed in 60MHz ticks */
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_SET_SPEED1;
	*(U16 *)(usb_data+2) = 60.0 * 2 * 128 / crystal_frequency;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_SET_SPEED1);
}

//===========================================================
//  RS08 Flash memory handling
//
OSBDM_API unsigned char _opensourcebdm_target_readyfor_datald(void){
   usb_data[0] = 2;  // receive up to 2 bytes
   usb_data[1] = TCMD_NEXT_DLDATA;
   if (bdm_usb_recv_ep0(usb_data) != 0)
      return 1;
// ToDo - review
   return ((usb_data[0] == TCMD_NEXT_DLDATA) &&
          (usb_data[1] != BDM_RC_FLASH_PROGRAMING_BUSY));
}

OSBDM_API void _opensourcebdm_mem_dlstart(void){
   usb_data[0] = 1;  // receive up to 2 bytes
   usb_data[1] = CMD_FLASH_DLSTART;
   bdm_usb_recv_ep0(usb_data);
}

OSBDM_API void _opensourcebdm_mem_dlend(void){
   usb_data[0] = 1;  // receive up to 2 bytes
   usb_data[1] = CMD_FLASH_DLEND;
   bdm_usb_recv_ep0(usb_data);
}

//=========================================================
// Memory access
//=========================================================

//=================================
//  CFv1 Memory read
//
//  Returns : TRUE  => success
//            FALSE => error of some kind (pretty useless API!)
//
BOOL OSBDM_API _opensourcebdm_read_mem(unsigned int addr,     unsigned int size,
                                       unsigned int memSpace, unsigned char *data) {
   unsigned int blockSize;
   unsigned int count = size;
   const unsigned int CF_MAX_DATA_SIZE = (MAX_PACKET_SIZE-1)&~1; 
   
   print("_opensourcebdm_read_mem(addr=0x%X,size=%d,space=%d,buff=0x%X?)\r\n", 
                                  addr, size, memSpace, data);
   while (count>0) {
      blockSize = count;
      if (blockSize > CF_MAX_DATA_SIZE)
         blockSize = CF_MAX_DATA_SIZE;
      usb_data[0] = blockSize+1;         // max # of bytes in USB response
      usb_data[1] = CMD_CF_READ_MEM;
      usb_data[2] = (unsigned char)addr;
      usb_data[3] = (unsigned char)(addr>>8);
      usb_data[4] = (unsigned char)(addr>>16);
      usb_data[5] = (unsigned char)blockSize;
// ToDo     usb_data[?] = (unsigned char)memSpace;
// ToDO - check error code
      bdm_usb_recv_ep0(usb_data);
      memcpy(data, usb_data+1, blockSize);
      
      data  += blockSize;   // update destination in buffer
      addr  += blockSize;   // update address
      count -= blockSize;   // update count
   }
   printDump(data-size, size);
   
   // ToDo - check this - error handling
   return TRUE;
}

//=================================
//  CFv1 Memory write
//
//  Returns : TRUE  => success
//            FALSE => error of some kind (pretty useless API!)
//
BOOL OSBDM_API _opensourcebdm_write_mem(unsigned int addr, unsigned int count, 
                                                unsigned int memSpace, unsigned char *data) {
   char tt[10];
   unsigned int blockSize;
   const unsigned int CF_MAX_DATA_SIZE = (MAX_PACKET_SIZE-6)&~1; 

   print("_opensourcebdm_write_mem(addr=0x%X,count=%d,space=%d,buff=0x%X?)\r\n",
                                   addr, count, memSpace, data);
   printDump(data, count);

   while (count>0) {
      blockSize = count;
      if (blockSize > CF_MAX_DATA_SIZE)
         blockSize = CF_MAX_DATA_SIZE;
      usb_data[0] = blockSize+5;         // # of bytes to send (SETUP+DATA)
      usb_data[1] = CMD_CF_WRITE_MEM;
      usb_data[2] = (U8)addr;
      usb_data[3] = (U8)(addr>>8);
      usb_data[4] = (U8)(addr>>16);
      usb_data[5] = (U8)blockSize;
      memcpy(usb_data+6, data, blockSize);
      // ToDO - error check
      bdm_usb_send_ep0(usb_data);
      
      // do handshake
      tt[0]=2;  /* receive 2 bytes */
      tt[1]=CMD_GET_LAST_STATUS;
      bdm_usb_recv_ep0(tt);
      
      data  += blockSize;   // update location in buffer
      addr  += blockSize;   // update address
      count -= blockSize;   // update count
   }
// ToDo - error handling
   return TRUE;
}

//===========================================================
/* reads one byte from memory */
OSBDM_API unsigned char _opensourcebdm_read_byte(unsigned int address) {
   usb_data[0] = 2;  // receive up to 2 bytes
	usb_data[1] = CMD_READ_8;
	*(U16 *)(usb_data+2) = address;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[1]);
}

/* Writes one byte to memory */
OSBDM_API void _opensourcebdm_write_byte(unsigned int address, unsigned char data) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_8;
	*(U16 *)(usb_data+2) = address;
	usb_data[4] = data;
	bdm_usb_recv_ep0(usb_data);
}

/* reads block from memory */
/* data is returned to the user supplied buffer */
OSBDM_API void _opensourcebdm_read_block(unsigned int addr, unsigned int count, unsigned char *data) {

   unsigned int blockSize;
   const unsigned int MAX_DATA_SIZE = (MAX_PACKET_SIZE-1)&~1; 

   while (count > 0) {
      blockSize = count;
      if (blockSize > MAX_DATA_SIZE)
         blockSize = MAX_DATA_SIZE;
      
      if ((count>MAX_DATA_SIZE)&&(addr&0x1))
         /* if address is odd and we are going to read more than one
          *  block make the first packet 1 byte short to align the address */
         blockSize--;

      usb_data[1]             = CMD_READ_BLOCK1;
      *((U16 *)(usb_data+2))  = addr;
      usb_data[4]             = blockSize;
      #ifdef FAST_BLOCK
         usb_data[0] = 4;
         tbdml_usb_send_ep2(usb_data);
         usb_data[0] = blockSize+1;
         tbdml_usb_recv_ep2(usb_data);
      #else
         usb_data[0] = blockSize+1;
         bdm_usb_recv_ep0(usb_data);
      #endif
      memcpy(data, usb_data+1, blockSize);
      
      data  += blockSize;   // update location in buffer
      addr  += blockSize;   // update address
      count -= blockSize;   // update count
   }
}

/* writes block to memory */
OSBDM_API void _opensourcebdm_write_block(unsigned int addr, unsigned int count, unsigned char *data) {

   unsigned int blockSize;
   const unsigned int MAX_DATA_SIZE = (MAX_PACKET_SIZE-5)&~1; 

   while (count > 0) {
      blockSize = count;
      if (blockSize > MAX_DATA_SIZE)
         blockSize = MAX_DATA_SIZE;
      
      if ((count>MAX_DATA_SIZE)&&(addr&0x1))
         /* if address is odd and we are going to read more than one
          *  block make the first packet 1 byte short to align the address */
         blockSize--;

      usb_data[0]             = blockSize+5;
      usb_data[1]             = CMD_WRITE_BLOCK1;
      *((U16 *)(usb_data+2))  = addr;
      usb_data[4]             = blockSize;
      memcpy(usb_data+5, data, blockSize);
      
      #ifdef FAST_BLOCK
         tbdml_usb_send_ep2(usb_data); /* Tx data */
         usb_data[0]=1;                /* read status */
         tbdml_usb_recv_ep2(usb_data);
      #else
         bdm_usb_send_ep0(usb_data);
         // do handshake
         usb_data[0]=1;  /* receive 1 bytes */
         usb_data[1]=CMD_GET_LAST_STATUS;
         bdm_usb_recv_ep0(usb_data);
      #endif
      
      data  += blockSize;   // update location in buffer
      addr  += blockSize;   // update address
      count -= blockSize;   // update count
   }
}

/* Reads one word from memory (address must be aligned) */
OSBDM_API unsigned int _opensourcebdm_read_word(unsigned int address) {
   usb_data[0] = 3;  // receive up to 3 bytes
	usb_data[1] = CMD_READ_16;
	*((U16 *)(usb_data+2)) = address;
	bdm_usb_recv_ep0(usb_data);
	return (*(U16 *)(usb_data+1));
}

/* writes one word to memory (address must be aligned) */
OSBDM_API void _opensourcebdm_write_word(unsigned int address, unsigned int data) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_16;
	*(U16 *)(usb_data+2) = address;
	*(U16 *)(usb_data+4) = data;
	bdm_usb_recv_ep0(usb_data);
}

/* reads one byte from the BDM area of memory */
/* returns byte read from memory */
OSBDM_API unsigned char _opensourcebdm_read_bd(unsigned int address) {
   usb_data[0] = 2;  // receive up to 2 bytes
	usb_data[1] = CMD_READ_BD;
	*((U16 *)(usb_data+2)) = address;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[1]);
}

/* writes one byte to the BDM area of memory */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_write_bd(unsigned int address, unsigned char data) {
   usb_data[0] = 2;  // receive up to 2 bytes
	usb_data[1] = CMD_WRITE_BD;
	*((U16 *)(usb_data+2)) = address;
	usb_data[4] = data;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_WRITE_BD);
}

//==========================================================
//  Register Access
//==========================================================

/* Read Coldfire data or address register */
OSBDM_API unsigned int  _opensourcebdm_read_reg(unsigned char regNo) {
   print("_opensourcebdm_read_reg(%d)\r\n", regNo);
   usb_data[0] = 5;  // receive up to 5 bytes
   usb_data[1] = CMD_CF_READ_REG;
   usb_data[2] = regNo;
   bdm_usb_recv_ep0(usb_data);
   return (*(U32 *)(usb_data+1));
}

/* Read Coldfire Control register */
OSBDM_API unsigned int  _opensourcebdm_read_creg(unsigned char regNo) {
   print("_opensourcebdm_read_creg(%d)\r\n", regNo);
   usb_data[0] = 5;  // receive up to 5 bytes
   usb_data[1] = CMD_CF_READ_CREG;
   usb_data[2] = regNo;
   bdm_usb_recv_ep0(usb_data);
   return (*(U32 *)(usb_data+1));
}

/* Read Coldfire Debug register */
OSBDM_API unsigned int  _opensourcebdm_read_dreg(unsigned char regNo) {
   print("_opensourcebdm_read_dreg(%d)\r\n", regNo);
   usb_data[0] = 5;  // receive up to 5 bytes
   usb_data[1] = CMD_CF_READ_DREG;
   usb_data[2] = regNo;
   bdm_usb_recv_ep0(usb_data);
   return (*(U32 *)(usb_data+1));
}

/* Read Coldfire MSB of XCSR register */
OSBDM_API unsigned int  _opensourcebdm_read_xcsr_byte(void) {
   print("_opensourcebdm_read_xcsr_byte()\r\n");
   usb_data[0] = 2;  // receive up to 2 bytes
   usb_data[1] = CMD_CF_READ_XCSR;
   bdm_usb_recv_ep0(usb_data);
   return (usb_data[1]);
}

/* Read Coldfire MSB of CSR2 register */
OSBDM_API unsigned int  _opensourcebdm_read_csr2_byte(void) {
   print("_opensourcebdm_read_csr2_byte()\r\n");
   usb_data[0] = 2;  // receive up to 2 bytes
  // ToDo
   //usb_data[1] = CMD_CF_READ_XCSR;
  // bdm_usb_recv_ep0(usb_data);
   return (usb_data[1]);
}

/* Read Coldfire MSB of CSR3 register */
OSBDM_API unsigned int  _opensourcebdm_read_csr3_byte(void) {
   print("_opensourcebdm_read_csr3_byte()\r\n");
   usb_data[0] = 2;  // receive up to 2 bytes
   // ToDo
   //usb_data[1] = CMD_CF_READ_XCSR;
   //bdm_usb_recv_ep0(usb_data);
   return (usb_data[1]);
}

OSBDM_API void  opensourcebdm_write_xcsr_byte(unsigned char value) {
	usb_data[0] = 1; // receive 1 bytes
	usb_data[1] = CMD_CF_WRITE_XCSR;
	usb_data[2] = value;
	bdm_usb_recv_ep0(usb_data);
}

OSBDM_API void  opensourcebdm_write_csr2_byte(unsigned char value) {
   usb_data[0] = 1; // receive 1 bytes
	usb_data[1] = CMD_CF_WRITE_CSR2;
	usb_data[2] = value;
	bdm_usb_recv_ep0(usb_data);
}

OSBDM_API void  opensourcebdm_write_csr3_byte(unsigned char value) {
   usb_data[0] = 1; // receive 1 bytes
	usb_data[1] = CMD_CF_WRITE_CSR3;
	usb_data[2] = value;
	bdm_usb_recv_ep0(usb_data);
}


OSBDM_API void _opensourcebdm_write_reg(unsigned char regNo, unsigned int value) {
   char tt[2];
   print("_opensourcebdm_write_reg(%d,%X)\r\n", regNo, value);
   if (regNo>15) // debugger tries to write to illegal registers !
      return;
   // ToDo - Review
   usb_data[0] = 6;  // send 6 bytes
   usb_data[1] = CMD_CF_WRITE_REG;
   usb_data[2] = regNo;
   *((U32 *)(usb_data+3)) = value;
   bdm_usb_send_ep0(usb_data);

   // do handshake
   tt[0] = 1;  /* receive 1 bytes */
   tt[1] = CMD_GET_LAST_STATUS;
   bdm_usb_recv_ep0(tt);
}

/* Write Coldfire Control register */
OSBDM_API void _opensourcebdm_write_creg(unsigned char regNo, unsigned int value) {
   char tt[20];
   print("_opensourcebdm_write_creg(%d,%X)\r\n", regNo, value);
   if (regNo>15)
      return;
   // ToDo - Review
   usb_data[0] = 6;  // send 6 bytes
   usb_data[1] = CMD_CF_WRITE_CREG;
   usb_data[2] = regNo;
   *((U32 *)(usb_data+3)) = value;
   bdm_usb_send_ep0(usb_data);

   // do handshake
   tt[0] = 1;  /* receive 1 bytes */
   tt[1] = CMD_GET_LAST_STATUS;
   bdm_usb_recv_ep0(tt);
}

/* Write Coldfire Debug register */
OSBDM_API void _opensourcebdm_write_dreg(unsigned char regNo, unsigned int value) {
   char tt[20];
   print("_opensourcebdm_write_dreg(%d,%X)\r\n", regNo, value);
   if (regNo>31)
      return;
   // ToDo - Review
   usb_data[0] = 6;  // send 6 bytes
   usb_data[1] = CMD_CF_WRITE_DREG;
   usb_data[2] = regNo;
   *((U32 *)(usb_data+3)) = value;
   bdm_usb_send_ep0(usb_data);

   // do handshake
   tt[0] = 1;  /* receive 1 bytes */
   tt[1] = CMD_GET_LAST_STATUS;
   bdm_usb_recv_ep0(tt);
}






/* reads contents of core registers */
/* returns 0 on success and non-zero on failure */
OSBDM_API unsigned char _opensourcebdm_read_regs(registers_t *registers) {
   usb_data[0] = 13;  // receive up to 13 bytes
	usb_data[1] = CMD_READ_REGS;
	bdm_usb_recv_ep0(usb_data);
   // Note: HCS08/RS08 structs are compatible
	registers->hcs08.pc  = *(U16 *)(usb_data+1);
	registers->hcs08.sp  = *(U16 *)(usb_data+3);
	registers->hcs08.hx  = *(U16 *)(usb_data+5);
	registers->hcs08.a   = usb_data[7];
	registers->hcs08.ccr = usb_data[8];
	return (usb_data[0]!=CMD_READ_REGS);
}

/* writes PC */
OSBDM_API void _opensourcebdm_write_reg_pc(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_PC;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes SP */
OSBDM_API void _opensourcebdm_write_reg_sp(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_SP;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes H:X (HCS08) or IX (HC12) */
OSBDM_API void _opensourcebdm_write_reg_x(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_X;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes IY (HC12 only) */
OSBDM_API void _opensourcebdm_write_reg_y(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_Y;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes B:A (HC12) or A (HCS08) */
OSBDM_API void _opensourcebdm_write_reg_d(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_D;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes CCR */
OSBDM_API void _opensourcebdm_write_reg_ccr(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_CCR;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

OSBDM_API unsigned char _opensourcebdm_read_status(void){
   usb_data[0] = 2;  // receive up to 2 bytes
   usb_data[1] = CMD_READ_STATUS;
   bdm_usb_recv_ep0(usb_data);
   return (usb_data[1]);
}

OSBDM_API void _opensourcebdm_write_control(unsigned char value){
   usb_data[0] = 1;  // receive up to 1 bytes
   usb_data[1] = CMD_WRITE_CONTROL;
   usb_data[2] = value;
   bdm_usb_recv_ep0(usb_data);
}

//================================================
//
OSBDM_API void _opensourcebdm_write_bkpt(unsigned int value){
   usb_data[0] = 1;  // receive up to 1 bytes
   usb_data[1] = CMD_WRITE_BKPT;
   *(U16 *)(usb_data+2) = (U16)value;
   bdm_usb_recv_ep0(usb_data);
}

OSBDM_API unsigned int _opensourcebdm_read_bkpt(void){
   usb_data[0] = 3;  // receive up to 3 bytes
   usb_data[1] = CMD_READ_BKPT;
   bdm_usb_recv_ep0(usb_data);
   return (*(U16*)(usb_data+1));
}

#if defined (COMMANDLINE)
void _opensourcebdm_debug(char *usb_data) {
   usb_data[0] = 7;  // receive up to 7 bytes
   usb_data[1] = CMD_DEBUG;
   bdm_usb_recv_ep0(usb_data);
}
#endif

#if defined(DLL)
static
#endif
void _opensourcebdm_initialize(HINSTANCE hDLLInst){
   
   openLogFile(NULL);
  
   bdm_usb_init();       /* initialise USB */
   
   initialised = 1;
}

#if defined(DLL)
static
#endif
void _opensourcebdm_uninitialize(void) {
   closeLogFile();
   initialised = 0;
}

#if defined(DLL)
BOOL OSBDM_API DllMain(HINSTANCE _hDLLInst, DWORD fdwReason, LPVOID lpvReserved) {
   hDLLInst = _hDLLInst;
   switch (fdwReason) {
      case DLL_PROCESS_ATTACH:
         _opensourcebdm_initialize(hDLLInst);
         break;
      case DLL_PROCESS_DETACH:
         _opensourcebdm_uninitialize();
         break;
      case DLL_THREAD_ATTACH:
         break;
      case DLL_THREAD_DETACH:
         break;
 }
 return TRUE;
}
#endif
