/*
    Turbo BDM Light - interface DLL
    Based on TBDML interface DLL by
    Copyright (C) 2005  Daniel Malik

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <windows.h>
#include <stdio.h>
#include "Configure.h"
#include "log.h"
#include "version.h"

//#define TBDML_API  // Mark this file as exporting TBDML functiosn

#include "tbdml.h"
#include "common_usb.h"
#include "commands.h"
#include "Common.h"

/* define symbol "LOG" in "log.h" during compilation to produce a log file tbdml_dll.log */

static unsigned char usb_data[MAX_PACKET_SIZE+1]; // Extra +1 for debug byte
static int           initialised = 0;
static HINSTANCE hDLLInst;

/* returns version of the DLL in BCD format */
unsigned char TBDML_API _tbdml_dll_version(void) {
	return(TBDML_DLL_VERSION);
}

/* initialises USB and returns number of devices found */
unsigned char TBDML_API _tbdml_init(void) {
	unsigned char i;
	bdm_usb_find_devices();	/* look for devices on all USB busses */
	i=bdm_usb_cnt();
	print("OSBDM_INIT: Usb initialised, found %d device(s)\r\n",i);
	return(i);					/* count the devices found and return the number */
}

/* opens a device with given number (0...) */
/* returns 0 on success and 1 on error */
unsigned char TBDML_API _tbdml_open(unsigned char device_no) {
int rc = 0;

	print("OSBDM_OPEN: Trying to open device #%d\r\n",device_no);
   rc = bdm_usb_open(device_no);
   print("OSBDM_OPEN: rc = #%d\r\n", rc);

   if (rc == 0)
      configureDialog(hDLLInst);

   return(rc);
}

/* closes currently open device */
void TBDML_API _tbdml_close(void) {
	print("OSBDM_CLOSE: Trying to close the device\r\n");
   usb_data[0]=2;  /* receive 2 byte */
   usb_data[1]=CMD_SET_TARGET;
   usb_data[2]=0xFF;
   bdm_usb_recv_ep0(usb_data);
	bdm_usb_close();
}

/* gets version of the interface (HW and SW) in BCD format */
unsigned int TBDML_API _tbdml_get_version(void) {
   usb_data[0] = 3;  // receive up to 3 bytes
	usb_data[1] = CMD_GET_VER;
	bdm_usb_recv_ep0(usb_data);
	return ((*((U16 *)(usb_data+1))));
}

/* returns status of the last command (either the command number or CMD_FAILED or CMD_UNKNOWN */
unsigned char TBDML_API _tbdml_get_last_sts(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_GET_LAST_STATUS;
	bdm_usb_recv_ep0(usb_data);
	return ((usb_data[0]==CMD_FAILED)||(usb_data[0]==CMD_UNKNOWN));
}

/* sets target MCU type */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_set_target_type(target_type_e target_type) {
   
   configureBDM();

   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_SET_TARGET;
	usb_data[2]=target_type;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[0]!=CMD_SET_TARGET);
}

/* measures BDM frequency of the target using the SYNC feature and connects to it */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_target_sync(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_CONNECT;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[0]!=CMD_CONNECT);
}

/* resets the target to normal or special mode */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_target_reset(target_mode_e target_mode) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_RESET;
	usb_data[2]=target_mode;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[0]!=CMD_RESET);
}

/* fills user supplied structure with actual state of BDM communication channel */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_bdm_sts(bdm_status_t *bdm_status) {
	usb_data[0]=2;	 /* receive 2 bytes */
	usb_data[1]=CMD_GET_STATUS;
	bdm_usb_recv_ep0(usb_data);
	if (usb_data[0]!=CMD_GET_STATUS)
	   return(1);
	bdm_status->ackn_state = (ackn_state_e)(usb_data[1]&0x01);
	bdm_status->reset_state = (reset_state_e)((usb_data[1]&0x02)>>1);
	bdm_status->connection_state = (connection_state_e)((usb_data[1]&0x18)>>3);
	return(0);
}

/* starts target execution from current PC address */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_target_go(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_GO1;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[0]!=CMD_GO1);
}

/* steps over a single target instruction */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_target_step(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_STEP1;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[0]!=CMD_STEP1);
}

/* brings the target into active background mode */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_target_halt(void) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_HALT;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[0]!=CMD_HALT);
}

/* returns crystal frequency of the target in MHz */
float TBDML_API _tbdml_get_speed(void) {
	usb_data[0]=3;	 /* receive 3 bytes */
	usb_data[1]=CMD_READ_SPEED1;
	bdm_usb_recv_ep0(usb_data);
	return(60.0 * 2 * 128 / (*((unsigned int *)(usb_data+1))&0xffff));
}

/* sets the communication speed, parameter is crystal frequency in MHz */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_set_speed(float crystal_frequency) {
	/* the command parameter in this case is expected length of SYNC pulse expressed in 60MHz ticks */
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_SET_SPEED1;
	*(U16 *)(usb_data+2) = 60.0 * 2 * 128 / crystal_frequency;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[0]!=CMD_SET_SPEED1);
}

//===========================================================
/* reads one byte from memory */
unsigned char TBDML_API _tbdml_read_byte(unsigned int address) {
	usb_data[0]=2;	 /* receive 2 bytes */
	usb_data[1]=CMD_READ_8;
	*(U16 *)(usb_data+2) = address;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[1]);
}

/* writes one byte to memory */
void TBDML_API _tbdml_write_byte(unsigned int address, unsigned char data) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1]=CMD_WRITE_8;
	*(U16 *)(usb_data+2) = address;
	usb_data[4] = data;
	bdm_usb_recv_ep0(usb_data);
}

/* reads block from memory */
/* data is returned to the user supplied buffer */
void TBDML_API _tbdml_read_block(unsigned int addr, unsigned int count, unsigned char *data) {

   unsigned int blockSize;
   const unsigned int MAX_DATA_SIZE = (MAX_PACKET_SIZE-1)&~1; 

   while (count > 0) {
      blockSize = count;
      if (blockSize > MAX_DATA_SIZE)
         blockSize = MAX_DATA_SIZE;
      
      if ((count>MAX_DATA_SIZE)&&(addr&0x1))
         /* if address is odd and we are going to read more than one
          *  block make the first packet 1 byte short to align the address */
         blockSize--;

      usb_data[1]             = CMD_READ_BLOCK1;
      *((U16 *)(usb_data+2))  = addr;
      usb_data[4]             = blockSize;
      #ifdef FAST_BLOCK
         usb_data[0] = 4;
         tbdml_usb_send_ep2(usb_data);
         usb_data[0] = blockSize+1;
         tbdml_usb_recv_ep2(usb_data);
      #else
         usb_data[0] = blockSize+1;
         bdm_usb_recv_ep0(usb_data);
      #endif
      memcpy(data, usb_data+1, blockSize);
      
      data  += blockSize;   // update location in buffer
      addr  += blockSize;   // update address
      count -= blockSize;   // update count
   }
}

/* writes block to memory */
void TBDML_API _tbdml_write_block(unsigned int addr, unsigned int count, unsigned char *data) {

   unsigned int blockSize;
   const unsigned int MAX_DATA_SIZE = (MAX_PACKET_SIZE-5)&~1; 

   while (count > 0) {
      blockSize = count;
      if (blockSize > MAX_DATA_SIZE)
         blockSize = MAX_DATA_SIZE;
      
      if ((count>MAX_DATA_SIZE)&&(addr&0x1))
         /* if address is odd and we are going to read more than one
          *  block make the first packet 1 byte short to align the address */
         blockSize--;

      usb_data[0]             = blockSize+5;
      usb_data[1]             = CMD_WRITE_BLOCK1;
      *((U16 *)(usb_data+2))  = addr;
      usb_data[4]             = blockSize;
      memcpy(usb_data+5, data, blockSize);
      
      #ifdef FAST_BLOCK
         tbdml_usb_send_ep2(usb_data); /* Tx data */
         usb_data[0]=1;                /* read status */
         tbdml_usb_recv_ep2(usb_data);
      #else
         bdm_usb_send_ep0(usb_data);
         // do handshake
         usb_data[0]=1;  /* receive 1 bytes */
         usb_data[1]=CMD_GET_LAST_STATUS;
         bdm_usb_recv_ep0(usb_data);
      #endif
      
      data  += blockSize;   // update location in buffer
      addr  += blockSize;   // update address
      count -= blockSize;   // update count
   }
}

/* reads one word from memory (address must be aligned) */
unsigned int TBDML_API _tbdml_read_word(unsigned int address) {
	usb_data[0]=3;	 /* receive 3 bytes */
	usb_data[1]=CMD_READ_16;
	*((U16 *)(usb_data+2)) = address;
	bdm_usb_recv_ep0(usb_data);
	return (*(U16 *)(usb_data+1));
}

/* writes one word to memory (address must be aligned) */
void TBDML_API _tbdml_write_word(unsigned int address, unsigned int data) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_16;
	*(U16 *)(usb_data+2) = address;
	*(U16 *)(usb_data+4) = data;
	bdm_usb_recv_ep0(usb_data);
}

/* reads one byte from the BDM area of memory */
/* returns byte read from memory */
unsigned char TBDML_API _tbdml_read_bd(unsigned int address) {
	usb_data[0]=2;	 /* receive 2 bytes */
	usb_data[1]=CMD_READ_BD;
	*((U16 *)(usb_data+2)) = address;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[1]);
}

/* writes one byte to the BDM area of memory */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_write_bd(unsigned int address, unsigned char data) {
	usb_data[0]=2;	 /* receive 2 byte */
	usb_data[1]=CMD_WRITE_BD;
	*((U16 *)(usb_data+2)) = address;
	usb_data[4] = data;
	bdm_usb_recv_ep0(usb_data);
	return(usb_data[0]!=CMD_WRITE_BD);
}

/* reads contents of core registers */
/* returns 0 on success and non-zero on failure */
unsigned char TBDML_API _tbdml_read_regs(registers_t *registers) {
	usb_data[0]=13;	 /* return 13 bytes */
	usb_data[1]=CMD_READ_REGS;
	bdm_usb_recv_ep0(usb_data);
	registers->hc12.pc = (*((unsigned int*)(usb_data+1)))&0xffff;
	registers->hc12.sp = (*((unsigned int*)(usb_data+3)))&0xffff;
	registers->hc12.ix = (*((unsigned int*)(usb_data+5)))&0xffff;
	registers->hc12.iy = (*((unsigned int*)(usb_data+7)))&0xffff;
	registers->hc12.d  = (*((unsigned int*)(usb_data+9)))&0xffff;
	registers->hc12.ccr= (*((unsigned char*)(usb_data+11)));
	return(usb_data[0]!=CMD_READ_REGS);
}

/* writes PC */
void TBDML_API _tbdml_write_reg_pc(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_PC;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes SP */
void TBDML_API _tbdml_write_reg_sp(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_SP;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes H:X (HCS08) or IX (HC12) */
void TBDML_API _tbdml_write_reg_x(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_X;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes IY (HC12 only) */
void TBDML_API _tbdml_write_reg_y(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_Y;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes B:A (HC12) or A (HCS08) */
void TBDML_API _tbdml_write_reg_d(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_D;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}

/* writes CCR */
void TBDML_API _tbdml_write_reg_ccr(unsigned int value) {
   usb_data[0] = 1;  // receive up to 1 bytes
	usb_data[1] = CMD_WRITE_REG_CCR;
	*(U16 *)(usb_data+2) = value;
	bdm_usb_recv_ep0(usb_data);
}


unsigned char TBDML_API _tbdml_read_status(void){
   usb_data[0]=2;  /* send 2 bytes */
   usb_data[1]=CMD_READ_STATUS;
   bdm_usb_recv_ep0(usb_data);
   return(usb_data[1]);
}

void TBDML_API _tbdml_write_control(unsigned char value){
   usb_data[0] = 1;  // receive up to 1 bytes
   usb_data[1] = CMD_WRITE_CONTROL;
   usb_data[2] = value;
   bdm_usb_recv_ep0(usb_data);
}

//================================================
//
void TBDML_API _tbdml_write_bkpt(unsigned int value){
   usb_data[0] = 1;  // receive up to 1 bytes
   usb_data[1] = CMD_WRITE_BKPT;
   *(U16 *)(usb_data+2) = (U16)value;
   bdm_usb_recv_ep0(usb_data);
}

unsigned int TBDML_API _tbdml_read_bkpt(void){
   usb_data[0] = 3;  // receive up to 3 bytes
   usb_data[1] = CMD_READ_BKPT;
   bdm_usb_recv_ep0(usb_data);
   return (*(U16*)(usb_data+1));
}

static
void _tbdml_initialize(HINSTANCE hDLLInst){
   
   openLogFile(NULL);

   bdm_usb_init();       /* initialise USB */
   
   initialised = 1;
}

static
void _tbdml_uninitialize(void) {
   closeLogFile();
   initialised = 0;
}

#if defined(DLL)
BOOL TBDML_API DllMain(HINSTANCE _hDLLInst, DWORD fdwReason, LPVOID lpvReserved) {
   hDLLInst = _hDLLInst;
   switch (fdwReason) {
      case DLL_PROCESS_ATTACH:
         _tbdml_initialize(hDLLInst);
         break;
      case DLL_PROCESS_DETACH:
         _tbdml_uninitialize();
         break;
      case DLL_THREAD_ATTACH:
         break;
      case DLL_THREAD_DETACH:
         break;
 }
 return TRUE;
}
#endif
