     ; Include derivative-specific definitions
     INCLUDE 'derivative.inc'

     ABSENTRY   Start

PAGEWINDOW  equ  $00C0

;********************************************************************************
; Fast RAM ($0 - $D)
                  org   $0
;            
; Programming parameters (values given are dummies for testing)
;
Destination    equ   $3F11
             
SourceAddr        dc.b  $08                             ; Address in Zero Page (Buffer) for source
Buffer            dc.b  $88,$77,$66,$55,$44,$33,$22,$11 ; Buffer for data to program
DestinationAddr   dc.b  MAP_ADDR_6(Destination)         ; Address in Zero Page (Paging Window) for destination
DestinationPage   dc.b  HIGH_6_13(Destination)          ; Page for destination
;
; Timing constants for Flash Programming 
;          
; Assumes clock freq =  32kHz * 512 / 2 ~ 8.2 MHz  => 122 ns
;
Time5us          dc.b  2   ; PGM/MASS to HVEN setup time >5 us ~41 cycles
                           ; HVEN hold time for PGM
Time10us         dc.b  4   ; HVEN to program setup time >10 us ~82 cycles
Time40us         dc.b  16  ; Byte program time          >40 us ~328 cycles 
  
;********************************************************************************
; Flash programming code in Z-Page RAM
;            
   org   $20

Start:
   mov   #HIGH_6_13(FLCR),PAGESEL   ; 4 cy
   bset  FLCR_PGM,MAP_ADDR_6(FLCR)  ; 5 cy     ; Enable Flash programming

   lda   DestinationPage            ; 3 cy     ; Set window to page to program
   sta   PAGESEL                    ; 3 cy  
   lda   DestinationAddr            ; 3 cy     ; Dummy write to Flash block
   tax                              ; 2 cy
   sta   ,X                         ; 2 cy

   lda   Time5us                    ; 3 cy     ; Wait Tnvs 
   dbnza *                          ; 4*N cy      

   mov   #HIGH_6_13(FLCR),PAGESEL   ; 4 cy
   bset  FLCR_HVEN,MAP_ADDR_6(FLCR) ; 5 cy     ; Enable Flash high voltage

   lda   Time10us                   ; 3 cy     ; Wait Tpgs
   dbnza *                          ; 4*N cy           

   lda   DestinationPage            ; 3 cy     ; Set window to page to program
   sta   PAGESEL                    ; 3 cy

Loop:
   lda   SourceAddr                 ; 3 cy     ; Copy byte to flash (program)
   tax                              ; 2 cy
   lda   ,X                         ; 3 cy
   ldx   DestinationAddr            ; 5 cy
   sta   ,X                         ; 2 cy 

   lda   Time40us                   ; 3 cy     ; Wait Tprog
   dbnza *                          ; 4*N cy
   
   inc   DestinationAddr            ; 4 cy     ; Update ptrs/counter
   dbnz  SourceAddr,Loop            ; 7 cy     ; Finished? - no, loop
   
   mov   #HIGH_6_13(FLCR),PAGESEL   ; 4 cy
   bclr  FLCR_PGM,MAP_ADDR_6(FLCR)  ; 5 cy     ; Disable Flash programming

   lda   Time5us                    ; 3 cy     ; Wait Tnvh
   dbnza *                          ; 4*N cy     

   bclr  FLCR_HVEN,MAP_ADDR_6(FLCR) ; 5 cy     ; Disable High voltage
   
   bgnd                             ; 5+ cy    ; Suspend
   
   org   $3FFD
   jmp   $0020