/*
   Open Source BDM - processing of commands received over USB
   *** Prominent Notice-This software was modified from TBDML software - 12/05 ***

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*   
Change Hisory
+====================================================================
| 11 Apr 2008 | Added reconnect code (SYNC on READ_STATUS)     - pgo
| 11 Apr 2008 | Added option code                              - pgo
| 23 Mar 2008 | Changed return code to use BDM_RC_... codes    - pgo
|  3 Mar 2008 | Started changes for JM60 - lots                - pgo
+====================================================================
*/
#include <string.h>
#include "derivative.h"
#include "Common.h"
#include "Configure.h"
#include "bdm.h"
#include "commands.h"
#include "CmdProcessing.h"
#include "usb.h"
#include "led.h"
#include "icp.h"
#include "TargetDefines.h"
#include "main.h"

#pragma MESSAGE DISABLE C4000 // Disable warnings about allways true
#pragma MESSAGE DISABLE C5900 // Disable warnings about result is zero

/* buffer for Rx and Tx of commands & results */
/* data is always received starting at commandBuffer+1 and transmitted starting at commandBuffer+0 */
/* this is to make sure that status of the last command is preserved when receving the next command from the host */

#pragma DATA_SEG __SHORT_SEG Z_PAGE
U8 commandBuffer[MAX_PACKET_SIZE];   
#pragma DATA_SEG DEFAULT

static U8 makeStatusByte(void) {
U8 status = 0;

   if (bdm_status.ackn==ACKN) 
      status |= 0x01;                /* target supports ACKN and the feature is enabled */
#if (CAPABILITY&CAP_RESET)
   if (bdm_status.reset==RESET_DETECTED) {
      status |= 0x02;                /* the target was resently reset externally */
      if (RESET_IN==1) 
         bdm_status.reset=NO_RESET_ACTIVITY;  /* clear the flag if reset pin has gone high already */
   }
#endif
   if (bdm_status.speed==SPEED_USER_SUPPLIED) /* do not blame me if it does not work... */
      status |= 0x18; 
   else if (bdm_status.speed==SYNC_SUPPORTED) /* SYNC feature used to detect speed */ 
      status |= 0x08; 
   else if (bdm_status.speed==SPEED_GUESSED)  /* speed guessed by trial & error */ 
      status |=0x10; 
      
   return status;
}

//===============================================
//  Compatibility layer for original TBDML/OSBDM - incomplete!
//
// Returns : 
U8 compatibleCommandExec(void){
U8 size;
U8 command = commandBuffer[1]; // save command as it is used to indicate OK result

   size = commandExec(); // Pass to actual execute routine
   
   if (commandBuffer[1] == 0) // If no error return original command byte to indicate success
      commandBuffer[1] = command;
      
   return size; 
}

//====================================================
//  Debug Layer
//
// Adds extra byte to buffer containing the current
// status byte.
// 
U8 debugCommandExec(void){
U8 size;

   size = commandExec(); // Pass to actual execute routine

   commandBuffer[size++] = makeStatusByte();
   
   return size; 
}
//==========================================================================
//  Processes all commands received over USB
//  The command is expected to be in commandBuffer[1]
//  Returns number of bytes left in the buffer to be sent back as response
U8 commandExec(void) {
U8 rc;

#ifdef DEBUG_PIN
   DEBUG_PIN_DDR = 1;
   DEBUG_PIN = 1;
   DEBUG_PIN = 0;
#endif   
   led_state = LED_BLINK;     /* blink the LED to indicate a command */

   if (commandBuffer[1] == CMD_GET_LAST_STATUS) {
      // Return status from last command received
      return(1);
   }

   if (commandBuffer[0] == BDM_RC_BUSY) {
      // Don't accept further commands until last one has completed
      // Problems here with async thread!!!!!
      commandBuffer[0] = BDM_RC_OVERRUN;
      return(1);
   }

   commandBuffer[0] = BDM_RC_OK;    // Assume the command will execute OK
   
   switch (commandBuffer[1]) {
#if (DEBUG&DEBUG_COMMANDS)
      // Various debugging & testing commands
      case CMD_DEBUG:
         switch (commandBuffer[2]) {
            case BDM_DBG_ACKN: // try the ACKN feature
               bdm_ackn_init();  
               commandBuffer[1] = makeStatusByte(); // return the status byte
               return(2);    
            case BDM_DBG_SYNC: // try the sync feature
               rc = bdm_sync_meas();  
               if (rc != 0) {
                  commandBuffer[0] = rc;
                  return (1);
               }
               commandBuffer[1] = makeStatusByte(); // return the status byte
               return(2);    
            case BDM_DBG_TESTPORT: // Check port I/O timing - hangs USB interface
               bdm_checkWaitTiming();
               bdm_checkTiming();
               return(1);
            case BDM_DBG_USBDISCONNECT: // Something to try - hangs USB interface
               PTE_PTE3    = '1';
               PTE_PTE4    = '1';
               UADDR_USBEN = 0;
               (*(void(** far)(void)) 0xFFFE)();
               return(1);
            case BDM_DBG_VPP_OFF: // Programming voltage off
               VPP_OFF();
               return(1);
            case BDM_DBG_VPP_ON: // Programming voltage on
               VPP_ON();
               return(1);
            case BDM_DBG_VDD_OFF: // Target Vdd voltage off
               VDD_OFF();
               return(1);
            case BDM_DBG_VDD3_ON: // Target Vdd voltage on
               VDD3_ON();
               return(1);
            case BDM_DBG_VDD5_ON: // Target Vdd voltage on
               VDD5_ON();
               return(1);
            case BDM_DBG_FLASH12V_OFF: // 12V charge pump off
               FLASH12V_OFF();
               return(1);
            case BDM_DBG_FLASH12V_ON: // 12V charge pump on
               FLASH12V_ON();
               return(1);
            case BDM_DBG_CYCLE_POWER: // Cycle power to target
               commandBuffer[0] = bdm_cycle_Target_Vdd();
               return(1);
            case BDM_DBG_MEASURE_VDD: // Measure Target Vdd  
               (*(U16*)(commandBuffer+2)) = bdm_Target_Vdd_measure(); // return the value
               commandBuffer[1] = commandBuffer[3];
               return(3);    
#if (DEBUG&STACK_DEBUG)
            case BDM_DBG_STACKSIZE: // Measure stack size
               {
               extern char far __SEG_START_SSTACK[];  // Bottom of stack space
               extern char far __SEG_END_SSTACK[];    // Top of stack space

               asm {
                  ldhx @__SEG_START_SSTACK // Bottom of stack space
               loop:
                  incx
                  tst   0,x     // Find 1st used (non-zero) byte
                  beq   loop
                  
                  // Calculate size of stack in use
                  pshx
                  pshh
                  lda   @__SEG_END_SSTACK
                  sub   2,sp
                  sta   commandBuffer:1
                  lda   @__SEG_END_SSTACK:MSB
                  sbc   1,sp
                  sta   commandBuffer:2
                  ais   #2
                  }
               return(3);
               }
#endif
            default:
               commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
               return(1);    
         }
#endif

      /* set/clear various options */
      case CMD_OPTION: {
         U8 option = commandBuffer[2];
         U8 value  = commandBuffer[3];
         switch (option) {
            case BDM_GET_CAPABILITIES : // Gets capability vector for hardware
               commandBuffer[1] = CAPABILITY;
               commandBuffer[2] = 0;
               return (3);
               break;
            case BDM_OPT_SET_TARGET_VDD :       // Set target Vdd (off, 3V3 or 5V)
               if (value > BDM_TARGET_VDD_5V) {
                  commandBuffer[0] = BDM_RC_ILLEGAL_PARAMS;
                  return (1);
               }
               bdm_option.targetVdd = value;
               break;
            case BDM_OPT_CYCLE_TARGET_VDD_ON_RESET : // Enable/disable Cycling Target Vdd if connection problems
               bdm_option.cycleVddOnReset = (value != 0);
               break;
            case BDM_OPT_CYCLE_TARGET_VDD_ON_CONNECT : // Enable/disable Cycling Target Vdd if connection problems
               bdm_option.cycleVddOnConnect = (value != 0);
               break;
            case BDM_OPT_AUTOMATIC_RECONNECT :  // Enable/disable automatic reconnect
               bdm_option.autoReconnect = (value != 0);
               break;
            case BDM_OPT_GUESS_SPEED :          // Enable/disable guessing speed for target w/o ACKN feature
               bdm_option.guessSpeed = (value != 0);
               break;
            case BDM_OPT_LEAVE_TARGET_POWERED : // Enable/disable leaving target power on exit
               bdm_option.leaveTargetPowered = (value != 0);
               break;
            case BDM_OPT_USE_ALT_BDM_CLOCK : // Enable/disable using alternative BDM clock source in target (HCS08)
               bdm_option.useAltBDMClock = (value != 0);
               break;
            case BDM_OPT_USE_RESET_SIGNAL :  // Enable/disable use of RESET signal on BDM interface
               bdm_option.useResetSignal = (value != 0);
               break;
            default :
               commandBuffer[0] = BDM_RC_ILLEGAL_PARAMS;
               return (1);
         }
         return (1);
      }

      /* get HW & SW version */
      case CMD_GET_VER:
         *((U16 *)(commandBuffer+1)) = VERSION;
         *((U16 *)(commandBuffer+3)) = ICPVersion;
         return(5);                              /* return cmd + 4 bytes of version */
      /* set target type */
      case CMD_SET_TARGET:
         commandBuffer[0] = bdm_init(commandBuffer[2]);
         return (1);
      /* try to connect to the target */
      case CMD_CONNECT:
         commandBuffer[0] = bdm_connect();
         return(1);
      /* request ICP on next reset */
      case CMD_SET_BOOT:  /* request bootloader action on next power-up */
         if (strncmp(&commandBuffer[2],"BOOT",sizeof("BOOT")-1) == 0) {
            forceICPOnReset();    /* program the flash to force ICP on reboot */    
            return(1);
         }
         commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
         return(1);
      /* reset Target */
      case CMD_RESET:
         commandBuffer[0] = bdm_target_reset(commandBuffer[2]);
         return(1);
      /* return status of BDM communication */
      case CMD_GET_STATUS:
         commandBuffer[1] = makeStatusByte(); // Intels LSB
         return(2);
      /* read BDM byte */
      case CMD_READ_BD:
         BDM12_CMD_BDREADB((U16)256*commandBuffer[3]+commandBuffer[2],commandBuffer+1);
         return(2);
      /* write BDM byte */
      case CMD_WRITE_BD:
         BDM12_CMD_BDWRITEB((U16)256*commandBuffer[3]+commandBuffer[2],commandBuffer[4]);
         return(1);
      case CMD_CF_READ_XCSR:
#if 0
      BDMCF_CMD_READ_XCSR(commandBuffer+1);
         return(2);
#endif
      case CMD_READ_STATUS: {
         rc = 0;
         if (bdm_option.autoReconnect &&           // Auto re-connect enabled &
             (bdm_status.speed==SYNC_SUPPORTED)) { // Target supports Sync
            rc = bdm_connect(); // Make sure of connection
            }
         if (rc == 0)
            rc = bdm_readBDMStatus(commandBuffer+1);
         if (rc != 0) {
            commandBuffer[0] = rc;
            return 1;
         }
         return(2);
         }
      case CMD_WRITE_CONTROL:
         if (bdm_option.useAltBDMClock)
            rc = bdm_writeBDMControl(commandBuffer[2]&~HC08_BDCSCR_CLKSW); // Force Alt clock selection
         else
            rc = bdm_writeBDMControl(commandBuffer[2]);
         if (rc != 0)
            commandBuffer[0] = rc;
         return(1);
      /* stop the target */
      case CMD_HALT:
         bdm_halt();
         return(1);
      /* */
      case CMD_MASS_ERASE:
         bdmRS08_mass_erase();
         return(1) ;
      /* set comm speed to user supplied value */
      case CMD_SET_SPEED1: {
         U8  rc;
         U16 sync_length = (U16)((commandBuffer[3]<<8)+commandBuffer[2]); /* save the speed */
         
         rc = bdmHC12_confirm_speed(sync_length);
         
         if (rc == 0) { // OK so far
             bdm_ackn_init();               // Try ACKN feature
             rc = bdm_enableBDM();          // Try to enable BDM
         }
         if (rc != 0) { // Failed
            commandBuffer[0] = rc;      /* no Rx or Tx routine for this speed or enable BDM failed */
            return(2);
         }
         bdm_status.speed = SPEED_USER_SUPPLIED; /* user told us */
         return(1);
      }
      /* Read current speed */
      case CMD_READ_SPEED1:
         commandBuffer[2] = (bdm_status.sync_length)>>8;   /* store speed in little endian */
         commandBuffer[1] = (bdm_status.sync_length)&0xff;
         return(3);
      /* Start code execution */
      case CMD_GO1:
         //DEBUG_PIN = 1;
         bdm_go();
         return(1);

      /////////////////////////////////////////
      /* Step over 1 instruction */
      case CMD_STEP1:
         bdm_step();
         return(1);

      /* read byte */
      case CMD_READ_8:
         commandBuffer[1] = 0xAA; // Dummy data value to return on fail
         switch (bdm_status.target_type) {
            case HC12:
               BDM12_CMD_READB((U16)256*commandBuffer[3]+commandBuffer[2],commandBuffer+1);
               return(2);
            case HCS08:
            case RS08:
               BDM08_CMD_READB((U16)((commandBuffer[3]<<8)+commandBuffer[2]), commandBuffer+1);
               return(2);
            }
         break;
      case CMD_READ_16:
         commandBuffer[1] = 0xAA; // Dummy data values to return on fail
         BDM12_CMD_READW((U16)256*commandBuffer[3]+commandBuffer[2],commandBuffer+2);
         commandBuffer[1] = commandBuffer[3];  /* swap bytes around for little endian */
         return(3);

      /* read registers */
      case CMD_READ_REGS:
         switch (bdm_status.target_type) {
            case HC12:
               BDM12_CMD_READ_PC(commandBuffer+2);
               commandBuffer[1] = commandBuffer[3];  /* swap bytes around for little endian */
               BDM12_CMD_READ_SP(commandBuffer+4);
               commandBuffer[3] = commandBuffer[5];  /* swap bytes around for little endian */
               BDM12_CMD_READ_X(commandBuffer+6);
               commandBuffer[5] = commandBuffer[7];  /* swap bytes around for little endian */
               BDM12_CMD_READ_Y(commandBuffer+8);
               commandBuffer[7] = commandBuffer[9];  /* swap bytes around for little endian */
               BDM12_CMD_READ_D(commandBuffer+10);
               commandBuffer[9] = commandBuffer[11]; /* swap bytes around for little endian */
               BDM12_CMD_BDREADW(HC12_BDMCCR,commandBuffer+11);  /* CCR is in little endian format already */
               commandBuffer[12] = 0;
               return(13);
            case HCS08:
               BDM08_CMD_READ_PC(commandBuffer+2);
               commandBuffer[1] = commandBuffer[3];  /* swap bytes around for little endian */
               BDM08_CMD_READ_SP(commandBuffer+4);	 //S08
               commandBuffer[3] = commandBuffer[5];  /* swap bytes around for little endian */
               BDM08_CMD_READ_HX(commandBuffer+6);
               commandBuffer[5] = commandBuffer[7];  /* swap bytes around for little endian */
               BDM08_CMD_READ_A(commandBuffer+7);
               BDM08_CMD_READ_CCR(commandBuffer+8);					
               return(9);
            case RS08:
               BDM08_CMD_READ_PC(commandBuffer+2);    // RS08 Read CCR+PC
               commandBuffer[1] = commandBuffer[3];  // swap bytes around for little endian
               BDM08_CMD_READ_SP(commandBuffer+4);	 // RS08 Read Shadow PC
               commandBuffer[3] = commandBuffer[5];  // swap bytes around for little endian
               commandBuffer[5] = 0;                  // RS08 doesn't have Read HX
               commandBuffer[6] = 0;
               BDM08_CMD_READ_A(commandBuffer+7);     // RS08 Read A
               commandBuffer[8] = 0;                  // RS08 doesn't have Read CCR
               return(9);
            }
         break;
      // Read CFV1 General register
      case CMD_CF_READ_REG:
         BDMCF_CMD_READ_REGe(commandBuffer[2],commandBuffer+1);
         return(5);
      // Read CFV1 Debug register
      case CMD_CF_READ_DREG:
         BDMCF_CMD_READ_DREGe(commandBuffer[2],commandBuffer+1);
         return(5);
      // Read CFV1 Control register
      case CMD_CF_READ_CREG:
         BDMCF_CMD_READ_CREGe(commandBuffer[2],commandBuffer+1);
         return(5);
         
      // Write CFV1 General register
      case CMD_CF_WRITE_REG:
         BDMCF_CMD_WRITE_REGe(commandBuffer[2],(*(U32 *)(commandBuffer+3)));
         return(1);        
      // Write CFV1 Debug register
      case CMD_CF_WRITE_DREG:
         BDMCF_CMD_WRITE_DREGe(commandBuffer[2],(*(U32 *)(commandBuffer+3)));
         return(1);        
      // Write CFV1 Control register
      case CMD_CF_WRITE_CREG:
         BDMCF_CMD_WRITE_CREGe(commandBuffer[2],(*(U32 *)(commandBuffer+3)));
         return(1);        

      // Write CFV1 Memory
      case CMD_CF_WRITE_MEM: {
         U8 count      = commandBuffer[5];
         U8 *data_ptr  = commandBuffer+6;
         U32 addr;
         asm {
            clr  addr:0
            lda  commandBuffer:4
            sta  addr:1
            lda  commandBuffer:3
            sta  addr:2
            lda  commandBuffer:2
            sta  addr:3
         } 
#if 1
         while (count > 0) {
            // This is sub-optimal but MUCH smaller!
            // Do at least 1 byte (to set address) and any odd bytes
            BDMCF_CMD_WRITE_MEM_B(addr,*data_ptr);
            data_ptr++;
            addr++;
            count--;
            while (((addr&0x03) == 0) && (count >=4)) { 
               // Longword boundary & a longword remains to be written
               BDMCF_CMD_FILL_MEM_L(*(U32 *)data_ptr);
               data_ptr += 4;
               addr     += 4;
               count    -= 4;
            }
         }
#else
         // (0,-)(1,B)(2,B/B)(2,W)(3,B/W)(3,W/B)(4,B/W/B)(4,W/B/B)(4,L)
         if (((addr & 1) && (count >= 1)) || (count == 1)) {
            BDMCF_CMD_WRITE_MEM_B(addr,*data_ptr); // odd byte
            data_ptr++;
            addr++;
            count--;
         }
         if (((addr & 2) && (count >= 2)) || (count == 2) || (count == 3))  {
            BDMCF_CMD_WRITE_MEM_W(addr,*data_ptr); // odd word
            data_ptr += 2;
            addr     += 2;
            count    -= 2;
         }
         if (count >= 4) {
            BDMCF_CMD_WRITE_MEM_L(addr,*data_ptr); // longword
            data_ptr += 4;
            count    -= 4;
         }
         while (count >= 4) {
            BDMCF_CMD_FILL_MEM_L(*data_ptr);    // subsequent longwords
            data_ptr += 4;
            count    -= 4;
         }
         while (count >= 1) {
            BDMCF_CMD_FILL_MEM_B(*data_ptr); // trailing bytes
            data_ptr++;
            count--;
         }
#endif
         return(1);
      }

      // Read CFV1 Memory
      case CMD_CF_READ_MEM: {
         U8 count      = commandBuffer[5];
         U8 orig_count = commandBuffer[5];
         U8 *data_ptr  = commandBuffer+1;
         U32 addr;
         asm {
            clr  addr:0
            lda  commandBuffer:4
            sta  addr:1
            lda  commandBuffer:3
            sta  addr:2
            lda  commandBuffer:2
            sta  addr:3
         }
         if (count>MAX_PACKET_SIZE-1) {
            commandBuffer[0] = BDM_RC_ILLEGAL_PARAMS; /* requested block+status is too long to fit into the buffer */
            return(1);
         }
         (void)memset(data_ptr,0x33, count);
#if 1
         while (count > 0) {
            // This is sub-optimal but MUCH smaller!
            // Do at least 1 byte (to set address) and any odd bytes
            data_ptr[0] = 0xA0;
            BDMCF_CMD_READ_MEM_B(addr,data_ptr);
            data_ptr++;
            addr++;
            count--;
            while (((addr&0x03) == 0) && (count >=4)) { 
               // Longword boundary & a longword remains to be read
               data_ptr[0] = 0xB0;
               data_ptr[1] = 0xB1;
               data_ptr[2] = 0xB2;
               data_ptr[3] = 0xB3;
               BDMCF_CMD_DUMP_MEM_L(data_ptr);
               data_ptr += 4;
               addr     += 4;
               count    -= 4;
            }
         }
#else
         // (0,-)(1,B)(2,B/B)(2,W)(3,B/W)(3,W/B)(4,B/W/B)(4,W/B/B)(4,L)
         if (((addr & 1) && (count >= 1)) || (count == 1)) {
            BDMCF_CMD_READ_MEM_B(addr,data_ptr); // odd byte
            data_ptr++;
            addr++;
            count--;
         }
         if (((addr & 2) && (count >= 2)) || (count == 2) || (count == 3))  {
            BDMCF_CMD_READ_MEM_W(addr,data_ptr); // odd word
            data_ptr += 2;
            addr     += 2;
            count    -= 2;
         }
         if (count >= 4) {
            BDMCF_CMD_READ_MEM_L(addr,data_ptr); // longword
            data_ptr += 4;
            count    -= 4;
         }
         while (count >= 4) {
            BDMCF_CMD_DUMP_MEM_L(data_ptr);    // subsequent longwords
            data_ptr += 4;
            count    -= 4;
         }
         while (count >= 1) {
            BDMCF_CMD_DUMP_MEM_B(data_ptr); // trailing bytes
            data_ptr++;
            count--;
         }
#endif            
         return(orig_count+1);
      }

         /* read block of data from memory */
      case CMD_READ_BLOCK1: {
         /* NOTE: This command returns block of BYTES, endianism does not matter here!!! */
         U8 count = commandBuffer[4];
         U8 count_orig = commandBuffer[4];
         U16 addr = (U16)((commandBuffer[3]<<8)+commandBuffer[2]);
         U8 *data_ptr = commandBuffer+1;
         
         if (count>MAX_PACKET_SIZE-1) {
            commandBuffer[0] = BDM_RC_ILLEGAL_PARAMS; /* requested block is too long to fit into the buffer */
            return(1);
         }
            
         switch (bdm_status.target_type) {
            case HC12:
               if (addr&0x0001) {
                  /* start address is odd, fetch the first byte separately */
                  BDM12_CMD_READB(addr,data_ptr);  
                  addr++;
                  data_ptr++;
                  count--;
               }
               while(count&0xfe) {
                  /* while there are at least 2 bytes to fetch */
                  BDM12_CMD_READW(addr,data_ptr);    /* fetch a word */
                  addr+=2;                           /* increment address */
                  data_ptr+=2;                       /* increment pointer */
                  count-=2;                          /* decrement count */
               }
               if (count) {
                  /* there is one extra byte to read at the end of the block */
                  BDM12_CMD_READB(addr,data_ptr);  
               }
               return(count_orig+1);
            case HCS08:
            case RS08:
               while(count>0) {
                  /* while there are bytes to fetch */
                  BDM08_CMD_READB(addr,data_ptr);    /* fetch a byte */
                  addr++;                            /* increment address */
                  data_ptr++;                        /* increment pointer */
                  count--;                           /* decrement count */
               }
               return(count_orig+1);
            }
         break;
      }
      /* write byte */
      case CMD_WRITE_8:
         switch (bdm_status.target_type) {
            case HC12:
               BDM12_CMD_WRITEB((U16)((commandBuffer[3]<<8)+commandBuffer[2]),commandBuffer[4]);
               return(1);
            case HCS08:
            case RS08:
               BDM08_CMD_WRITEB((U16)((commandBuffer[3]<<8)+commandBuffer[2]),commandBuffer[4]);
               return(1);
            }
         break;
      /* write word */
      case CMD_WRITE_16:
         BDM12_CMD_WRITEW((commandBuffer[3]<<8)+commandBuffer[2],(commandBuffer[5]<<8)+commandBuffer[4]);
         return(1);        
      /* write block of bytes to memory */
      case CMD_WRITE_BLOCK1: {
         U8 count = commandBuffer[4];
         U16 addr = (U16)((commandBuffer[3]<<8)+commandBuffer[2]);
         U8 *data_ptr = commandBuffer+5;

         switch (bdm_status.target_type) {
            case HC12:
               if (addr&0x0001) {
                  /* start address is odd, write the first byte separately */
                  BDM12_CMD_WRITEB(addr,*data_ptr);  
                  addr++;
                  data_ptr++;
                  count--;
               }
               while(count&0xfe) {
                  /* while there are at least 2 bytes to fetch */
                  BDM12_CMD_WRITEW(addr,*((U16 *)data_ptr)); /* write a word */
                  addr    +=2;   /* increment address */
                  data_ptr+=2;   /* increment pointer */
                  count   -=2;   /* decrement count */
               }
               if (count) {
                  /* there is one extra byte to write at the end of the block */  
                  BDM12_CMD_WRITEB(addr,*data_ptr);  
               }
               return(1);
            case RS08:
               if (flashState == FLASH_READY) {
                  (void)memcpy(flashProgrammingBuffer, commandBuffer, sizeof(flashProgrammingBuffer));
                  flashState = FLASH_BUSY;
                  bdmRS08_Program_Flash();
                  flashState = FLASH_READY;
                  // bdmRS08_restoreTrim();
                  //commandBuffer[0] = 0x04; // Return code of 4 BUT....
                  //return (0);               //  It doesn't actually return anything !!!!!!!!! (and its not polled for either)
                  return (1);
               }
               // Fall through otherwise
            case HCS08:
               while(count-->0) {
                  /* while there are at least 2 bytes to write */
                  BDM08_CMD_WRITEB(addr,*data_ptr);
                  addr++;  
                  data_ptr++;
               }
               return(1);
            }
         break;
      }
      /* write to PC */
      case CMD_WRITE_REG_PC:
         switch (bdm_status.target_type) {
            case HC12:
               BDM12_CMD_WRITE_PC((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
               return(1);
            case HCS08:
            case RS08:
               BDM08_CMD_WRITE_PC((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
               return(1);
            }
         break;
      /* write to SP */
      case CMD_WRITE_REG_SP:
         switch (bdm_status.target_type) {
            case HC12:
               BDM12_CMD_WRITE_SP((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
               return(1);
            case HCS08:
            case RS08:
               BDM08_CMD_WRITE_SP((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
               return(1);
            }
         break;
      /* write to IX */
      case CMD_WRITE_REG_X:
         switch (bdm_status.target_type) {
            case HC12:
               BDM12_CMD_WRITE_X((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
               return(1);
            case RS08:
               BDM08_CMD_WRITE_HX((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
               return(1);
            case HCS08: // Illegal - No Register H/X on RS08!
               commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
               return(1);
            }
         break;
      /* write to IY */
      case CMD_WRITE_REG_Y:
         switch (bdm_status.target_type) {
            case HC12: // Write Y
         BDM12_CMD_WRITE_Y((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
         return(1);
            case HCS08: // Illegal - No Register Y on HCS08!
            case RS08:  // Illegal - No Register Y on RS08!
               commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
               return(1);
            }
         break;
      /* write to B:A */
      case CMD_WRITE_REG_D:
         switch (bdm_status.target_type) {
            case HC12:
               BDM12_CMD_WRITE_D((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
               return(1);
            case HCS08:
            case RS08:
               BDM08_CMD_WRITE_A(commandBuffer[2]);
               return(1);
            }
         break;
      /* write to CCR */
      case CMD_WRITE_REG_CCR:
         /* CCR is stored in little endian notation */
         switch (bdm_status.target_type) {
            case HC12:
               BDM12_CMD_BDWRITEW(HC12_BDMCCR,*((U16 *)(commandBuffer+2))); 
               return(1);
            case HCS08:
               BDM08_CMD_WRITE_CCR(commandBuffer[2]);
               return(1);
            case RS08:  // Illegal - Write CCR+PC is used instead
               commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
               return(2);
            }
         break;
      case CMD_WRITE_BKPT:
         BDM08_CMD_WRITE_BKPT((U16)((commandBuffer[3]<<8)+commandBuffer[2]));
         return(1);
      case CMD_READ_BKPT:
         BDM08_CMD_READ_BKPT(commandBuffer+2);	
         commandBuffer[1] = commandBuffer[3];
         return(3);
      /////////////////////////////
      case CMD_FLASH_DLSTART:
         if (bdm_status.target_type != RS08) {
            commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
            return(1);
         }
         bdm_halt();               // Stop processor
         BDM08_CMD_WRITEB(RS_ICSTRM,0x80);   // Set clock trim
         BDM08_CMD_WRITEB(RS_ICS2,RS_BDIV2); // Set clock division
         rc = bdm_connect();                 // Re-connect after clock change
         if (rc != 0) {
            commandBuffer[0] = rc;
            return 1;
         }
         VPP_ON();//Turn VPP On
         bdmRS08_mass_erase();
         flashState = FLASH_READY;
         return(1);
      ////////////////////////////////////////////
      case TCMD_NEXT_DLDATA:
         if (bdm_status.target_type != RS08) {
            commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
            return(1);
         }
         if (flashState == FLASH_READY)
            commandBuffer[0] = BDM_RC_OK;
         else if (flashState == FLASH_BUSY)
            commandBuffer[0] = BDM_RC_FLASH_PROGRAMING_BUSY;
         else
            commandBuffer[0] = BDM_RC_WRONG_PROGRAMMING_MODE;
         return(1);
      ////////////////////////////////////////////
      case CMD_FLASH_DLEND:
         if (bdm_status.target_type != RS08) {
            commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
            return(1);
         }
         if (flashState == FLASH_READY) { // Programming in progress?
            bdmRS08_restoreTrim();       // Re-program trim if saved
            VPP_OFF();                   // Turn Proramming voltage off
            flashState=FLASH_IDLE;     // Programming idle now
            (void)bdm_software_reset(0);
            return(1);
         } 
         
         VPP_OFF();                    // Turn Proramming voltage off
         flashState=FLASH_IDLE;      // Programming idle now
         
         commandBuffer[0] = BDM_RC_WRONG_PROGRAMMING_MODE;
         return (1);
         
      /* unknown command */
      default: ;   
         // Do nothing - handled on exit from switch  
   }
   
   /* unknown or illegal command */
   commandBuffer[0] = BDM_RC_ILLEGAL_COMMAND;
   return(2);
}
#pragma MESSAGE DEFAULT C4000 // Restore warnings about allways true
#pragma MESSAGE DEFAULT C5900 // Restore warnings about result is zero

