#ifndef _ICP_H_
#define _ICP_H_

/*
**  This function must be provided in user code.
**  It is called from the ICP boot code to
**  determine if the user code wants ICP on this
**  reset.
**
**  See main.c.
**
**  Returns: 0 - normal boot
**           1 - ICP boot
*/
extern char userDetectICP(void);

/*
**  This function is provided in the ICP flash area and
**  accessed through a vector in a fixed location.
**  This function may be called from user code to 
**  re-program the flash to force ICP on all later resets.
**  This is a persistant change.
*/  
#define forceICPOnReset (*(void(** far)(void)) 0xFFDE)

/*
**  User vector table is in user Flash area
**  This is a jump table!
**
**  See UserVectorTable.c
**
*/
#define USER_VECTORTABLE_LOCATION  (0xF7CC)

typedef struct {
   char JMP;
   void (* const address)(void);
} UserVector;

extern const UserVector userVectorTable[17] @USER_VECTORTABLE_LOCATION;

/*
** Version number of ICP boot code
*/
#define ICP_VERSION_LOCATION (0xF800)
extern const int ICPVersion @ ICP_VERSION_LOCATION;

#endif