/*
   Open Source BDM - BDM communication
   
   *** Prominent Notice-This software was modified from TBDML software - 12/05 ***
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 bitCountle Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*   
Change Hisory
+====================================================================
| 10 Jun 2008 | Added Colfire V1 interface                     - pgo
|  6 May 2008 | Ported to JB16 from JM60 - lotsa changes       - pgo
| 12 Apr 2008 | Added reconnect before software reset          - pgo
|  7 Apr 2008 | Fixed WAIT_WITH_TIMEOUT.. macros               - pgo
|  7 Apr 2008 | Fixed glitch in bdm_init                       - pgo
|  6 Apr 2008 | Added Vdd change detect handler & POR          - pgo
| 23 Mar 2008 | Fixed several BMD commands not using ACKN      - pgo
|  3 Mar 2008 | Started changes for JM60 - lots                - pgo
+====================================================================
*/

#include "derivative.h"
#include "hidef.h"
#include "Common.h"
#include "Configure.h"
#include "bdm.h"
#include "commands.h"
#include "usb.h"
#include "TargetDefines.h"
#include "LED.h"
#include "CmdProcessing.h"

/* Function prototypes */
U8          bdm_sync_meas(void);
static void bdm_idle(void);
static U8   bdmHC12_alt_speed_detect(void);
static void bdm_check_Target_Vdd(void);

static const bdm_status_t bdm_status_default = {0,HCS08,WAIT,NO_RESET_ACTIVITY,NO_INFO,0,0,0};

//========================================================
//
#pragma DATA_SEG __SHORT_SEG Z_PAGE
// bitCount is used as a general purpose variable in the Tx & Rx routines.
// MUST be placed into the direct segment (assumed in ASM code).
static U8 rxTiming1;
static U8 rxTiming2;
static U8 rxTiming3;
static U8 txTiming1;
static U8 txTiming2;
static U8 txTiming3;
static U8 bitCount;

// pointers to Rx & Tx routines
U8   (*bdm_rx_ptr)(void) = bdm_rxEmpty;
void (*bdm_tx_ptr)(U8)   = bdm_txEmpty;

/* Status of the BDM communication */
bdm_status_t bdm_status   = {0}; // Status of the BDM interface
bdm_option_t   bdm_option = {0}; // Options for BDM interface

//========================================================
//
#pragma DATA_SEG DEFAULT
static   U8 savedTrimValue = 0; // Clock trim value for RS08 (saved from Flash)

volatile U8 flashState     = FLASH_IDLE;

U8 flashProgrammingBuffer[5 + MAX_FLASH_PROGRAM]; // 5 byte header + data to program
#pragma DATA_SEG DEFAULT

// Flash writing program to download into RS08 Direct RAM
const U8 RS08_Flash_Program[] = { 
   0x3E,0x08,0x1F,0x10,0xD1,0xCA,0xFF,0xC9,0xEF,0xEE,0xCB,0x4B,0xFE,0x3E,0x08,0x1F,
   0x16,0xD1,0xCC,0x4B,0xFE,0xCA,0xFF,0xC0,0xEF,0xCE,0x4E,0x09,0x0F,0xEE,0xCD,0x4B,
   0xFE,0x29,0x3B,0x00,0xF2,0x3E,0x08,0x1F,0x11,0xD1,0xCB,0x4B,0xFE,0x17,0xD1,0xBF 
   };

/*==============================================================
  Timer Usage:
  
  IC_BDM_TIMING_CHANNEL  (either T1CH01 or T2CH01)
      Time delays using Timer overflow (TOF)
      Input capture using Channel 0 (CH0F) - SYNC & ACN timing

  IC_VDD_SENSING_CHANNEL (either T1CH01 or T2CH01)
      Input capture using Channel 0 (CH0F) - Target Vdd changes
==============================================================*/

// T1 (SYNC & ACKN) timer constants, 6MHz ticks
#define MEAS_TIMER_PRESCALE_MASK (0)
#define MEAS_TIMER_PRESCALE      (1<<MEAS_TIMER_PRESCALE_MASK)
#define MEAS_TIMER_FREQ          (BUS_FREQ/MEAS_TIMER_PRESCALE)
#define MEAS_MICROSECOND         (MEAS_TIMER_FREQ/1000000UL)   // SYNC timer ticks in 1 us
#define T1SC_MEAS_VALUE          (T1SC_TRST_MASK|MEAS_TIMER_PRESCALE_MASK) // Set prescale & resets timer

// T1 Fast timer constants ~333ns resolution, usable to 20ms 
#define FAST_TIMER_PRESCALE_MASK (1)
#define FAST_TIMER_PRESCALE      (1<<FAST_TIMER_PRESCALE_MASK)
#define FAST_TIMER_FREQ          (BUS_FREQ/FAST_TIMER_PRESCALE)
#define MICROSECOND              (FAST_TIMER_FREQ/1000000UL)      // Fast timer ticks in 1 us
#define T1_FAST_VALUE            (T1SC_TRST_MASK|FAST_TIMER_PRESCALE_MASK)

// T1 Slow timer constants ~10us resolution, usable to 650ms
#define SLOW_TIMER_PRESCALE_MASK (FAST_TIMER_PRESCALE_MASK+5)
#define SLOW_TIMER_PRESCALE      (1<<SLOW_TIMER_PRESCALE_MASK)
#define SLOW_TIMER_FREQ          (BUS_FREQ/SLOW_TIMER_PRESCALE)
#define MILLISECOND              (SLOW_TIMER_FREQ/1000UL)      // Slow timer ticks in 1 ms
#define T1_SLOW_VALUE            (T1SC_TRST_MASK|SLOW_TIMER_PRESCALE_MASK)

#if (SLOW_TIMER_PRESCALE_MASK >= 8)
#error "Timer prescale value out of range"
#endif

//=======================================================================
//
static void fastTimerWait(U16 delay /* in fast timer ticks */){
   IC_BDM_TIMING_TMOD = delay;           // time to wait
   IC_BDM_TIMING_TSC = T1_FAST_VALUE;    // reset the timer and start counting
   IC_BDM_TIMING_TSC_TOF = 0;            // clear TOF
   while(IC_BDM_TIMING_TSC_TOF==0) {     // wait for timeout
   }
}

//=======================================================================
//
static void slowTimerWait(U16 delay /* in slow timer ticks */){
   IC_BDM_TIMING_TMOD = delay;           // time to wait
   IC_BDM_TIMING_TSC = T1_SLOW_VALUE;    // reset the timer and start counting
   IC_BDM_TIMING_TSC_TOF = 0;            // clear TOF
   while(IC_BDM_TIMING_TSC_TOF==0) {     // wait for timeout
   }
}

#define WAIT_US(t) fastTimerWait((t)*MICROSECOND)
#define WAIT_MS(t) slowTimerWait((t)*MILLISECOND)
#define WAIT_S(t)  {int tt = 4*(t); \
                    while(tt-->0)   \
                       slowTimerWait(250*MILLISECOND);}

//=======================================================================
// Wait for given microseconds or until condition is true
#define WAIT_WITH_TIMEOUT_US(t,c)               \
   IC_BDM_TIMING_TMOD = t*MICROSECOND;                       \
   /* reset the timer and start counting  */             \
   IC_BDM_TIMING_TSC = T1_FAST_VALUE;                        \
   IC_BDM_TIMING_TSC_TOF = 0;                                \
   while(!(c) && (IC_BDM_TIMING_TSC_TOF==0)) {     /* wait for timeout */   \
   }

//=======================================================================
// Wait for given millseconds or until condition is true
#define WAIT_WITH_TIMEOUT_MS(t,c)               \
   IC_BDM_TIMING_TMOD = t*MILLISECOND;                       \
   /* reset the timer and start counting  */             \
   IC_BDM_TIMING_TSC = T1_SLOW_VALUE;                        \
   IC_BDM_TIMING_TSC_TOF = 0;                                \
   while(!(c) && (IC_BDM_TIMING_TSC_TOF==0)) { /* wait for timeout */   \
   }

//=======================================================================
// Wait for given seconds or until condition is true
// Condition is only checked every 10 ms and affects the timing
#define WAIT_WITH_TIMEOUT_S(t,c)        \
   { int tt = 100*(t);                  \
      do {                              \
         slowTimerWait(10*MILLISECOND); \
      } while (!(c) & (tt-->0));        \
   }

#pragma MESSAGE DISABLE C4000 // Disable warnings about Condition always true
#pragma MESSAGE DISABLE C5909 // Disable warnings about Assignment in condition

//=======================================================================
/* Read Target BDC status from BDCSC(HC08) or BDMSTS(HC12)
   Returns:
      0                     => success
      BDM_RC_UNKNOWN_TARGET => unknown target
 */
U8 bdm_readBDMStatus(char *bdm_sts){

   switch (bdm_status.target_type) {
      case HC12:
         BDM12_CMD_BDREADB(HC12_BDMSTS,bdm_sts); 
         return 0;
      case HCS08:
      case RS08:
         BDM08_CMD_READSTATUS(bdm_sts);
         return 0;
      case CFV1:
         BDMCF_CMD_READ_XCSR(bdm_sts);
         return 0;
      default:
         return BDM_RC_UNKNOWN_TARGET; // Don't know how to check status on this one!
      }
}      

//=======================================================================
// Write Target BDC control to BDCSC(HC08) or BDMSTS(HC12)
// Returns:
//    0                     => success
//    BDM_RC_UNKNOWN_TARGET => unknown target
//
U8 bdm_writeBDMControl(char bdm_ctrl){

   switch (bdm_status.target_type) {
      case HC12:
         BDM12_CMD_BDWRITEB(HC12_BDMSTS,bdm_ctrl);
         return 0;
         break;
      case HCS08:
      case RS08:
         BDM08_CMD_WRITECONTROL(bdm_ctrl);
         return 0;
      case CFV1:
         BDMCF_CMD_WRITE_XCSR(bdm_ctrl);
         return 0;
      default:
         return BDM_RC_UNKNOWN_TARGET; // Don't know how to check status on this one!
      }
}      

//=======================================================================
// If BDM not enabled yet, enable it so it can be made active
// Returns:
//    0                     => success
//    BDM_RC_UNKNOWN_TARGET => unknown target
//    BDM_RC_BDM_EN_FAILED  => enabling BDM failed (target not connected or wrong speed ?)
//
U8 bdm_enableBDM(){
U8 bdm_sts;
U8 rc;

   if ((rc = bdm_readBDMStatus(&bdm_sts)) != 0)
      return rc;
      
   if (bdm_status.target_type==CFV1) {
      if ((bdm_sts&CFV1_XCSR_ENBDM) != 0) // BDM already enabled?
         return 0;
      if ((rc=bdm_writeBDMControl(bdm_sts|CFV1_XCSR_ENBDM))!=0) // Try to enable
         return rc;
      if ((rc = bdm_readBDMStatus(&bdm_sts)) != 0) // Enabled now?
         return rc;
      return ((bdm_sts&CFV1_XCSR_ENBDM)!=0)?0:BDM_RC_BDM_EN_FAILED;
   }
   else {
      if ((bdm_sts&HC12_BDMSTS_ENBDM)!=0) // BDM already enabled?
         return 0;
      if ((rc=bdm_writeBDMControl(bdm_sts|HC12_BDMSTS_ENBDM))!=0) // Try to enable
         return rc;
      if ((rc = bdm_readBDMStatus(&bdm_sts)) != 0) // Enabled now?
         return rc;
      return ((bdm_sts&HC12_BDMSTS_ENBDM)!=0)?0:BDM_RC_BDM_EN_FAILED;
   }
}
#pragma MESSAGE DEFAULT C4000 // Restore warnings about Condition always true
#pragma MESSAGE DEFAULT C5909 // Restore warnings about assignment in condition

//======================================================================================
// Used before some commands to ensure there is a useful connection to target available
// Some non-intrusive commands still require a running processor (e.g. READ_BYTE)
// This function will force a stopped CPU into active BDM mode 
U8 bdm_make_active_if_stopped(void) {

U8 rc;
U8 bdm_sts;

   if (bdm_status.target_type == HC12) // Not supported on HC12
      return 0;

   rc = 0;
   if (bdm_option.autoReconnect &&           // Auto re-connect enabled &
       (bdm_status.speed==SYNC_SUPPORTED)) { // Target supports Sync
      rc = bdm_sync_meas(); // Check connection speed
      if (rc == 0)
         rc = bdm_rx_tx_select();
      }
   if (rc != 0)
      return rc;

   rc = bdm_readBDMStatus(&bdm_sts);
   if (rc != 0)
      return rc;
   
   // If HC08/RS08 processor is WAITing or STOPped then force background
   if (bdm_sts&HC08_BDCSCR_WS)
      bdm_halt();
      
   return 0;   
}

//=======================================================================
// Connect to HC08/RS08/HC12/HCS12 target
// Returns:
//    0                       => success
//    BDM_RC_BDM_TIMEOUT      => BDM pin stuck low
//    BDM_RC_RESET_TIMEOUT    => RESET pin stuck low
//      
U8 bdm_connect(void) {
U8 rc;
U8 bdm_sts;

   bdm_idle(); // Make sure interface is idle

   bdm_status.ackn  = WAIT;                  /* clear the ACKN feature */
   bdm_status.reset = NO_RESET_ACTIVITY;     /* clear the reset flag */
   bdm_status.ackn  = NO_INFO;
   bdm_rx_ptr = bdm_rxEmpty;     // clear the Tx/Rx pointers
   bdm_tx_ptr = bdm_txEmpty;     // i.e. no routines found

   /* Wait with timeout until both RESET and BDM are high*/
#if (CAPABILITY&CAP_RESET)
   if (bdm_option.useResetSignal) {
      WAIT_WITH_TIMEOUT_MS(RESET_WAIT,(RESET_IN==1)&&(BDM_IN==1));
      if (RESET_IN==0) 
         return(BDM_RC_RESET_TIMEOUT);/* RESET timeout */
   }
#else   
   WAIT_WITH_TIMEOUT_MS(RESET_WAIT,(BDM_IN==1));
#endif   
      
   if (BDM_IN==0) 
      return(BDM_RC_BKGD_TIMEOUT);/* BDM timeout */
   
   rc = bdm_sync_meas();
   if ((rc!=0) &&                          // Trying to measure SYNC was not successful
       (bdm_option.guessSpeed) &&          // Try alternative method if enabled
       (bdm_status.target_type == HC12)) { // Try alternative method for HC12 only
      rc = bdmHC12_alt_speed_detect(); // Try alternative method (guessing!)
   }
   if (rc!= 0)
      return(rc);  
   
   // If at least one of the two methods succeeded, we can select
   //  the right Rx and Tx routines
   rc = bdm_rx_tx_select();
   if (rc != 0)
      return(rc); 
      
   bdm_ackn_init();  // Try the ACKN feature
   
   /* try to enable BDM */
   rc = bdm_enableBDM();
   if (rc != 0)
      return(rc); 

   // If HC08/RS08 processor is halted then force background
   rc = bdm_readBDMStatus(&bdm_sts);
   if (rc != 0)
      return rc;

   if (bdm_option.useAltBDMClock) {
      if (bdm_status.target_type == CFV1)
         (void)bdm_writeBDMControl(bdm_sts&~CFV1_XCSR_CLKSW); // Force Alt clock selection
      else
         (void)bdm_writeBDMControl(bdm_sts&~HC08_BDCSCR_CLKSW); // Force Alt clock selection
      }

//      
//   if ((bdm_status.target_type != HC12) && (bdm_sts&HC08_BDCSCR_WS))
//      BDM_CMD_BACKGROUND();   
      
   return 0;   
}

//=======================================================================
//  Mass erase (RS08)
//
// Assumes VPP is already on
//
void  bdmRS08_mass_erase(void) {

   // DisableInterrupts;
   
   // Retrieve and save the trim value from RS08 Flash
   BDM08_CMD_READB(RS_TRIM_FLASH,&savedTrimValue) ;

   // Set Up PAGESEL to point to end of flash 
   BDM08_CMD_WRITEB(RS_PAGESEL,0xfe);

   // Set the Flash Mass Erase Bit
   BDM08_CMD_WRITEB(RS_FLCR,RS_MASS);

   // Access the Flash through the paging window	 
   BDM08_CMD_WRITEB(RS_PAGEWIN+1,0x55);
   
   // Set the HVEN Bit   
   BDM08_CMD_WRITEB(RS_FLCR,RS_MASS|RS_HVEN);

   // Wait 500ms
   WAIT_MS(250);
   WAIT_MS(250);

   // Clear the Mass Bit
   BDM08_CMD_WRITEB(RS_FLCR,RS_HVEN);

   WAIT_MS(100);

   // Clear HVEN
   BDM08_CMD_WRITEB(RS_FLCR,0x00);

   (void)bdm_software_reset(0);
   
   bdm_halt();
   //bdm_connect(); 
}

//=======================================================================
//  Memory Map for RS08 Flash programming variables
//
//         ;********************************************************************************
//         ; Fast RAM ($0 - $D)
//                           org   $0
//         ;            
//         ; Programming variables - re-initialise each call
//         ;            
// a000000 SourceAddr        ds.b  1 ; Address/Count in Zero Page (Buffer) for source
// a000001 Buffer            ds.b  8 ; Buffer for data to program (reverse order)
// a000009 DestinationAddr   ds.b  1 ; Address in Zero Page (Paging Window) for destination
// a00000A DestinationPage   ds.b  1 ; Page for destination
//         ;
//         ; Timing parameters for Flash Programming - constants
//         ;          
// a00000B TimeTnvs          ds.b  1
// a00000C TimeTpgs          ds.b  1
// a00000D TimeTnvh          ds.b  1
//                        


//=======================================================================
//  This Routine programs a block of up to 8 bytes in the RS08 Flash.
//  It assumes that the Flash programming code & data have been downloaded
//  into the RS08 RAM previously .
//
//  It then tells the RS08 to execute the program in RAM and waits
//  for completion.
//
//  Assumes VPP is already on
//
static void bdmRS08_Program_Flash_Block(void) {  
U8 bdm_sts;

   // Set the PC to start of code in RAM     
   BDM08_CMD_WRITE_PC(RS_DIR_RAM);

   // We are now ready to program - execute the code on RS08
   bdm_go();
   
   // Wait until RS08 completes execution and re-enters BGND mode
   // Timeout accounts for a maximum of 8 bytes of programming, 40us * 8 = 300us      
   WAIT_WITH_TIMEOUT_US(400,((bdm_readBDMStatus(&bdm_sts) == 0) && ((bdm_sts&HC08_BDCSCR_BDMACT)!=0)));
}

//=======================================================================
//  Program Osc Trim value
//  Assumes Flash programming code is already in RS08 RAM
//  Assumes VPP is already on
//
void bdmRS08_restoreTrim(void) {
   U8 currentTrimValue;
   
   // Check if Trim already programmed
   BDM08_CMD_READB(RS_TRIM_FLASH,&currentTrimValue);
   if (currentTrimValue != 0xff) 
      return;

   // Check if there is a saved trim value - otherwise use neutral value
   if (savedTrimValue == 0)
      savedTrimValue = 0x80;
      
   // # of bytes to program
   BDM08_CMD_WRITEB(RS_FAST_RAM+0, 1);
   
   // Fill buffer (1 byte!)
   BDM08_CMD_WRITEB(RS_FAST_RAM+1, savedTrimValue);

   // Address in Zero Page (Paging Window) for destination
   BDM08_CMD_WRITEB(RS_FAST_RAM+0x09, RS_WIN_ADDR(RS_TRIM_FLASH));
   
   // Page for destination
   BDM08_CMD_WRITEB(RS_FAST_RAM+0x0A, RS_PAGENO(RS_TRIM_FLASH)); 
   
   bdmRS08_Program_Flash_Block();  
}

#pragma MESSAGE DISABLE C5900 // Disable warnings about Result is zero

//=======================================================================
//  This routine programs a block of up to 64 bytes into the RS08
//  Method:
//  * Initially a program is downloaded into the RS08 RAM.  This program
//    can program up to 8-bytes of data into the Flash at a time.
//  * The data is then downloaded in small blocks and the program used
//    to copy it to the Flash memory.
//  This process is repeated until the whole block is programmed into the Flash
//
//  Uses flashProgrammingBuffer[] as follows:
//      [3]:[2] = Start address (little-endian)
//      [4]     = # of bytes to program (<= 64 bytes)
//      [5...]  = data to program
//
void bdmRS08_Program_Flash(void) {  
   U8  dataPtr;
   U16 addr;
   U8  bytesRemaining;
   U8  bytesThisBlock;
   
   // Enter Active BG mode change speed and re/sync    
   bdm_halt();

   //BDM08_CMD_WRITEB(RS_ICS2,  RS_BDIV2); 
   //BDM08_CMD_WRITEB(RS_ICSTRM,128); 
   //bdm_connect();    

   // Download Flash Programming code to RS08 Direct RAM
   addr = RS_DIR_RAM; // Start address for Flashing program in RAM
   for(dataPtr=0; dataPtr<sizeof(RS08_Flash_Program); addr++, dataPtr++ ) {
      BDM08_CMD_WRITEB(addr,RS08_Flash_Program[dataPtr]);
   }      
   
   // Set timing Constants  
   BDM08_CMD_WRITEB(RS_FAST_RAM+0x0B,  2); // Tnvs, Tnvh = 5 us
   BDM08_CMD_WRITEB(RS_FAST_RAM+0x0C,  4); // Tpgs = 10 us
   BDM08_CMD_WRITEB(RS_FAST_RAM+0x0D, 16); // Tprog = 40 us

   // Address to start programming at
   addr    = (flashProgrammingBuffer[3]<<8)+flashProgrammingBuffer[2];

   // Offset in buffer of data to program
   dataPtr = 5;

   // Number of bytes remaining to program
   bytesRemaining = flashProgrammingBuffer[4];

   do { 
      // Program in blocks of up to 8 bytes within an 8-byte boundary
      bytesThisBlock = 8 - (addr & 7);
      
      // Check if this is more than remaining bytes to program
      if (bytesRemaining < bytesThisBlock) 
         bytesThisBlock = bytesRemaining;
         
      // Set # of bytes to program
      BDM08_CMD_WRITEB(RS_FAST_RAM+0x00,bytesThisBlock); 

      // Address in Zero Page (within Paging Window) for destination
      BDM08_CMD_WRITEB(RS_FAST_RAM+0x09,RS_WIN_ADDR(addr));
      
      // Page for destination
      BDM08_CMD_WRITEB(RS_FAST_RAM+0x0A,RS_PAGENO(addr));  
      
      bytesRemaining -= bytesThisBlock;
      addr           += bytesThisBlock;

      do {      
         // Copy data to buffer in RS08 (in reverse order!)
         BDM08_CMD_WRITEB(RS_FAST_RAM+bytesThisBlock,flashProgrammingBuffer[dataPtr]);
         dataPtr++;
         } while (--bytesThisBlock > 0);

      bdmRS08_Program_Flash_Block(); // Program up to 8 bytes
      
   } while ( bytesRemaining > 0);
   
   (void)bdm_software_reset(0);
   
   //flashState = FLASH_READY; 
   bdm_halt();
   //bdm_connect(); 
   //EnableInterrupts;  
}
#pragma MESSAGE DEFAULT C5900 // Restore warnings about Result is zero

//==============================================================
// Turns on Target Vdd if enabled.
//
//  Returns :
//   0                        => Target Vdd confirmed on target
//   BDM_RC_VDD_NOT_PRESENT   => Target Vdd not present
//
static U8 setTargetVdd( void ) {
U8 rc = 0;
#if (CAPABILITY&CAP_VDDCONTROL)
    switch (bdm_option.targetVdd) {
      case BDM_TARGET_VDD_OFF :
         VDD_OFF();
         // Check for externally supplied target Vdd (> 2 V)
         WAIT_US(VDD_RISE_TIME); // Wait for Vdd to rise & stabilise
         if (bdm_Target_Vdd_measure()<((2*255)/5)*9/10)
            rc = BDM_RC_VDD_NOT_PRESENT;
         break;
      case BDM_TARGET_VDD_3V3 :
         VDD3_ON();
         // Wait for Vdd to rise to 90% of 3V3
         WAIT_WITH_TIMEOUT_MS( 250 /* ms */, (bdm_Target_Vdd_measure()>((3*255)/5)*9/10));
         WAIT_US(VDD_RISE_TIME); // Wait for Vdd to rise & stabilise
         if (bdm_Target_Vdd_measure()<((3*255)/5)*9/10) {
            rc = BDM_RC_VDD_NOT_PRESENT;
            VDD_OFF(); // In case of Vdd overload
            }
         break;
      case BDM_TARGET_VDD_5V  :
         VDD5_ON();
         // Wait for Vdd to rise to 90% of 5V
         WAIT_WITH_TIMEOUT_MS( 250 /* ms */, (bdm_Target_Vdd_measure()>((5*255)/5)*9/10));
         WAIT_US(VDD_RISE_TIME); // Wait for Vdd to rise & stabilise
         if (bdm_Target_Vdd_measure()<((5*255)/5)*9/10) {
            rc = BDM_RC_VDD_NOT_PRESENT;
            VDD_OFF(); // In case of Vdd overload
            }
         break;
    }
#endif // CAP_VDDCONTROL

#if (CAPABILITY&CAP_VDDSENSE)
   IC_VDD_SENSING_TSC0_CH0F = 0;  // Clear Vdd monitoring flag
#endif
   bdm_check_Target_Vdd(); // Update Target Vdd LED
   
   return (rc);
}

//===========================================================
//  Cycle power to target
//  BDM is held low when power is re-applied to start
//  target with BDM active
//  Returns :
//   0                        => Target Vdd cycled OK
//   BDM_RC_VDD_WRONG_MODE    => Target Vdd not controlled by BDM interface
//   BDM_RC_VDD_NOT_REMOVED   => Target Vdd failed to fall
//   BDM_RC_VDD_NOT_PRESENT   => Target Vdd failed to rise
//   BDM_RC_RESET_TIMEOUT     => RESET signal failed to rise
//   BDM_RC_BKGD_TIMEOUT      => BKGD signal failed to rise

U8 bdm_cycle_Target_Vdd(void) {
U8 rc = 0;

#if (CAPABILITY&CAP_VDDCONTROL)

   bdm_check_Target_Vdd();

   if (bdm_option.targetVdd == BDM_TARGET_VDD_OFF)
      return BDM_RC_VDD_WRONG_MODE;

   bdm_idle();  // Make sure BDM interface is idle
      
#if (DEBUG&CYCLE_DEBUG)
   DEBUG_PIN     = 0;
   DEBUG_PIN     = 1;
#endif

   // BDM pin=L, RESET pin=L
   DATA_PORT      = BDM_DIR_Rx_WR;
   DATA_PORT_DDR  = BDM_DIR_Rx_MASK|RESET_OUT_MASK|BDM_OUT_MASK;
   RESET_ENABLE();
   
   // Power off & wait for Vdd to fall to ~5%
   VDD_OFF();
   WAIT_WITH_TIMEOUT_S( 3 /* s */, (bdm_Target_Vdd_measure()<10) );

#if (DEBUG&CYCLE_DEBUG)
   DEBUG_PIN   = 1;
   DEBUG_PIN   = 0;
#endif
   
   if (bdm_Target_Vdd_measure()>=15) { // Vpp didn't turn off!
      rc = BDM_RC_VDD_NOT_REMOVED;
      goto cleanUp;
      }

#if (DEBUG&CYCLE_DEBUG)
   DEBUG_PIN     = 0;
   DEBUG_PIN     = 1;
#endif

   bdm_check_Target_Vdd(); // Update Target Vdd LED

   // Wait a while with power off
   WAIT_US(RESET_SETTLE);

   // Power on (RESET & BDM Low)
   rc = setTargetVdd();
   if (rc != 0) // No target Vdd
      goto cleanUp;

#if (DEBUG&CYCLE_DEBUG)
   DEBUG_PIN     = 1;
   DEBUG_PIN     = 0;
#endif

   // Release RESET
   // RESET rise may be delayed by target POR
   DATA_PORT_DDR &= ~RESET_OUT_MASK;  // 3-state RESET
   RESET_3STATE();
   if (bdm_option.useResetSignal)
      WAIT_WITH_TIMEOUT_MS( 250 /* ms */, (RESET_IN==1) );

#if (DEBUG&CYCLE_DEBUG)
   DEBUG_PIN   = 0;
   DEBUG_PIN   = 1;
#endif

   // Let signals settle & CPU to finish reset (with BDM held low)
   WAIT_US(RESET_SETTLE);

   if (bdm_option.useResetSignal && (RESET_IN==0)) { 
      // RESET didn't rise
      rc = BDM_RC_RESET_TIMEOUT;
      goto cleanUp;
      }

#if (DEBUG&CYCLE_DEBUG)
   DEBUG_PIN     = 1;
   DEBUG_PIN     = 0;
#endif

   bdm_idle();  // Make sure BDM interface is idle (BDM now high)

   // Let processor start up
   WAIT_US(RESET_RECOVERY);

   if (BDM_IN==0) { // BDM didn't rise!
      rc = BDM_RC_BKGD_TIMEOUT;
      goto cleanUp;
      }

cleanUp:
   bdm_idle();  // Make sure BDM interface is idle
   WAIT_MS( 250 /* ms */);
   // Clear Vdd monitoring interrupt (ACMP interrupts)
#if (CAPABILITY&CAP_VDDSENSE)
   IC_VDD_SENSING_TSC0_CH0F = 0;  // Clear Vdd monitoring flag
#endif   
   bdm_check_Target_Vdd(); // Update Target Vdd LED

#endif // CAP_VDDCONTROL

   return(rc);
}

#if (CAPABILITY&CAP_RESET)

//======================================================================================
// Resets the target using the BDM reset line.
// Note: Not all targets have a reset pin or support reset to BDM mode with a reset pin.
//
// mode == 0 -> reset to special mode,
// mode == 1 -> reset to normal mode
//
// Returns:
//    0                       => success
//    BDM_RC_BDM_TIMEOUT      => BDM pin stuck low
//    BDM_RC_RESET_TIMEOUT    => RESET pin stuck low
//      
U8 bdm_hardware_reset(U8 mode) {

   KBSCR_IMASKK = 1;    // Mask KBD (Reset) interrupts

   bdm_idle();  // Make sure BDM interface is idle
   
   if (!bdm_option.useResetSignal)
      return BDM_RC_ILLEGAL_COMMAND;

   // Wait with timeout until both RESET and BDM are high
   WAIT_WITH_TIMEOUT_MS(RESET_WAIT, (RESET_IN==1)&&(BDM_IN==1));
   
   if (RESET_IN==0) return(BDM_RC_RESET_TIMEOUT);
   if (BDM_IN==0)   return(BDM_RC_BKGD_TIMEOUT);
   
   bdm_tx_prepare(); // BDM & RESET 3-state control on DIR_PORT
   
   if (mode==0) {
      BDM_OUT = 0;        // Drive BDM low 
      BDM_ENABLE_TX();     
      
      WAIT_US(RESET_SETTLE);                        // Wait for signals to settle
   }

   RESET_OUT = 0;          // Drive RESET low
   DATA_PORT_DDR |= RESET_OUT_MASK;
   RESET_ENABLE();
   WAIT_MS(RESET_LENGTH);  // Wait for reset pulse duration

   DATA_PORT_DDR &= ~RESET_OUT_MASK;  // 3-state RESET
   RESET_OUT = 1;          
   RESET_3STATE();
   
   // Wait with timeout until RESET is high
   WAIT_WITH_TIMEOUT_MS(RESET_WAIT, (RESET_IN==1));

   // Assume RESET risen - check later after cleanUp

#if (DEBUG&RESET_DEBUG)
   DEBUG_PIN   = 0;
   DEBUG_PIN   = 1;
#endif
   
   // wait recovery time before allowing anything else to happen on the BDM
   WAIT_MS(RESET_RECOVERY);                        // Wait for Target to start up after reset

   bdm_idle(); // Place interface in idle state

#if (DEBUG&RESET_DEBUG)
   DEBUG_PIN   = 1;
   DEBUG_PIN   = 0;
#endif

   if (RESET_IN==0) return(BDM_RC_RESET_TIMEOUT);    // RESET failed to rise
   
   KBSCR_ACKK=1;                          /* acknowledge KBD interrupt */
   KBSCR_IMASKK = 0;                      /* enable KBD interrupts again */

   return(0);
}

#endif //(CAPABILITY&CAP_RESET)
   
//=======================================================================
// Resets the target using BDM commands
// Note: Not all targets support reset using BDM commands
//
// mode == 0 -> reset to special mode,
// mode == 1 -> reset to normal mode
//
// Returns:
//    0                       => success
//    BDM_RC_BDM_TIMEOUT      => BDM pin stuck low
//    BDM_RC_RESET_TIMEOUT    => RESET pin stuck low
//    BDM_RC_UNKNOWN_TARGET   => don't know how to reset this type of target!
//      
U8 bdm_software_reset(U8 mode) {
U8 rc;

   if (bdm_status.target_type == HC12) // Doesn't support s/w reset
      return BDM_RC_UNKNOWN_TARGET;

   // Make sure of connection   
   rc = bdm_connect();

   if ((rc!=0) && bdm_option.cycleVddOnConnect) { // Still no connection to target - cycle power
      (void)bdm_cycle_Target_Vdd(); // Ignore errors
      rc = bdm_connect(); // Try connect again
   }

   if (rc!=0) { // Still no connection to target - give up
      return rc;
   }
   
   // Make sure Active background mode (in case target is stopped!)
   bdm_halt();

   switch (bdm_status.target_type) {
      case HCS08:
         BDM08_CMD_RESET();
         break;
      case RS08:
         BDMRS08_CMD_RESET();
         break;
      case CFV1:
         // Force reset (& set up to halt in BDM mode on various errors)
         if (mode == 0)
            BDM_CMD_1B_0_T(CF_WRITE_CSR2_BYTE,CFV1_CSR2_BDFR|CFV1_CSR2_COPHR|CFV1_CSR2_IOPHR|CFV1_CSR2_IADHR|CFV1_CSR2_BFHBR);
         else
            BDM_CMD_1B_0_T(CF_WRITE_CSR2_BYTE,CFV1_CSR2_BDFR|CFV1_CSR2_COPHR|CFV1_CSR2_IOPHR|CFV1_CSR2_IADHR);
         break;
      default:
         return BDM_RC_UNKNOWN_TARGET; // Don't know how to reset this one!
   }

   // BDM Interface is left in Tx mode by the above reset commands

   if (mode == 0) { // Special mode - need BDM held low out of reset
      BDM_OUT  = 0;                                // drive BDM low
      BDM_ENABLE_TX();
   }
   
#if (DEBUG&RESET_DEBUG)
   DEBUG_PIN   = 0;
   DEBUG_PIN   = 1;
#endif

   WAIT_US(RESET_SETTLE);   // Wait for target to start reset (and possibly assert reset)

#if (CAPABILITY&CAP_RESET)
   if (bdm_option.useResetSignal) {
      // Wait with timeout until RESET is high (may be held low by processor)
      WAIT_WITH_TIMEOUT_MS(RESET_WAIT, (RESET_IN==1));
      // Assume RESET risen - check later after cleanup
      }
#endif // (CAPABILITY&CAP_RESET)

#if (DEBUG&RESET_DEBUG)
   DEBUG_PIN   = 1;
   DEBUG_PIN   = 0;
#endif

   if (mode == 0) { // Special mode - release BDM
      WAIT_US(BKGD_WAIT);                   // Wait for BKGD assertion time after reset rise
      BDM_3STATE_TX();
      WAIT_US(RESET_SETTLE);     // time to wait for signals to settle
   }

   bdm_tx_finish(); 

#if (DEBUG&RESET_DEBUG)
   DEBUG_PIN   = 0;
   DEBUG_PIN   = 1;
#endif
   
   // wait recovery time before allowing anything else to happen on the BDM
   WAIT_MS(RESET_RECOVERY);               // Wait for Target to start up after reset

   bdm_idle();                            // Place interface in idle state

#if (DEBUG&RESET_DEBUG)
   DEBUG_PIN   = 1;
   DEBUG_PIN   = 0;
#endif
   
   //  BDM_CMD_BACKGROUND();  // For RS08?
   return 0;
}

//=======================================================================
// Resets the target using Vdd power cycle (POR), software or hardware reset.
//
// mode == 0 -> reset to special mode,
// mode == 1 -> reset to normal mode
//
// Returns:
//    0        => success
//    != 0     => various errors
//      
U8 bdm_target_reset( U8 mode ) {
U8 rc = 1;

   // Power-on-reset - applies to all chips
   if (bdm_option.cycleVddOnReset)
      rc = bdm_cycle_Target_Vdd();

   // Software (BDM Command) reset - HCS08, RS08 & Coldfire
   if (rc != 0)
      rc = bdm_software_reset(mode);

#if (CAPABILITY&CAP_RESET)
   // Hardware (RESET pin) reset - Some HCS08s (may not result in BDM mode) and HC12
   if ((rc != 0) && bdm_option.useResetSignal)
      rc = bdm_hardware_reset(mode); 
#endif //(CAPABILITY&CAP_RESET)
      
   // Assume we have lost connection after reset
   bdm_status.speed = NO_INFO;               // No info about speed
   bdm_status.ackn   = WAIT;                  // clear the ACKN feature
   bdm_status.reset  = NO_RESET_ACTIVITY;     // clear the reset flag
   bdm_status.ackn   = NO_INFO;
   bdm_rx_ptr        = bdm_rxEmpty;     // clear the Tx/Rx pointers
   bdm_tx_ptr        = bdm_txEmpty;     // i.e. no routines found

   return rc;
}

//=======================================================================
// Interrupt function servicing the IC interrupt from RESET_IN assertion 
//
interrupt
void bdm_reset_sense(void) {
   KBSCR_ACKK        = 1;                 // Acknowledge RESET IC interrupt
   bdm_status.reset  = RESET_DETECTED;    // Record the fact that reset was asserted
}

//===============================================================================
//  Target power has been externally cycled. Holds reset/bdm low while Vdd rises
//
U8 bdm_power_on_reset(void) {
U8 rc = 0;

#if (CAPABILITY&CAP_VDDSENSE)

   bdm_idle();  // Make sure BDM interface is idle

   // BDM pin=L, RESET pin=L
   DATA_PORT      = BDM_DIR_Rx_WR;
   DATA_PORT_DDR  = BDM_DIR_Rx_MASK|RESET_OUT_MASK|BDM_OUT_MASK;
   RESET_ENABLE();
   WAIT_US(RESET_SETTLE);

   // Release reset
   // Wait for Vdd to rise within 50% of 3V and RESET to return high
   // RESET rise may be delayed by target POR
   DATA_PORT = BDM_DIR_Rx_WR|RESET_OUT_MASK;
   RESET_3STATE();

   WAIT_WITH_TIMEOUT_MS( 300 /* ms */, (bdm_Target_Vdd_measure()>75)&&
                                     ((RESET_IN==1)||!bdm_option.useResetSignal));

   // Let signals settle & CPU to finish reset (with BDM held low)
   WAIT_US(BKGD_WAIT);

   if (bdm_Target_Vdd_measure()<=70) // Vpp didn't turn on!
      rc = BDM_RC_VDD_NOT_PRESENT;

   if (bdm_option.useResetSignal && (RESET_IN==0)) // RESET didn't rise
      rc = BDM_RC_RESET_TIMEOUT;

   bdm_idle();  // Make sure BDM interface is idle (BDM now high)

   // Let signals settle
   WAIT_US(BKGD_WAIT);

   if (BDM_IN==0) // BDM didn't rise!
      rc = BDM_RC_BKGD_TIMEOUT;

   bdm_status.reset = RESET_DETECTED;    // Record the fact that reset was asserted

#endif // (CAPABILITY&CAP_VDDSENSE)
   return(rc);
}

//=======================================================================
// 
static void bdm_check_Target_Vdd(void) {
#if (CAPABILITY&CAP_VDDSENSE)
   if (bdm_Target_Vdd_measure() > 75)
      RED_LED_ON();
   else
      RED_LED_OFF();
#endif // CAP_VDDSENSE
}

//=======================================================================
// Interrupt function servicing the IC interrupt from Vdd changes
interrupt
void bdm_Target_Vdd_sense(void) {
#if (CAPABILITY&CAP_VDDSENSE)
   IC_VDD_SENSING_TSC0_CH0F = 0; // Acknowledge Vdd change interrupt
   VDD_SENSE_DDR = 0;

   if (VDD_SENSE == 0) {  // Vdd rising
      (void)bdm_power_on_reset(); // Needs to be done on non-interrupt thread?
      bdm_check_Target_Vdd();
      }
   else { // Vdd falling
      VDD_OFF();   // Turn off Vdd in case it's an overload
      bdm_idle();  // Make sure BDM interface is idle
      // BDM pin=L, RESET pin=L
      DATA_PORT      = BDM_DIR_Rx_WR;
      DATA_PORT_DDR  = BDM_DIR_Rx_MASK|RESET_OUT_MASK|BDM_OUT_MASK;
      RESET_ENABLE();
      bdm_check_Target_Vdd();
      }
#endif // CAP_VDDSENSE
}

//=======================================================================
// Waits 64 BDM cycles of the target MCU
void bdm_wait64(void) {
   asm {
      LDA  bdm_status.wait64_cnt  /* number of loop iterations to wait */
   loop:  
      DBNZA loop                  /* 3 cycles per iteration */
   }
}

//=======================================================================
// Waits 150 BDM cycles of the target MCU
void bdm_wait150(void) {
   asm {
      LDA  bdm_status.wait150_cnt   /* number of loop iterations to wait */
   loop:  
      DBNZA loop                    /* 3 cycles per iteration */
   }
}


//=======================================================================
// Measures the SYNC length and writes the result into bdm_status structure
// Returns:
//    0                       => success
//    BDM_RC_BDM_TIMEOUT      => BDM pin stuck low or no response
//      
U8 bdm_sync_meas(void) {
   U16 time;

   bdm_status.speed = NO_INFO;  // indicate that we do not have a clue about target speed at the moment...

   bdm_idle();                            // Make sure BDM interface is idle

   // Wait with timeout until BDM is high
   WAIT_WITH_TIMEOUT_US(BDM_SYNC_REQ, (BDM_IN==1));

   if (IC_BDM_TIMING_TSC_TOF)                          // timeout!
      return(BDM_RC_BKGD_TIMEOUT); // timeout !
   
   // BDM pin=L, RESET pin=3S (RESET 3-state & pulled high)
   BDM_OUT = 0;
   BDM_DIR_Rx = 0;                        // Bring BDM low
   WAIT_US(BDM_SYNC_REQ);                 // Wait SYNC request time

#if (DEBUG&SYNC_DEBUG)
   DEBUG_PIN     = 0;
   DEBUG_PIN     = 1;
#endif
   // Set up Input capture & timeout timers
   IC_BDM_TIMING_TMOD    = SYNC_TIMEOUT*MEAS_MICROSECOND; // Set timeout
   IC_BDM_TIMING_TSC0    = T1SC0_ELS0B_MASK;              // Capture falling edges
   IC_BDM_TIMING_TSC     = T1SC_MEAS_VALUE;               // Reset the timer, clear TOF and start counting at fastest rate
   IC_BDM_TIMING_TSC_TOF = 0;                             // Clear TOF 
   IC_BDM_TIMING_TSC0_CH0F=0;                             // Clear capture flag

   asm {
      LDHX  #@DATA_PORT                            // Point to DATA port
      LDA   #BDM_OUT_MASK|RESET_OUT_MASK|BDM_DIR_Rx_RD            // Mask to 3-state BDM (reset is 3-state)

      MOV   #BDM_OUT_MASK|RESET_OUT_MASK|BDM_DIR_Rx_WR,DATA_PORT  // [4 ppwp]  Drive BDM high (reset is 3-state)
      STA   ,X                                     // [2   wp]  Release BDM   
      // It took 333 ns cycles from bringing BDM high to 3-state (2 cycles/6 MHz)
      // Target drives BDM line after 16 BDM clock cycles
      // Fast enough up to approx 333ns/(16 BDM cycles) = 48 MHz BDM Frequency
   }

   while ((IC_BDM_TIMING_TSC0_CH0F==0)&&(IC_BDM_TIMING_TSC_TOF==0)) {      /* wait for capture or timeout */  
   }
   time=IC_BDM_TIMING_TCH0;                                  /* capture start of the SYNC */
   IC_BDM_TIMING_TSC0 = T1SC0_ELS0A_MASK;                    /* capture rising edge, clear capture flag */

   /* it takes 32 cycles to reenable capture (worst case) which is good enough up to 128*6/32 = 24 MHz on JB16 */ 
   while ((IC_BDM_TIMING_TSC0_CH0F==0)&&(IC_BDM_TIMING_TSC_TOF==0)) {      /* wait for capture or timeout */  
   }
   time=IC_BDM_TIMING_TCH0-time;                             /* calculate length of the SYNC */
#if (DEBUG&SYNC_DEBUG)
   DEBUG_PIN   = 1;
   DEBUG_PIN   = 0;
#endif

   if (IC_BDM_TIMING_TSC_TOF) 
      return(BDM_RC_SYNC_TIMEOUT);         // timeout !

#if (BUS_FREQ==3000000UL)
   bdm_status.sync_length=(time<<2)+(time<<4);  /* multiply by 20 to get the time in 60MHz ticks */
#elif (BUS_FREQ==6000000UL)
   bdm_status.sync_length=(time<<1)+(time<<3);  /* multiply by 10 to get the time in 60MHz ticks */
#elif (BUS_FREQ==24000000UL)
   bdm_status.sync_length=(time<<1)+(time>>1);  // multiply by 2.5 to get the time in 60MHz ticks
#else
   #error "Please fix SYNC length calculation for new Bus Frequency"
#endif
   bdm_status.speed = SYNC_SUPPORTED;           /* SYNC feature is supported by the target */

   return(0);
}

//=======================================================================
// Enables ACKN and prepares the timer for easy ACKN timeout use
void bdm_ackn_init(void) {


   IC_BDM_TIMING_TMOD = ACKN_TIMEOUT * MEAS_MICROSECOND;  // The timer will set the TOF flag as soon as the timeout time is reached
   IC_BDM_TIMING_TSC  = T1SC_MEAS_VALUE;                  // Reset the timer, clear TOF and start counting at fastest rate
   IC_BDM_TIMING_TSC0 = T1SC0_ELS0A_MASK;	                // Capture rising edges

#if (DEBUG&ACK_DEBUG)
   DEBUG_PIN     = 0;
   DEBUG_PIN     = 1;
   DEBUG_PIN     = 0;
#endif

   bdm_status.ackn = ACKN;                   // Switch ACKN on
   if (bdm_status.target_type==CFV1)
      BDMCF_CMD_ACK_ENABLE();
   else
      BDM_CMD_ACK_ENABLE();                     // Send the ACK enable command to the target
}

//=======================================================================
// Waits for ACKN pulse from the target (with timeout)
void bdm_ackn(void) {
   IC_BDM_TIMING_TSC = T1SC_TRST_MASK;                    /* restart the timer */
   IC_BDM_TIMING_TSC_TOF = 0;                             /* clear TOF (must be RMW cycle!) */

   while ((IC_BDM_TIMING_TSC0_CH0F==0)&&(IC_BDM_TIMING_TSC_TOF==0)){   // wait for capture or timeout
   }
   IC_BDM_TIMING_TSC0 = T1SC0_ELS0A_MASK;                 /* capture rising edge, clear capture flag */
   if (IC_BDM_TIMING_TSC_TOF) {      // timeout
      bdm_status.ackn = WAIT;        // switch the ackn feature off

#if (DEBUG&ACK_DEBUG)
   DEBUG_PIN     = 1;
#endif

#if (DEBUG&ACK_DEBUG)
   DEBUG_PIN   = 0; // Flag ACKN failed
#endif
   }
}

//============================================================
// Initialises BDM module for the given target processor
//
// Entry :
//  target - Target processors (HC12, HCS08, RS08 or T_OFF)
//
U8 bdm_init(U8 target) {
U8 rc = 0;

   if (commandBuffer[2] == T_OFF) {
      bdm_off();
      return 0;
      }
      
   VPP_OFF();
   FLASH12V_OFF();
   flashState = FLASH_IDLE;
   
   bdm_idle();     // Default 'idle' port setup
   
   // Set default status/settings
   bdm_status = bdm_status_default; 

   bdm_status.target_type  = target;
   
   rc = setTargetVdd(); // Turn on Vdd if appropriate
   if (rc != 0)
      return rc;
   
#if (CAPABILITY&CAP_VDDSENSE)
   //====================================================================
   // Timer input capture is used to detect Target Vdd changes
   IC_VDD_SENSING_TMOD      = 0xFFFF;        // No timeout
   IC_VDD_SENSING_TSC       = 0;             // Start counting at bus rate
   IC_VDD_SENSING_TSC0_CH0F = 0;             // Clear capture flag
   // IC on rising or falling edge, Interrupt enabled
   IC_VDD_SENSING_TSC0      = T1SC0_CH0IE_MASK|T1SC0_ELS0B_MASK|T1SC0_ELS0A_MASK;
#endif

#if (CAPABILITY&CAP_RESET)
   if (bdm_option.useResetSignal) {
      KBIER |= RESET_IN_MASK;    // Enable RESET_IN as keyboard interrupt pin
      KBSCR  = KBSCR_ACKK_MASK;  // Acknowledge any pending interrupt and enable KBD to cause IRQ (once enabled)
   }
   else
      KBIER &= ~RESET_IN_MASK;    // Disable RESET_IN as keyboard interrupt pin
#endif

   if (target==RS08)
      FLASH12V_ON(); // Turn on charge pump for RS08 programming

   // Now give the BDM interface enough time to soft reset in case it was doing something before
   WAIT_US(SOFT_RESET);  // Wait for the longest SOFT RESET time possible
   
   switch (target) {
#if (CAPABILITY&CAP_RESET)
      case HC12:
         bdm_option.useResetSignal = 1; // Must use RESET signal on HC12
         bdm_status.initialised    = 1;
         break;
#endif
      case HCS08:
      case RS08:
      case CFV1:
         bdm_status.initialised = 1;
         break;
         
      default:
         return BDM_RC_UNKNOWN_TARGET; // Illegal target set
      }

   if (bdm_option.cycleVddOnReset && !bdm_option.leaveTargetPowered)
      rc = bdm_cycle_Target_Vdd();
      
   return rc;
}

//===========================================================
//  Halts the processor using a BDM_CMD_BACKGROUND() command
//
//  All signal undriven
//  All voltages off.
//
void bdm_halt(void) {

   if (bdm_status.target_type==CFV1)
      BDMCF_CMD_BACKGROUND();
   else
      BDM_CMD_BACKGROUND();
}

//============================================================
// Commences full-speed execution on the target
//
void bdm_go(void) {
U32 x;

   if (bdm_status.target_type == CFV1) {
      // Clear Single-step mode
      BDMCF_CMD_READ_DREG(CFV1_CSR, &x);
      x &= ~CFV1_CSR_SSM;
      BDMCF_CMD_WRITE_DREG(CFV1_CSR,x);
      
      BDMCF_CMD_GO();
      }
   else
      BDM_CMD_GO();
}


//============================================================
//  Executes a single instruction on the target
//
void bdm_step(void) {
U32 x;

   if (bdm_status.target_type == CFV1) {
      // Set Single-step mode
      BDMCF_CMD_READ_DREG(CFV1_CSR, &x);
      x |= CFV1_CSR_SSM;
      BDMCF_CMD_WRITE_DREG(CFV1_CSR,x);

      BDMCF_CMD_GO();
      }
   else
      BDM_CMD_TRACE1();
}

//================================================
//  Sets the BDM interface to a suspended state
//
//  All signal undriven
//  All voltages off.
//
void bdm_suspend(void) {
   flashState = FLASH_IDLE;
   VPP_OFF(); 
   FLASH12V_OFF();
   VDD_OFF();
   bdm_idle();
}

//=========================================================
//  Turns off the BDM interface
//  Depending upon settings, may leave target power on.
//
void bdm_off( void ) {
   flashState = FLASH_IDLE;
   VPP_OFF(); 
   FLASH12V_OFF();
   if (!bdm_option.leaveTargetPowered)
      VDD_OFF();
   bdm_idle();
}

//=======================================================
//  Sets the BDM hardware interface to an idle condition
//
//  BDM   signal undriven
//  RESET signal undriven
//  VPP,Flash voltage etc unchanged.
//
static void bdm_idle(void) {
   // Enable pull-ups on I/O lines (RST_IO has double pull-ups)
   POCR         |= POCR_PAP_MASK|POCR_PCP_MASK;
   POCR         |= POCR_PTDLDD_MASK;               // Set PTD to 10mA LED drive

   RESET_3STATE();  // Set up transceivers as read direction (through DIR_PORT)
   BDM_3STATE_TX();

   DATA_PORT     = BDM_DIR_Rx_RD|RESET_OUT_MASK;   //    through both DIR_PORT and DATA_PORT
   DATA_PORT_DDR = BDM_DIR_Rx_MASK|BDM_OUT_MASK;   // BDM_OUT & BDM_DIR_Rx enabled on DATA_PORT (pull-ups hold RESET_OUT high)
   DIR_PORT_DDR &= ~BDM_DIR_MASK;                  // Transfer BDM 3-state control to BDM_DIR_Rx on DATA_PORT
   DIR_PORT_DDR |=  RESET_DIR_MASK;                // RESET_DIR driven (holding RESET 3-state) 
   
//   - BDM & RESET pins on BDM cable are 3-state and may be read through BDM_IN & RESET_IN
//   - BDM_OUT is enabled and low
//   - BDM_DIR_Rx is enabled and BDM_DIR is 3-state so BDM direction is controlled by BDM_DIR_Rx
//   - BDM_DIR_Rx is BDM_DIR_Rx_RD (and hence BDM pin is 3-state)
}

//=======================================================================
// Prepares for transmission of BDM data
// Interrupts need to be disabled for the duration of all BDM commands
// It is up to the caller to see to this
void bdm_tx_prepare(void) {

   // Hand over BDM direction control from BDM_DIR_Rx (on BDM_DATA_PORT) to BDM_DIR (on BDM_DIR_PORT)
   // Note: direction of the BDM driver is controlled during transition to avoid glitches on BDM
                                                // Keep BDM 3-state until transfer
   DATA_PORT     = BDM_OUT_MASK|BDM_DIR_Rx_RD;  // Make sure BDM is currently 3-state & 
                                                //    will be driven high once the driver is enabled
   BDM_3STATE_TX();                             // Make sure BDM is 3-state when control is transferred to DIR_PORT
                                                
   DIR_PORT_DDR  |= BDM_DIR_MASK;               // Transfer 3-state control to DIR_PORT
   DATA_PORT_DDR  = BDM_OUT_MASK;                // BDM_OUT active others 3-state (pull-ups hold RESET_OUT high)
   
   // BDM direction controlled by BDM_DIR in DIR_PORT
   // BDM_DIR is set to disable the BDM driver
   //    Macros used to enable/disable BDM drive
   //      BDM_ENABLE_ASM_TX/BDM_ENABLE_TX()            // Enable BDM
   //      BDM_3STATE_ASM_TX/BDM_3STATE_TX()            // 3-state BDM         
   // BDM is driven via BDM_OUT in DATA_PORT - hard coded into the assembly code
   // Other bits of DATA_PORT are configured as inputs and have no effect when written
   // DATA_PORT = 0xxxxxxxx => BDM driven low  (if enabled by BDM_DIR)
   // DATA_PORT = 1xxxxxxxx => BDM driven high (if enabled by BDM_DIR)
   //
}

//=======================================================================
// Finishes transmission of BDM data and sets up for reception
void bdm_tx_finish(void) {

   // Hand over BDM direction control from BDM_DIR (on BDM_DIR_PORT) to BDM_DIR_Rx (on BDM_DATA_PORT)
   // Note: direction of the BDM driver must be controlled during transition to avoid glitches on BDM
   BDM_3STATE_TX();                              // Make sure BDM is currently 3-state
   DATA_PORT     = BDM_OUT_MASK|BDM_DIR_Rx_RD;   // Make sure BDM will remain 3-state after handover

   DATA_PORT_DDR  = BDM_OUT_MASK|BDM_DIR_Rx_MASK; // Transfer BDM_DIR control from DIR_PORT to DATA_PORT
   DIR_PORT_DDR  &= ~BDM_DIR_MASK;

   // BDM is driven via BDM_OUT in DATA_PORT with 3-state control on BDM_DIR_Rx in DATA_PORT.
   // Other bits of PTA are configured as inputs and have no effect when written.
   // The following examples assume active low enables of the buffer
   // PTA = 0xxx0xxxx => BDM driven low
   // PTA = 1xxx1xxxx => BDM 3-state
   // PTA = 1xxx0xxxx => BDM driven high (not used by Rx routines)
   // PTA = 0xxx1xxxx => BDM 3-state (not used by Rx routines?)
}

#pragma MESSAGE DISABLE C1404 // Disable warnings about no return statement

//==============================================================
// Tx Routines bdm_tx..
//==============================================================
// Transmit 8 bits of data, MSB first
// These routines assume the following:
//    BDM direction to be controlled by BDM_DIR in DIR_PORT
//      (use BDM_3STATE_TX()/BDM_3STATE_ASM_TX or BDM_ENABLE_TX()/BDM_ENABLE_ASM_TX macros)
//    BDM output value may be controlled by PTA.7
//    The rest of PTA must be disabled (call bdm_tx_finish())
// Leaves BDM_OUT 3-state via BDM_DIR (call bdm_tx_finish() to clean up)
// Timings shown are for a JB16
#if (BDM_OUT_BIT != 7)
#error "The bdm_txN(void) routines requires BDM_OUT=PTA.7"
#endif


//=========================================================================
#pragma MESSAGE DISABLE C5703 // Disable warnings about unused parameter
void bdm_txEmpty(U8 data) {
   // If BDM command is executed with this routine set command failed...
   commandBuffer[0] = BDM_RC_NO_RX_ROUTINE; 
}

//=========================================================================
// (2,2,12)
// This meets the timing but at a slow data rate
void bdm_tx0(U8 data) {
   asm {
      LDHX  @DATA_PORT             // Point HX at DATA_PORT
      BDM_ENABLE_ASM_TX            // Enable BDM
      STA  bitCount                // Save data
      LDA  #BDM_OUT_MASK           // Mask to drive BDM_OUT high
      SEC
      ROL  bitCount                // Get first data bit (and add sentinel)
      
   Loop:
      BCS  IsOne                   // [3]  '1' -> skip
      
   IsZero:
      CLR  ,X                      // [2]   Drive BDM low
                                   // --- 12
      CLR  ,X                      // [2]   Drive data (0) to BDM
                                   // --- 2
      STA  ,X                      // [2]   Drive BDM high
                                   // --- 2
      ASL  bitCount                // [4]   Get next data bit
      BEQ  Done                    // [3]   Finished? - yes, skip
      BCC  IsZero                  // [3]  '0' -> skip

   IsOne:
      CLR  ,X                      // [2]   Drive BDM low
                                   // --- 12   
      STA  ,X                      // [2]   Drive data (1) to BDM
                                   // --- 2
      STA  ,X                      // [2]   Drive BDM high
                                   // --- 2
      ASL  bitCount                // [4]   Get next data bit
      BNE  Loop                    // [3]   Finished? - no, loop
   
   Done:
      BDM_3STATE_ASM_TX            // [4]   3-state BDM         

      }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
   /* it takes 8 cycles from end of the last bit till enable of the ACKN capture */
   /* that is short enough for BDM freq of: (32+16)*3/8 = 18 MHz */
   /* 32+16 comes from minimum delay between command and ACKN (32 BDM cycles) and 16 cycles of the ACKN pulse (capturing its rising edge) */
}


//=========================================================================
// (2,3,4)
void bdm_tx1(U8 data) {
   asm {
      LDHX  @DATA_PORT     // Point HX at DATA_PORT
      BDM_ENABLE_ASM_TX    // Enable BDM (high)
      
      /* bit 7 (MSB) */
      SEC                  // [1]   Used by ROR
      CLR   ,X             // [2]   Drive BDM low
                           // --- 4
      STA   ,X             // [2]   Drive data to BDM
                           // --- 2
      ROR   ,X             // [3]   Drive BDM high
                           // --- 3
      LSLA                 // [1]   Next bit
                           // ===========
      /* bit 6 */
      SEC                  // [1]   Used by ROR
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ROR   ,X             // [3]   Drive BDM high
      LSLA                 // [1]   Next bit

      /* bit 5 */
      SEC                  // [1]   Used by ROR
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ROR   ,X             // [3]   Drive BDM high
      LSLA                 // [1]   Next bit

      /* bit 4 */
      SEC                  // [1]   Used by ROR
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ROR   ,X             // [3]   Drive BDM high
      LSLA                 // [1]   Next bit

      /* bit 3 */
      SEC                  // [1]   Used by ROR
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ROR   ,X             // [3]   Drive BDM high
      LSLA                 // [1]   Next bit

      /* bit 2 */
      SEC                  // [1]   Used by ROR
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ROR   ,X             // [3]   Drive BDM high
      LSLA                 // [1]   Next bit

      /* bit 1 */
      SEC                  // [1]   Used by ROR
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ROR   ,X             // [3]   Drive BDM high
      LSLA                 // [1]   Next bit

      /* bit 0 */
      SEC                  // [1]   Used by ROR
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ROR   ,X             // [3]   Drive BDM high

      BDM_3STATE_ASM_TX    // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (2,4,3)
void bdm_tx2(U8 data) {
   asm {
      LDHX  @DATA_PORT     // Point HX at DATA_PORT
      BDM_ENABLE_ASM_TX    // Enable BDM (high)
      
      /* bit 7 (MSB) */
      CLR   ,X             // [2]   Drive BDM low
                           // --- 3
      STA   ,X             // [2]   Drive data to BDM
                           // --- 2
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
                           // --- 4
      LSLA                 // [1]   Next bit
                           // ===========
      /* bit 6 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      LSLA                 // [1]   Next bit
      /* bit 5 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      LSLA                 // [1]   Next bit
      /* bit 4 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      LSLA                 // [1]   Next bit
      /* bit 3 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      LSLA                 // [1]   Next bit
      /* bit 2 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      LSLA                 // [1]   Next bit
      /* bit 1 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      LSLA                 // [1]   Next bit
      /* bit 0 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BDM_3STATE_ASM_TX    // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (2,5,4)
void bdm_tx3(U8 data) {
   asm {
      LDHX  @DATA_PORT     // Point HX at DATA_PORT
      BDM_ENABLE_ASM_TX    // Enable BDM (high)
      
      /* bit 7 (MSB) */
      CLR   ,X             // [2]   Drive BDM low
                           // --- 4
      STA   ,X             // [2]   Drive data to BDM
                           // --- 2
      NOP                  // [1]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
                           // --- 5
      NOP                  // [1]
      LSLA                 // [1]   Next bit
                           // ===========
      /* bit 6 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      NOP                  // [1]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 5 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      NOP                  // [1]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 4 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      NOP                  // [1]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 3 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      NOP                  // [1]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 2 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      NOP                  // [1]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 1 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      NOP                  // [1]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 0 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      NOP                  // [1]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BDM_3STATE_ASM_TX    // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (2,6,4)
void bdm_tx4(U8 data) {
   asm {
      BDM_ENABLE_ASM_TX    // Enable BDM
      CLRX                 // HX points to PTA = DATA_PORT
      CLRH
      /* bit 7 (MSB) */
      CLR   ,X             // [2]   Drive BDM low
                           // --- 4
      STA   ,X             // [2]   Drive data to BDM
                           // --- 2
      BIT   ,X             // [2]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
                           // --- 6
      NOP                  // [1]
      LSLA                 // [1]   Next bit
                           // ===========
      /* bit 6 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BIT   ,X             // [2]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 5 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BIT   ,X             // [2]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 4 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BIT   ,X             // [2]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 3 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BIT   ,X             // [2]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 2 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BIT   ,X             // [2]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 1 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BIT   ,X             // [2]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      NOP                  // [1]
      LSLA                 // [1]   Next bit
      /* bit 0 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BIT   ,X             // [2]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BDM_3STATE_ASM_TX    // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (2,7,5)
void bdm_tx5(U8 data) {
   asm {
      BDM_ENABLE_ASM_TX    // Enable BDM
      CLRX                 // HX points to PTA = DATA_PORT
      CLRH
      /* bit 7 (MSB) */
      CLR   ,X             // [2]   Drive BDM low
                           // --- 5
      STA   ,X             // [2]   Drive data to BDM
                           // --- 2
      BRN   0              // [3]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
                           // --- 7
      BIT   ,X             // [2]
      LSLA                 // [1]   Next bit
                           // ===========
      /* bit 6 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BRN   0              // [3]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BIT   ,X             // [2]
      LSLA                 // [1]   Next bit
      /* bit 5 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BRN   0              // [3]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BIT   ,X             // [2]
      LSLA                 // [1]   Next bit
      /* bit 4 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BRN   0              // [3]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BIT   ,X             // [2]
      LSLA                 // [1]   Next bit
      /* bit 3 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BRN   0              // [3]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BIT   ,X             // [2]
      LSLA                 // [1]   Next bit
      /* bit 2 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BRN   0              // [3]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BIT   ,X             // [2]
      LSLA                 // [1]   Next bit
      /* bit 1 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BRN   0              // [3]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BIT   ,X             // [2]
      LSLA                 // [1]   Next bit
      /* bit 0 */
      CLR   ,X             // [2]   Drive BDM low
      STA   ,X             // [2]   Drive data to BDM
      BRN   0              // [3]
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
      BDM_3STATE_ASM_TX    // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (2,8,5)
void bdm_tx6(U8 data) {
   asm {
      BDM_ENABLE_ASM_TX    // Enable BDM
      CLRX                 // HX points to PTA = DATA_PORT
      CLRH
      MOV   #8,bitCount    // # of bits to send
      BRA   IntoLoop       // Loop is rotated
      
   Loop:
      ROR   ,X             // [3]   Drive BDM high
                           // --- 8
      NOP                  // [1]
      LSLA                 // [1]   Next bit
   IntoLoop:
      SEC                  // [1]   This is used by ROR instruction!
      CLR   ,X             // [2]   Drive BDM low
                           // --- 5
      STA   ,X             // [2]   Drive data to BDM
                           // --- 2
      DBNZ  bitCount,Loop  // [5]   Count iteration
                           // ===========
      ROR   ,X             //       Drive BDM high
      BDM_3STATE_ASM_TX    // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (2,9,5)
void bdm_tx7(U8 data) {
   asm {
      BDM_ENABLE_ASM_TX    // Enable BDM
      CLRX                 // HX points to PTA = DATA_PORT
      CLRH
      MOV   #8,bitCount    // # of bits to send
      BRA   IntoLoop       // Loop is rotated
      
   Loop:
      ORA   #BDM_OUT_MASK  // [2]
      STA   ,X             // [2]   Drive BDM high
                           // --- 9
      BIT   ,X             // [2]
      LSLA                 // [1]   Next bit
   IntoLoop:
      CLR   ,X             // [2]   Drive BDM low
                           // --- 5
      STA   ,X             // [2]   Drive data to BDM
                           // --- 2
      DBNZ  bitCount,Loop  // [5]   Count iteration
                           // ===========
      ORA   #BDM_OUT_MASK  //
      STA   ,X             //       Drive BDM high
      BDM_3STATE_ASM_TX    // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (3,10,6)
void bdm_tx8(U8 data) {
   asm {
      BDM_ENABLE_ASM_TX                // Enable BDM
      LDX   #8                         // # of bits to send   
  Loop:      
      BSET  BDM_OUT_BIT,DATA_PORT      // [3]   Drive BDM high
                                       // --- 10
                                       // [1]
      NOP                              // [1]
      NOP                              // [1]
      BCLR  BDM_OUT_BIT,DATA_PORT      // [3]   Drive BDM low
                                       // --- 6
                                       // [1]
      STA   DATA_PORT                  // [2]   Drive data to BDM
                                       // --- 3
                                       // [1]
      NOP                              // [1]
      NOP                              // [1]
      LSLA                             // [1]   Next bit
                                       // ===========
      DBNZX Loop                       // [3]
      
      BSET  BDM_OUT_BIT,DATA_PORT      // Drive BDM high
      BDM_3STATE_ASM_TX                // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (3,12,7)
void bdm_tx9(U8 data) {
   asm {
      BDM_ENABLE_ASM_TX                // Enable BDM
      LDX   #8                         // # of bits to send   

   Loop:      
      BCLR  BDM_OUT_BIT,DATA_PORT      // [3]   Drive BDM low
                                       // --- 7
                                       // [1]
      STA   DATA_PORT                  // [2]   Drive data to BDM
                                       // --- 3
                                       // [1]
      LSLA                             // [1]   Next bit
      NOP                              // [1]
      BRN   0                          // [3]
      BRN   0                          // [3]
      BSET  BDM_OUT_BIT,DATA_PORT      // [3]   Drive BDM high
                                       // --- 12
                                       // [1]
      DBNZX Loop                       // [3]
                                       // ===========
      BDM_3STATE_ASM_TX                // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// (4,13,8)
void bdm_tx10(U8 data) {
   asm {
      BDM_ENABLE_ASM_TX                // Enable BDM
      LDX   #8                         // # of bits to send   

  Loop:      
      NOP                              // [1]
      BCLR  BDM_OUT_BIT,DATA_PORT      // [3]   Drive BDM low
                                       // --- 8
                                       // [1]
      NOP                              // [1]
      STA   DATA_PORT                  // [2]   Drive data to BDM
                                       // --- 4
                                       // [1]
      LSLA                             // [1]   Next bit
      NOP                              // [1]
      NOP                              // [1]
      BRN   0                          // [3]
      BRN   0                          // [3]
      BSET  BDM_OUT_BIT,DATA_PORT      // [3]   Drive BDM high
                                       // --- 13
                                       // [1]
      DBNZX Loop                       // [3]
                                       // ===========
      BDM_3STATE_ASM_TX                // [4]   3-state BDM         
   }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}

//=========================================================================
// >=9,>=11,>=15
void bdm_txGeneric(U8 data) {
   asm {
      BDM_ENABLE_ASM_TX             // Enable BDM
      MOV  #8,bitCount              // # of bits to send   
      
   Loop:
      LDX   txTiming3               // [3      ]
      DBNZX *+0                     // [3n     ]
      MOV   #0,DATA_PORT            // [4  ppwp]   Drive BDM low
                                    // --- 12+3n   (>=15)
      LDX   txTiming1               // [3      ]
      DBNZX *+0                     // [3n     ]
      STA   DATA_PORT               // [3   pwp]   Drive data to BDM
                                    // ---  6+3n   (>=9)
      LDX   txTiming2               // [3      ]
      DBNZX *+0                     // [3n     ]
      LSLA                          // [1      ]   Next bit
      MOV   #BDM_OUT_MASK,DATA_PORT // [4  ppwp]   Drive BDM high
                                    // ---  8+3n   (>=11)
      DBNZ  bitCount,Loop           // [5      ]
                                    // ===========
      BDM_3STATE_ASM_TX             // [4]   3-state BDM         
      }
   IC_BDM_TIMING_TSC0_CH0F = 0;    /* clear ACKN flag */
}
#pragma MESSAGE DEFAULT C5703 // Restore warnings about unused parameter


//=================================================================================
// DATA_PORT values for Rx Routines - assumes BDM_DIR_Rx controls BDM enable/direction
#define RxBDM_LOW         (BDM_DIR_Rx_WR|0)
#define RxBDM_3_STATE     (BDM_DIR_Rx_RD|BDM_OUT_MASK)  // BDM_OUT=1 as complement of value is used in code!


//==============================================================
// Rx Routines bdm_rx..
//==============================================================
// Receive 8 bit of data, MSB first
// These routines assume the following (call bdm_tx_finish()):
//    BDM direction to be controlled by BDM_DIR_Rx in PTA
//    BDM output value may be controlled by PTA.7 (hard coded in bdm_rxN())
//    BDM input value may be read from PTA.6 (hard coded in rx_stack_decode())
//    The rest of PTA must be disabled
// Leaves BDM_OUT 3-state via BDM_DIR_Rx
// Note: These routines have been re-timed to allow for the phase difference b/w read and write instructions.

#pragma MESSAGE DISABLE C20001 // Disable warnings about stackpointer

//======================================================================
/* decodes values recorded by RX functions */
/* expects LSB data in X and remaining 7 bytes on stack */
/* it is expected that caller will JUMP into this routine */

static void rx_stack_decode(void) {
   asm {
      MOV   #8,bitCount       // # of bits
decode:
#if (BDM_IN_BIT >= 4)         // Rotate left
#if (BDM_IN_BIT <= 4)
      ROLX                    // Get the interesting bit into C
#endif
#if (BDM_IN_BIT <= 5)
      ROLX                    
#endif
#if (BDM_IN_BIT <= 6)
      ROLX                    
#endif
      ROLX                    
#else // Rotate right
#if (BDM_IN_BIT >= 3)
      RORX                    // Get the interesting bit into C
#endif
#if (BDM_IN_BIT >= 2)
      RORX                    
#endif
#if (BDM_IN_BIT >= 1)
      RORX                    
#endif
      RORX                    
#endif
      RORA                    // and rotate it into A from the top
      PULX                    // get the next value from stack
      DBNZ  bitCount,decode
      PSHX                    // that was one pop too many, so push the value back
   }				
}
#pragma MESSAGE DEFAULT C20001 // Restore warnings about stackpointer

//======================================================================
// when no function appropriate for the target speed can be found the following routines are selected
// this is just to make things safe and to make sure there is a place to jump to in such a case
U8 bdm_rxEmpty(void) {
   // if BDM command is executed with this routine set as Rx it has failed...
   commandBuffer[0] = BDM_RC_NO_TX_ROUTINE; 
   return(0);
}

#pragma MESSAGE DISABLE C1404  // Disable warnings about no return statement

//======================================================================
// bdm_rx1() (2,2,6)
U8 bdm_rx1(void) {
#pragma NO_RETURN
   asm {
      LDHX  @DATA_PORT        // Point HX at DATA_PORT
 
      LDA   #RxBDM_3_STATE    // Writing A to PTA 3-states BDM
      STA   ,X                // BDM 3-state

      /* bit 7 (MSB) */
      COM   ,X                // [3   prw]   Drive BDM low
                              // --- 6
      STA   ,X                // [2    pw]   BDM 3-state
                              // --- 2
      LDX   ,X                // [2    pr]   Sample BDM_IN
                              // --- 2
      PSHX                    // [2      ]   Save bit on stack
      CLRX                    // [1      ]   Clear X again
      /* bit 6 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 5 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 4 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 3 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 2 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 1 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 0 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      LDX   ,X                // [2]   Sample BDM_IN
      /* now get the bit values (last value is in X, previous 7 on stack) */
      JMP   rx_stack_decode
   }
}

//======================================================================
// bdm_rx2() (2,3,6)
U8 bdm_rx2(void) {
#pragma NO_RETURN
   asm {
      LDHX  @DATA_PORT        // Point HX at DATA_PORT
      
      LDA   #RxBDM_3_STATE    // writing A to PTA 3-states BDM
      STA   ,X                // BDM 3-state
      /* bit 7 (MSB) */
      COM   ,X                // [3]   Drive BDM low
                              // --- 6
      STA   ,X                // [2]   BDM 3-state
                              // --- 2
      NOP                     // [1]       
      LDX   ,X                // [2]   Sample BDM_IN
                              // --- 3
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 6 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      NOP                     // [1]       
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 5 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      NOP                     // [1]       
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 4 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      NOP                     // [1]       
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 3 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      NOP                     // [1]       
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 2 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      NOP                     // [1]       
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 1 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      NOP                     // [1]       
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 0 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      NOP                     // [1]       
      LDX   ,X                // [2]   Sample BDM_IN
      /* now get the bit values (last value is in X, previous 7 on stack) */
      JMP   rx_stack_decode
   }
}

//======================================================================
// bdm_rx3() (2,4,6)
U8 bdm_rx3(void) {
#pragma NO_RETURN
   asm {
      LDHX  @DATA_PORT        // Point HX at DATA_PORT
      LDA   #RxBDM_3_STATE    // writing A to PTA 3-states BDM
      STA   ,X                // BDM 3-state
      /* bit 7 (MSB) */
      COM   ,X                // [3]   Drive BDM low
                              // --- 6
      STA   ,X                // [2]   BDM 3-state
                              // --- 2
      BIT   ,X                // [2]
      LDX   ,X                // [2]   Sample BDM_IN
                              // --- 4
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 6 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BIT   ,X                // [2]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 5 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BIT   ,X                // [2]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 4 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BIT   ,X                // [2]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 3 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BIT   ,X                // [2]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 2 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BIT   ,X                // [2]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 1 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BIT   ,X                // [2]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 0 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BIT   ,X                // [2]
      LDX   ,X                // [2]   Sample BDM_IN
      /* now get the bit values (last value is in X, previous 7 on stack) */
      JMP   rx_stack_decode
   }
}

//======================================================================
// bdm_rx4() (2,5,6)
U8 bdm_rx4(void) {
#pragma NO_RETURN
   asm {
      LDHX  @DATA_PORT        // Point HX at DATA_PORT
      LDA   #RxBDM_3_STATE    // writing A to PTA 3-states BDM
      STA   ,X                // BDM 3-state
      /* bit 7 (MSB) */
      COM   ,X                // [3]   Drive BDM low
                              // --- 6
      STA   ,X                // [2]   BDM 3-state
                              // --- 2
      BRN   0                 // [3]
      LDX   ,X                // [2]   Sample BDM_IN
                              // --- 5
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 6 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BRN   0                 // [3]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 5 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BRN   0                 // [3]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 4 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BRN   0                 // [3]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 3 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BRN   0                 // [3]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 2 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BRN   0                 // [3]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 1 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BRN   0                 // [3]
      LDX   ,X                // [2]   Sample BDM_IN
      PSHX                    // [2]   Save bit on stack
      CLRX                    // [1]   Clear X again
      /* bit 0 */                
      COM   ,X                // [3]   Drive BDM low
      STA   ,X                // [2]   BDM 3-state
      BRN   0                 // [3]
      LDX   ,X                // [2]   Sample BDM_IN
      /* now get the bit values (last value is in X, previous 7 on stack) */
      JMP   rx_stack_decode
   }
}

//======================================================================
// bdm_rx5() (3,5,7)
U8 bdm_rx5(void) {
   asm {
      LDHX  @DATA_PORT                 // Point HX at DATA_PORT
      MOV   #RxBDM_3_STATE,DATA_PORT   // BDM 3-state
      
      /* bit 7 (MSB) */
      COM   ,X                         // [3]   Drive BDM low
                                       // --- 7
      COM   ,X                         // [3]   BDM 3-state
                                       // --- 3
      BRN   0                          // [3]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2]   Sample BDM_IN
                                       // --- 5
                                       // [3]
      ROLA                             // [1]   Save bit in A
      /* bit 6 */                
      COM   ,X                         // [3]   Drive BDM low
      COM   ,X                         // [3]   BDM 3-state
      BRN   0                          // [3]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2+3] Sample BDM_IN
      ROLA                             // [1]   Save bit in A
      /* bit 5 */                
      COM   ,X                         // [3]   Drive BDM low
      COM   ,X                         // [3]   BDM 3-state
      BRN   0                          // [3]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2+3] Sample BDM_IN
      ROLA                             // [1]   Save bit in A
      /* bit 4 */                
      COM   ,X                         // [3]   Drive BDM low
      COM   ,X                         // [3]   BDM 3-state
      BRN   0                          // [3]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2+3] Sample BDM_IN
      ROLA                             // [1]   Save bit in A
      /* bit 3 */                
      COM   ,X                         // [3]   Drive BDM low
      COM   ,X                         // [3]   BDM 3-state
      BRN   0                          // [3]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2+3] Sample BDM_IN
      ROLA                             // [1]   Save bit in A
      /* bit 2 */                
      COM   ,X                         // [3]   Drive BDM low
      COM   ,X                         // [3]   BDM 3-state
      BRN   0                          // [3]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2+3] Sample BDM_IN
      ROLA                             // [1]   Save bit in A
      /* bit 1 */                
      COM   ,X                         // [3]   Drive BDM low
      COM   ,X                         // [3]   BDM 3-state
      BRN   0                          // [3]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2+3] Sample BDM_IN
      ROLA                             // [1]   Save bit in A
      /* bit 0 */                
      COM   ,X                         // [3]   Drive BDM low
      COM   ,X                         // [3]   BDM 3-state
      BRN   0                          // [3]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2+3] Sample BDM_IN
      ROLA                             // [1]   Save bit in A
   }
}

//======================================================================
// bdm_rx6() (3,7,7)
U8 bdm_rx6(void) {
   asm {
      LDHX  @DATA_PORT                 // Point HX at DATA_PORT
      MOV   #RxBDM_3_STATE,DATA_PORT   // BDM 3-state

      MOV   #8,bitCount                // # of iterations
      BRA   IntoLoop                   // Loop is rotated

      /* bits 7 down to 0 */
   Loop:
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2]   Sample BDM_IN
                                       // --- 7
                                       // [3]
      ROLA                             // [1]   Save bit in A
   IntoLoop:
      COM   ,X                         // [3]   Drive BDM low
                                       // --- 7
      COM   ,X                         // [3]   BDM 3-state
                                       // --- 3
      DBNZ  bitCount,Loop              // [5]   Count iteration

      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2]   Sample BDM_IN
                                       // --- 7
      ROLA                             //       Save bit in A
      }
}

//======================================================================
// bdm_rx7() (3,9,10)
U8 bdm_rx7(void) {
   asm {
      LDHX  @DATA_PORT                 // Point HX at DATA_PORT
      MOV   #RxBDM_3_STATE,DATA_PORT   // BDM 3-state

      MOV   #8,bitCount                // # of iterations
      BRA   IntoLoop                   // Loop is rotated

      /* bits 7 down to 0 */
   Loop:
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2]   Sample BDM_IN
                                       // --- 9
                                       // [3]
      BRN   0                          // [3]
      ROLA                             // [1]   Save bit in A
   IntoLoop:
      COM   ,X                         // [3]   Drive BDM low
                                       // --- 10
      COM   ,X                         // [3]   BDM 3-state
                                       // --- 3
      BIT   ,X                         // [2]
      DBNZ  bitCount,Loop              // [5]   Count iteration

      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2]   Sample BDM_IN
                                       // --- 9
      ROLA                             //       Save bit in A
      }
}

//======================================================================
// bdm_rx8() (4,11,12)
U8 bdm_rx8(void) {
   asm {
      LDHX  @DATA_PORT                 // Point HX at DATA_PORT
      MOV   #RxBDM_3_STATE,DATA_PORT   // BDM 3-state

      MOV   #8,bitCount                // # of iterations

      /* bits 7 down to 0 */
   Loop:
      COM   ,X                         // [3]   Drive BDM low
                                       // --- 12
      NOP                              // [1]
      COM   ,X                         // [3]   BDM 3-state
                                       // --- 4
      NOP                              // [1]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2]   Sample BDM_IN
                                       // --- 11
                                       // [3]
      ROLA                             // [1]   Save bit in A
      DBNZ  bitCount,Loop              // [5]   Count iteration
      }
}

//======================================================================
// bdm_rx9() (4,14,14)
U8 bdm_rx9(void) {
   asm {
      LDHX  @DATA_PORT                 // Point HX at DATA_PORT
      MOV   #RxBDM_3_STATE,DATA_PORT   // BDM 3-state

      MOV   #8,bitCount                // # of iterations

      /* bits 7 down to 0 */
   Loop:
      NOP                              // [1]
      COM   ,X                         // [3]   Drive BDM low
                                       // --- 14
      NOP                              // [1]
      COM   ,X                         // [3]   BDM 3-state
                                       // --- 4
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2]   Sample BDM_IN
                                       // --- 14
                                       // [3]
      NOP                              // [1]
      ROLA                             // [1]   Save bit in A
      DBNZ  bitCount,Loop              // [5]   Count iteration
   }
}

//======================================================================
// bdm_rx10() (5,16,18)
U8 bdm_rx10(void) {
   asm {
      LDHX  @DATA_PORT                 // Point HX at DATA_PORT
      MOV   #RxBDM_3_STATE,DATA_PORT   // BDM 3-state

      MOV   #8,bitCount                // # of iterations

      /* bits 7 down to 0 */
   Loop:
      COM   ,X                         // [3]   Drive BDM low
                                       // --- 18
      BIT   ,X                         // [2]
      COM   ,X                         // [3]   BDM 3-state
                                       // --- 5
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BRSET BDM_IN_BIT,DATA_PORT,*+2   // [2]   Sample BDM_IN
                                       // --- 16
                                       // [3]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      BIT   ,X                         // [2]
      ROLA                             // [1]   Save bit in A
      DBNZ  bitCount,Loop              // [5]   Count iteration
   }
}

//=======================================================================
// >7,>9,>19
U8 bdm_rxGeneric(void) {
   asm {
      LDA   #0x01                    // Value used as sentinel
      
   Loop:
      LDX   rxTiming1                // [3      ]
      MOV   #RxBDM_LOW,DATA_PORT     // [4  ppwp]   Drive BDM low
                                     // ---  16+3n
      DBNZX *+0                      // [3n     ]
      MOV   #RxBDM_3_STATE,DATA_PORT // [4  ppwp]   3-state BDM pin
                                     // ---  4+3n
      LDX   rxTiming2                // [3      ]  prpdp
      DBNZX *+0                      // [3n     ]
      BRSET BDM_IN_BIT,DATA_PORT,*+2 // [3   prp]   Sample BDM_IN
                                     // ---  6+3n
                                     // [2+   dp] 
      LDX   rxTiming3                // [3      ]
      DBNZX *+0                      // [3n     ]
      ROLA                           // [1      ]   Save sample
      BCC   Loop                     // [3      ]
   }
}

#pragma MESSAGE DEFAULT C1404  // Restore warnings about no return statement

//===============================================================================
//  When the SYNC length expressed in 60MHz ticks is ABOVE OR EQUAL to the value 
//  in the table, the corresponding pointer is selected
//  If SYNC is shorter than the second entry, the target runs too fast
//  If SYNC is longer or equal to the last entry, the target runs too slow
//
typedef struct {
   U16   syncThreshold;       // Threshold to use this function
   void  (*txFunc)(U8 data);  // Ptr to selected function
   U8    time1,time2,time3;   // Timing Parameters for function use
} TxConfiguration;

const TxConfiguration txConfiguration[] =
{
{ 0, bdm_txEmpty, 0, 0, 0 },//>21 MHz - Max Fequency
{ 366, bdm_tx0, 0, 0, 0 },//16.5 - 21 MHz, (2,2,12)
{ 461, bdm_tx1, 1, 0, 0 },//13.2 - 16.8 MHz, (2,3,4)
{ 565, bdm_tx2, 2, 0, 0 },//11 - 14 MHz, (2,4,3)
{ 668, bdm_tx3, 3, 0, 0 },//9.43 - 12 MHz, (2,5,4)
{ 772, bdm_tx4, 4, 0, 0 },//8.25 - 10.5 MHz, (2,6,4)
{ 875, bdm_tx5, 5, 0, 0 },//7.33 - 9.33 MHz, (2,7,5)
{ 979, bdm_tx6, 6, 0, 0 },//6.6 - 8.4 MHz, (2,8,5)
{ 1082, bdm_tx7, 7, 0, 0 },//6 - 7.64 MHz, (2,9,5)
{ 1234, bdm_tx8, 8, 0, 0 },//5.08 - 6.46 MHz, (3,10,6)
{ 1440, bdm_tx9, 9, 0, 0 },//4.4 - 5.6 MHz, (3,12,7)
{ 1647, bdm_tx10, 10, 0, 0 },//3.88 - 4.94 MHz, (4,13,8)
{ 1902, bdm_txGeneric, 1, 1, 1 },//3.3 - 4.2 MHz, (9,11,15)
{ 2213, bdm_txGeneric, 1, 2, 1 },//2.87 - 3.65 MHz, (9,14,15)
{ 2522, bdm_txGeneric, 1, 3, 1 },//2.54 - 3.23 MHz, (9,17,15)
{ 2830, bdm_txGeneric, 1, 4, 1 },//2.28 - 2.9 MHz, (9,20,15)
{ 3136, bdm_txGeneric, 1, 5, 1 },//2.06 - 2.63 MHz, (9,23,15)
{ 3454, bdm_txGeneric, 1, 6, 2 },//1.89 - 2.4 MHz, (9,26,18)
{ 3758, bdm_txGeneric, 1, 7, 2 },//1.74 - 2.21 MHz, (9,29,18)
{ 4213, bdm_txGeneric, 1, 9, 3 },//1.5 - 1.91 MHz, (9,35,21)
{ 4687, bdm_txGeneric, 2, 9, 4 },//1.4 - 1.79 MHz, (12,35,24)
{ 5164, bdm_txGeneric, 2, 11, 5 },//1.25 - 1.58 MHz, (12,41,27)
{ 5765, bdm_txGeneric, 3, 12, 5 },//1.12 - 1.42 MHz, (15,44,27)
{ 6517, bdm_txGeneric, 4, 14, 7 },//0.97 - 1.24 MHz, (18,50,33)
{ 7469, bdm_txGeneric, 5, 16, 8 },//0.86 - 1.09 MHz, (21,56,36)
{ 8366, bdm_txGeneric, 7, 17, 10 },//0.77 - 0.98 MHz, (27,59,42)
{ 9438, bdm_txGeneric, 10, 18, 11 },//0.67 - 0.86 MHz, (36,62,45)
{ 10907, bdm_txGeneric, 13, 20, 14 },//0.58 - 0.74 MHz, (45,68,54)
{ 12605, bdm_txGeneric, 17, 22, 16 },//0.5 - 0.64 MHz, (57,74,60)
{ 14645, bdm_txGeneric, 22, 24, 20 },//0.43 - 0.55 MHz, (72,80,72)
{ 16905, bdm_txGeneric, 28, 26, 23 },//0.38 - 0.48 MHz, (90,86,81)
{ 19224, bdm_txGeneric, 34, 28, 27 },//0.33 - 0.42 MHz, (108,92,93)
{ 21979, bdm_txGeneric, 40, 32, 31 },//0.29 - 0.37 MHz, (126,104,105)
{ 26483, bdm_txEmpty, 0, 0 ,0  },//<0.29 MHz - Min. Frequency
};

typedef struct {
   U16   syncThreshold;       // Threshold to use this function
   U8    (*txFunc)(void);     // Ptr to selected function
   U8    time1,time2,time3;   // Timing Parameters for function use
} RxConfiguration;

const RxConfiguration rxConfiguration[] =
{
{ 0, bdm_rxEmpty, 0, 0, 0 },//>20 MHz - Max Fequency
{ 384, bdm_rx1, 1, 0, 0 },//15 - 20 MHz, (2,2,6)
{ 501, bdm_rx2, 2, 0, 0 },//11.74 - 15.65 MHz, (2,3,6)
{ 625, bdm_rx3, 3, 0, 0 },//9.64 - 12.86 MHz, (2,4,6)
{ 749, bdm_rx4, 4, 0, 0 },//8.18 - 10.91 MHz, (2,5,6)
{ 873, bdm_rx5, 5, 0, 0 },//7.11 - 9.47 MHz, (3,5,7)
{ 1052, bdm_rx6, 6, 0, 0 },//5.65 - 7.5 MHz, (3,7,7)
{ 1297, bdm_rx7, 7, 0, 0 },//4.66 - 6.21 MHz, (3,9,10)
{ 1602, bdm_rx8, 8, 0, 0 },//3.7 - 4.93 MHz, (4,11,12)
{ 1974, bdm_rx9, 9, 0, 0 },//3.07 - 4.09 MHz, (4,14,14)
{ 2343, bdm_rx10, 10, 0, 0 },//2.62 - 3.5 MHz, (5,16,18)
{ 2772, bdm_rxGeneric, 1, 4, 1 },//2.2 - 2.93 MHz, (7,18,19)
{ 3205, bdm_rxGeneric, 1, 5, 2 },//1.96 - 2.61 MHz, (7,21,22)
{ 3578, bdm_rxGeneric, 1, 6, 3 },//1.76 - 2.35 MHz, (7,24,25)
{ 3957, bdm_rxGeneric, 1, 7, 3 },//1.71 - 2.14 MHz, (7,27,25)
{ 4184, bdm_rxGeneric, 2, 7, 5 },//1.48 - 1.97 MHz, (10,27,31)
{ 4856, bdm_rxGeneric, 3, 8, 6 },//1.27 - 1.69 MHz, (13,30,34)
{ 5602, bdm_rxGeneric, 4, 9, 8 },//1.11 - 1.48 MHz, (16,33,40)
{ 6345, bdm_rxGeneric, 5, 10, 9 },//0.99 - 1.32 MHz, (19,36,43)
{ 7076, bdm_rxGeneric, 6, 11, 11 },//0.89 - 1.19 MHz, (22,39,49)
{ 8021, bdm_rxGeneric, 8, 12, 13 },//0.78 - 1.03 MHz, (28,42,55)
{ 9066, bdm_rxGeneric, 9, 14, 15 },//0.69 - 0.92 MHz, (31,48,61)
{ 10210, bdm_rxGeneric, 10, 16, 18 },//0.62 - 0.82 MHz, (34,54,70)
{ 11262, bdm_rxGeneric, 11, 18, 20 },//0.56 - 0.75 MHz, (37,60,76)
{ 12446, bdm_rxGeneric, 12, 20, 22 },//0.51 - 0.68 MHz, (40,66,82)
{ 13769, bdm_rxGeneric, 14, 22, 26 },//0.46 - 0.61 MHz, (46,72,94)
{ 15132, bdm_rxGeneric, 16, 24, 30 },//0.42 - 0.56 MHz, (52,78,106)
{ 16594, bdm_rxGeneric, 18, 26, 32 },//0.38 - 0.51 MHz, (58,84,112)
{ 18572, bdm_rxGeneric, 21, 29, 36 },//0.34 - 0.45 MHz, (67,93,124)
{ 20825, bdm_rxGeneric, 24, 33, 42 },//0.3 - 0.4 MHz, (76,105,142)
{ 23369, bdm_rxGeneric, 28, 36, 48 },//0.27 - 0.36 MHz, (88,114,160)
{ 28444, bdm_rxEmpty, 0, 0 ,0  },//<0.27 MHz - Min. Frequency
};

//=======================================================================================
// Selects Rx and Tx routine to be used according to SYNC length in bdm_status structure
//
// Returns:
//   0                     => OK
//   BDM_RC_NO_TX_ROUTINE  => No suitable Tx routine found
//   BDM_RC_NO_RX_ROUTINE  => No suitable Rx routine found
//
U8 bdm_rx_tx_select(void) {
   const TxConfiguration  * far txConfigPtr;
   const RxConfiguration  * far rxConfigPtr;
   U16              sync_length;

   sync_length = bdm_status.sync_length;
   
   bdm_rx_ptr = bdm_rxEmpty; // clear the pointers
   bdm_tx_ptr = bdm_txEmpty;
   
   for (  txConfigPtr  = txConfiguration+sizeof(txConfiguration)/sizeof(txConfiguration[0]);
        --txConfigPtr >= txConfiguration; ) { // Search the table
        
      if (sync_length >= txConfigPtr->syncThreshold) {     // SYNC is >=
         bdm_tx_ptr    = txConfigPtr->txFunc; // Select this routine
         txTiming1     = txConfigPtr->time1;  // Save timing parameters
         txTiming2     = txConfigPtr->time2;
         txTiming3     = txConfigPtr->time3;
         break;                               // Quit search
      }
   }
   if (bdm_tx_ptr==bdm_txEmpty) // Return if no function found
      return(BDM_RC_NO_TX_ROUTINE);

   for (  rxConfigPtr  = rxConfiguration+sizeof(rxConfiguration)/sizeof(rxConfiguration[0]);
        --rxConfigPtr >= rxConfiguration; ) { // Search the table
        
      if (sync_length >= rxConfigPtr->syncThreshold) {     // SYNC is >=
         bdm_rx_ptr    = rxConfigPtr->txFunc; // Select this routine
         rxTiming1     = rxConfigPtr->time1;  // Save timing parameters
         rxTiming2     = rxConfigPtr->time2;
         rxTiming3     = rxConfigPtr->time3;
         break;                               // Quit search
      }
   }
   if (bdm_tx_ptr==bdm_txEmpty) // Return if no function found
      return(BDM_RC_NO_TX_ROUTINE);

   // there is plenty of overhead:  (at least 21 cycles)
   // JSR, LDA and RTS of the WAIT, RTS from the previous routine, JSR to the next routine
   /* cannot subtract more than the smallest possible result -1 as the number must be > 0 */
   bdm_status.wait64_cnt  = bdm_status.sync_length/(((3*60*128UL)/(BUS_FREQ/1000000)/64))-6;
   bdm_status.wait150_cnt = bdm_status.sync_length/(((3*60*128UL)/(BUS_FREQ/1000000)/150))-7;
   return(0);
}

#pragma MESSAGE DISABLE C5909 // Disable warnings about assignment in condition
//=======================================================================
// Confirm communication at given Sync value
// Only works on HC12 (and maybe only 1 of 'em!)
//
// Returns:
//   0                     => OK
//   != 0                  => various errors
//
U8 bdmHC12_confirm_speed(U16 syncValue) {
U8 rc;
U8 bdm_sts;

   bdm_status.sync_length = syncValue;
   
   if ((rc = bdm_rx_tx_select()) != 0) // Drivers available for this frequency?
      goto tidyUp;
   
   if ((rc = bdm_readBDMStatus(&bdm_sts)) != 0)
      goto tidyUp;

   if ((rc=bdm_writeBDMControl(bdm_sts|HC12_BDMSTS_ENBDM))!=0) // Try to enable BDM
      goto tidyUp;

   if ((rc = bdm_readBDMStatus(&bdm_sts)) != 0)
      goto tidyUp;
   
   rc = BDM_RC_BDM_EN_FAILED;
   if ((bdm_sts & HC12_BDMSTS_ENBDM) == 0) // Not enabled now? - Try next speed
      goto tidyUp;

   if ((rc=bdm_writeBDMControl(bdm_sts&~HC12_BDMSTS_ENBDM))!=0) // Try to disable BDM
      goto tidyUp;

   if ((rc = bdm_readBDMStatus(&bdm_sts)) != 0)
      goto tidyUp;
   
   rc = BDM_RC_BDM_EN_FAILED;
   if ((bdm_sts & HC12_BDMSTS_ENBDM) != 0) // Enabled now? - Try next speed
      goto tidyUp;

   return 0; // Success!
   
tidyUp:
   bdm_status.sync_length  = 1;
   bdm_status.speed        = NO_INFO; // Connection cannot be established at this speed */
   bdm_status.ackn         = WAIT;    // Clear indication of ACKN feature */
   return rc;
}
#pragma MESSAGE DEFAULT C5909 // Restore warnings about Assignment in condition

//=======================================================================
// Attempt to determine target speed by trial and error
//
// Basic process used to check for communication is:
//    (deleted)Hardware reset the target in special mode.
//    Attempt to read the BDM Status register (BDMSTS)
//    Attempt to set and clear the ENBDM bit within the BDMSTS
//
// The above is attempted for a range of 'nice' frequencies and then every Tx driver frequency.
// (deleted)The use of reset means that re-connecting to a running target after a speed change is not possible.
// 
static U8 bdmHC12_alt_speed_detect(void) {
   const TxConfiguration  * far txConfigPtr;
   int              sub;
   
#define SYNC_MULTIPLE(x)  ((128*60000UL)/(x))
   
static const U16 typicalSpeeds[] = { // Table of 'nice' BDM speeds to try 
   SYNC_MULTIPLE(8000UL),   //  8 MHz
   SYNC_MULTIPLE(16000UL),  // 16 MHz
   SYNC_MULTIPLE(4000UL),   //  4 MHz
   SYNC_MULTIPLE(32000UL),  // 32 MHz
   SYNC_MULTIPLE(24000UL),  // 24 MHz
   SYNC_MULTIPLE(20000UL),  // 20 MHz
   SYNC_MULTIPLE(2000UL),   //  2 MHz
   SYNC_MULTIPLE(10000UL),  // 10 MHz
   SYNC_MULTIPLE(1000UL),   //  1 MHz
   SYNC_MULTIPLE(500UL),    // 500kHz
   0
   };
static U16 lastGuess = SYNC_MULTIPLE(8000UL); // Used to remember last guess as first guess next time

   // Try last used speed
   if (bdmHC12_confirm_speed(lastGuess) == 0) {
      bdm_status.speed = SPEED_GUESSED;  // Speed found by trial and error
      return 0;
   }

   // Try some likely numbers!
   for (sub=0; typicalSpeeds[sub]>0; sub++){
      //rc = bdm_hardware_reset(0);
      //if (rc != 0)
      //   return rc;
      lastGuess = typicalSpeeds[sub];
      if (bdmHC12_confirm_speed(lastGuess) == 0) {
         bdm_status.speed = SPEED_GUESSED;  // Speed found by trial and error
         return 0;
      }
   }
   // Try each Tx driver BDM frequency   
   for (  txConfigPtr  = txConfiguration+(sizeof(txConfiguration)/sizeof(txConfiguration[0])-1);
        --txConfigPtr >= txConfiguration; ) { // Search the table
      //rc = bdm_hardware_reset(0);
      //if (rc != 0)
      //   return rc;
      lastGuess = (txConfigPtr->syncThreshold+(txConfigPtr+1)->syncThreshold)/2;
      if (bdmHC12_confirm_speed(lastGuess) == 0) {
         bdm_status.speed = SPEED_GUESSED;  // Speed found by trial and error
         return 0;
      }
   }
   return BDM_RC_UNKNOWN_SPEED;
}

//=======================================================================
// The following code was used to check the phase 
// relationship between port reads and writes
//
// ******   Warning -- This routine hangs the BDM  ********
//
void bdm_checkTiming(void) {

   DIR_PORT_DDR  = BDM_DIR_MASK|RESET_DIR_MASK;   // RESET/BDM direction controlled by DIR_PORT
   DIR_PORT      = BDM_DIR_WR  |RESET_DIR_RD;     // BDM out, RESET in
   
   DATA_PORT_DDR = BDM_OUT_MASK;  // BDM pin is driven, Reset pin is input
   

   asm {
      CLRH
      CLRX

   Loop1:
      NOP                               // [1]
      NOP                               // [1]
      NOP                               // [1]
      NOP                               // [1]
      NOP                               // [1]
   Loop:
      LDA  #BDM_OUT_MASK                // [2]
      STA  ,X                           // [2] BDM driven high
                                        // -----
      CLR  ,X                           // [2] BDM driven low for 333 ns
                                        // -----
      STA  ,X                           // [2] BDM driven high
                                        // -----
      NOP                               // [1]
      NOP                               // [1]
      LDA  ,X                           // [2] Sample RESET_IN
      BIT  #RESET_IN_MASK               // [2] Input high?
      BNE  Loop1                        // [3] Yes - leave BDM high
     
      CLR  ,X                           // [2] No  - make BDM low
      BRA  Loop                         // [3]  
  }
}

void bdm_checkWaitTiming(void){

   for(;;) {
      bdm_wait150();
#ifdef DEBUG_PIN      
      DEBUG_PIN   = 1;
      bdm_wait64();
      DEBUG_PIN   = 0;
#endif      
   }
}

//========================================================
//  Measures Target Vdd
//
//  Returns : N = 8-bit value representing the Vdd voltage
//            Vdd ~ (N/255) * 5V
//
U16 bdm_Target_Vdd_measure(void) {

#ifdef VDD_SENSE_DDR
   // Simple yes/no code as JB16 doesn't have a ADC
   VDD_SENSE_DDR = 0;
   asm {
      nop
      nop
      nop
   }
   return (VDD_SENSE?0:255);
#else
   return 255;   
#endif   
}
