#ifndef _TARGET_DEFINES_H_
#define _TARGET_DEFINES_H_

// HCS08 Registers addresses
//===============================
#define HCS08_SRS          (0x1800) 
#define HCS08_SBDFR        (0x1801)

// HCS08 Register bit masks
//===============================
#define HCS_SBDFR_BDFR (0x01)

// RS08 Registers addresses
//===============================
#define RS_PAGESEL     (0x001F) // Page select
#define RS_FLCR        (0x0211) // Flash Control Register
#define RS_PAGEWIN     (0x00C0) // Paging Window (0xC0-0xFF)
#define RS_ICS2        (0x0015) // Bus Freq. & Low power select
#define RS_ICSTRM      (0x0016) // ICS Trim Value
#define RS_ICSSC       (0x0017) // ICS Trim LSB
#define RS_FAST_RAM    (0x00)   // Fast scratch RAM (0x00-0x0D)
#define RS_DIR_RAM     (0x20)   // Direct page RAM (0x20-0x4F)
#define RS_TRIM_FLASH  (0x3ffa) // ICS Trim value in flash (copied to RS_ICSTRM)
#define RS_TRIM_FLASH2 (0x3ffb) // ICS Trim value LSB in flash (copied to RS_ICSSC)

// RS08 Macros
//=================
#define RS_PAGENO(addr)   (((addr)>>6) & 0xFF)       // Value for PAGEWINDOW from address
#define RS_WIN_ADDR(addr) (RS_PAGEWIN+(addr & 0x3F)) // Value for offset within window, includes PAGEWIN base address

// RS08 Register bit masks
//===============================
// FLCR
#define RS_FLCR (0x0211)
#define RS_HVEN (0x08)
#define RS_MASS (0x04)
#define RS_PGM  (0x01)

// ICSC2
#define RS_BDIV2  (1<<6)

// HC08 BDCSCR register masks 
#define HC08_BDCSCR_ENBDM  (0x80)
#define HC08_BDCSCR_BDMACT (0x40)
#define HC08_BDCSCR_CLKSW  (0x08)
#define HC08_BDCSCR_WS     (0x04)

// HC12 Register addresses (BDM Space)
#define HC12_BDMCCR     0xFF06	/* address of CCR register in BDM memory space */
#define HC12_BDMSTS     0xFF01	/* location of BDM Status register */
//#define BDM12_GUESS_ADDR     0xFF00   /* location of register to be read when trying to guess speed of the target */

// HC12 BDMSTS register masks 
#define HC12_BDMSTS_ENBDM  (0x80)
#define HC12_BDMSTS_BDMACT (0x40)

//==========================================================

// Coldfire V1 XCSR
#define CFV1_XCSR_HALT          (0x80)
#define CFV1_XCSR_STOP          (0x40)
#define CFV1_XCSR_CLKSW         (0x04)
#define CFV1_XCSR_ENBDM         (0x01)
#define CFV1_XCSR_CSTAT         (7<<3)
#define CFV1_XCSR_CSTAT_OK      (0<<3)
#define CFV1_XCSR_CSTAT_IVALID  (1<<3)
#define CFV1_XCSR_CSTAT_ILLEGAL (2<<3)
#define CFV1_XCSR_CSTAT_OVERRUN (4<<3)

// Coldfire V1 CSR2 (byte read/write)
#define CFV1_CSR2_COPHR          (0x20)
#define CFV1_CSR2_IOPHR          (0x10)
#define CFV1_CSR2_IADHR          (0x08)
#define CFV1_CSR2_BFHBR          (0x02)
#define CFV1_CSR2_BDFR           (0x01)

// Coldfire Debug Registers (32-bit read/write)
#define CFV1_CSR                 (0x00) // Debug register #
#define CFV1_CSR_SSM             (0x00000010UL) // Single Step mode

#endif //  _TARGET_DEFINES_H_