//====================================================================
//  NOTE - DO NOT CHANGE THIS FILE
//  If you need to create another configuration make a copy of this file 
//  under a new name and change Configure.h appropropriately.
//====================================================================

//================================================================================
// This file contains version specific information and configuration.
//  USBDM - Universal BDM, JB16 Version - See schematic provided
//  Supports HC12, HCS08, RS08 and CFv1
//================================================================================
#ifndef _CONFIGURE_H_
#define _CONFIGURE_H_

//=================================================================================
// Debug pin - used to check timing and hardware sequences etc.
//
#if (DEBUG&(ACK_DEBUG|SYNC_DEBUG|RESET_DEBUG|CYCLE_DEBUG))
#define DEBUG_PIN_DDR DDRE_DDRE0
#define DEBUG_PIN     PTE_PTE0
#endif


//==========================================================================================
// Capabilities of the hardware - used to enable/disable appropriate code
//
#define CAP_RESET       (1<<0)   // RESET can be driven (required for HC12)
#define CAP_FLASH       (1<<1)   // 12 V Flash programming supply available (RS08)
#define CAP_VDDCONTROL  (1<<2)   // Control over target Vdd
#define CAP_VDDSENSE    (1<<3)   // Sensing of target Vdd

#define CAPABILITY (CAP_RESET|CAP_FLASH|CAP_VDDCONTROL|CAP_VDDSENSE)

//==========================================================================================
// CPUs supported (just clock frequency changes)
//
#define JB8  (1)      // Not supported 
#define JB16 (2)
#define JM60 (3)

#define CPU  JB16     // Choose CPU being used

#ifndef CPU
#error "Please define CPU in Configure.h"
#define CPU JB16
#endif

#if (CPU == JB16)
#define OSC_FREQ     (12000000UL)               // Oscillator frequency
#define BUS_FREQ     (OSC_FREQ/2)               // Bus freq. derived from oscillator
//#error "JB16 is not supported by this version of the software"
#elif (CPU == JB8)
#define OSC_FREQ     (12000000UL)               // Oscillator frequency
#define BUS_FREQ     (OSC_FREQ/4)               // Bus freq. derived from oscillator
#error "JB8 is not supported by this version of the software"
#elif (CPU == JM60)
#define OSC_FREQ     (48000000UL)               // Oscillator frequency
#define BUS_FREQ     (OSC_FREQ/2)               // Bus freq. derived from oscillator
#error "JM60 is not supported by this version of the software"
#else
#error "Please correctly define CPU in Configure.h"
#define CPU JB16
#endif






//===========================================================================================
// Two types of BDM interface chips are supported
// I believe use of the transceiver has a hardware bug!
//
#define LVC125 (1)  // Hex buffer (LVC125, LVC2G125 etc) as used in original TBDML
#define LVC45  (2)  // Transceiver (LVC1T45, LVC2T45 etc) as used in OSBDM08

#define DRIVER  LVC125  // Choose driver IC being used

#ifndef DRIVER
#error "Please define DRIVER in Configure.h"
#define DRIVER LVC125
#endif

#if ((DRIVER!=LVC125) && (DRIVER!=LVC45))
#error "Please correctly define DRIVER in Configure.h"
#define DRIVER LVC125
#endif







//===========================================================================================
// Three platforms
//  This setting has two effects:
//       It controls how the hardware identifies itself to the host - either as TBDML or OSBDM
//       It controls the default target device (TBDML=>HC12 or OSBDML=>HCS08)
//  The code is able to support all devices (HC12/HCS12/HCS08) with either setting
// This is mainly to maintain backward compatibility with earlier driver DLLs (TBDML.dll & OSBDM.dll)
//
#define TBDML (1)
#define OSBDM (2)
#define USBDM (3)

//#define PLATFORM TBDML
#define PLATFORM USBDM   // Choose BDM hardware platform

#ifndef PLATFORM
#error "Please define PLATFORM in Configure.h"
#define PLATFORM USBDM   // Just to prevent further error messages.
#endif

#if (PLATFORM==TBDML)
#define VendorID  (0x0425)
#define ProductID (0x1000)
#define ProductDescription "OSBDM/TBDML Version 0.1 (TBDML build)"
#elif (PLATFORM==OSBDM) 
#define VendorID  (0x15A2)
#define ProductID (0x0021)
#define ProductDescription "OSBDM/TBDML Version 0.1 (OSBDM build)"
#elif (PLATFORM==USBDM) 
#define VendorID  (0x15A2)
#define ProductID (0x0021)
#define ProductDescription "USBDM Version 1.0"
#else
#error "Please correctly define PLATFORM in Configure.h"
#define PLATFORM OSBDM // Just to prevent further error messages.
#endif

#define VERSION_SW  ((0<<4)+1) // 0.1
#define VERSION_HW  ((0<<4)+1) // 0.1
#define VERSION     (VERSION_HW+256*VERSION_SW)  /* intel endianism */





//=================================================================================
// Rx & Tx Pin assignments
// Please note: pin assignments cannot be changed freely
// RX AND TX ROUTINES ARE DEPENDENT ON THIS SPECIFIC ASSIGNMENTS

//#error "Please check the I/O definitions in Configure.h"

//=================================================================================
// Data Port Bit masks - hard coded in Rx/Tx routines
//
#define DATA_PORT         PTA
#define DATA_PORT_DDR     DDRA

// BDM data out pin - hard coded in Rx/Tx routines
#define BDM_OUT           PTA_PTA7
#define BDM_OUT_BIT       (7)  // Bit number!
#define BDM_OUT_MASK      (1<<BDM_OUT_BIT)

// BDM data in pin - hard coded in Rx/Tx routines
#define BDM_IN            PTA_PTA0
#define BDM_IN_BIT        (0)  // Bit number!
#define BDM_IN_MASK       (1<<BDM_IN_BIT)

// Primary BDM data direction pin - only disabled during Tx routines
#define BDM_DIR_Rx        PTA_PTA6
#define BDM_DIR_Rx_MASK   PTA_PTA6_MASK

// Polarity of BDM buffer enable/direction varies with driver IC
#if (DRIVER == LVC125)
#define BDM_DIR_Rx_RD     BDM_DIR_Rx_MASK
#define BDM_DIR_Rx_WR     0

#elif (DRIVER == LVC45)
#define BDM_DIR_Rx_RD     0
#define BDM_DIR_Rx_WR     BDM_DIR_Rx_MASK
#endif

#if (CAPABILITY&CAP_RESET)

// RESET output pin
#define RESET_OUT         PTA_PTA4
#define RESET_OUT_MASK    PTA_PTA4_MASK

// RESET input pin
#define RESET_IN          PTA_PTA5
#define RESET_IN_MASK     PTA_PTA5_MASK

#else

#define RESET_OUT_MASK    0
#define RESET_IN_MASK     0

#endif // CAP_RESET

//=================================================================================
// Direction Port bit masks
//
#define DIR_PORT          PTC
#define DIR_PORT_DDR      DDRC

// Secondary BDM data direction pin - only enabled during Tx routines
#define BDM_DIR           PTC_PTC1
#define BDM_DIR_BIT       (1) // Bit #
#define BDM_DIR_MASK      (1<<BDM_DIR_BIT)

#define BDM_DIR_DDR       DDRC_DDRC1

// Polarity of BDM buffer enable/direction varies with driver
// Following MACROs only used by Tx related routines
#if (DRIVER == LVC125)
#define BDM_DIR_RD        BDM_DIR_MASK
#define BDM_DIR_WR        0
#define BDM_ENABLE_TX()   (BDM_DIR=0)
#define BDM_3STATE_TX()   (BDM_DIR=1)
#define BDM_ENABLE_ASM_TX BCLR BDM_DIR_BIT,DIR_PORT
#define BDM_3STATE_ASM_TX BSET BDM_DIR_BIT,DIR_PORT
#elif (DRIVER == LVC45)
#define BDM_DIR_RD        0
#define BDM_DIR_WR        BDM_DIR_MASK
#define BDM_ENABLE_TX()   (BDM_DIR=1)
#define BDM_3STATE_TX()   (BDM_DIR=0)
#define BDM_ENABLE_ASM_TX BSET BDM_DIR_BIT,DIR_PORT
#define BDM_3STATE_ASM_TX BCLR BDM_DIR_BIT,DIR_PORT
#endif

#if (CAPABILITY&CAP_RESET)

// RESET data direction pin
#define RESET_DIR         PTC_PTC0
#define RESET_DIR_BIT     (0) // Bit #
#define RESET_DIR_MASK    (1<<RESET_DIR_BIT)

// Polarity of RESET buffer enable/direction varies with driver
#if (DRIVER == LVC125)    // Change for LVC125 versus LVC45
#define RESET_DIR_RD      RESET_DIR_MASK
#define RESET_DIR_WR      0
#define RESET_ENABLE()    (RESET_DIR=0)
#define RESET_3STATE()    (RESET_DIR=1)
#elif (DRIVER == LVC45)
#define RESET_DIR_RD      0
#define RESET_DIR_WR      RESET_DIR_MASK
#define RESET_ENABLE()    RESET_DIR=1
#define RESET_3STATE()    RESET_DIR=0
#endif

#else // !(CAPABILITY&CAP_RESET)

#define RESET_DIR_MASK    0
#define RESET_DIR_RD      0
#define RESET_DIR_WR      0
#define RESET_ENABLE()    ;
#define RESET_3STATE()    ;

#endif // CAP_RESET





//=================================================================================
// LED Port bit masks
// JB16 has built in current limits on PTD.2-5 so no resistors are needed.
// A resistor is needed if PTD.0 is used (original TBDML).
//
#define GREEN_LED_MASK  (PTD_PTD3_MASK)
#define RED_LED_MASK    (PTD_PTD4_MASK)
#define LED_PORT_DATA   (PTD)
#define LED_PORT_DDR    (DDRD)

/* LEDs off, LED pins are outputs with Io limit */  
#define LED_INIT()         (LED_PORT_DATA |= RED_LED_MASK|GREEN_LED_MASK); \
                           (POCR_PTDLDD = 1); \
                           (LED_PORT_DDR  |= RED_LED_MASK|GREEN_LED_MASK)
#define GREEN_LED_ON()     (LED_PORT_DATA &=~GREEN_LED_MASK)
#define GREEN_LED_OFF()    (LED_PORT_DATA |= GREEN_LED_MASK)
#define GREEN_LED_TOGGLE() (LED_PORT_DATA ^= GREEN_LED_MASK)
#define RED_LED_ON()       (LED_PORT_DATA &=~RED_LED_MASK)
#define RED_LED_OFF()      (LED_PORT_DATA |= RED_LED_MASK)
#define RED_LED_TOGGLE()   (LED_PORT_DATA ^= RED_LED_MASK)

//=================================================================================
// Flash programming control
//
#if (CAPABILITY&CAP_FLASH)

// Flash 12V charge pump control
#define FLASH12_ENn      (PTD_PTD2)
#define FLASH12V_DDR     (DDRD_DDRD2)

// Drive transistor for VPP on/off
#define VPP_EN           (PTD_PTD5)
#define VPP_EN_DDR       (DDRD_DDRD5)

// Order of statements within the following macros may be important 
#define FLASH12V_ON()   FLASH12_ENn=0; FLASH12V_DDR=1;
#define FLASH12V_OFF()  FLASH12_ENn=1; FLASH12V_DDR=1;

#define VPP_ON()        VPP_EN=1;  VPP_EN_DDR=1; // VPP Control pin is output, VPP on
#define VPP_OFF()       VPP_EN=0;  VPP_EN_DDR=1; // VPP Control pin is output, VPP off 

#else // !CAP_FLASH
// No Flash programming supply
#define FLASH12V_ON()   ;
#define FLASH12V_OFF()  ;

#define VPP_ON()        ;
#define VPP_OFF()       ;

#endif //CAP_FLASH

//=================================================================================
// Target Vdd control

#if (CAPABILITY&CAP_VDDCONTROL)

//  Vdd on/off
#define VDD3_EN          (PTD_PTD1)
#define VDD3_EN_DDR      (DDRD_DDRD1)
#define VDD5_ENn         (PTD_PTD0)
#define VDD5_EN_DDR      (DDRD_DDRD0)

#define VDD_OFF()       VDD3_EN=0; VDD5_ENn=1; VDD3_EN_DDR=1; VDD5_EN_DDR=1; // Vdd Off
#define VDD3_ON()       VDD3_EN=1; VDD5_ENn=1; VDD3_EN_DDR=1; VDD5_EN_DDR=1; // Vdd = 3.3V
#define VDD5_ON()       VDD3_EN=0; VDD5_ENn=0; VDD3_EN_DDR=1; VDD5_EN_DDR=1; // Vdd = 5V

#else // !CAP_VDDCONTROL
// No Vdd control
#define VDD_OFF()       ; // Vdd Off
#define VDD3_ON()       ; // Vdd = 3.3V
#define VDD5_ON()       ; // Vdd = 5V

#endif // CAP_VDDCONTROL

//=================================================================================
// Use of 1k5 resistor on USB D- line
//
//  The JB16 has a programmable 1k5 pull-up resistor on the USB D- line.
//  Some boards have used an external resistor instead.
//
#define USBPUP_ON    (0) // Turn on internal PUP
#define USBPUP_OFF   (1) // Turn off internal PUP

#define USBPUP USBPUP_ON // Internal 1k5 PUP present on D-

#ifndef USBPUP
#error "Please define USBPUP in Configure.h"
#define USBPUP USBPUP_OFF
#endif

#if ((USBPUP != USBPUP_ON) && (USBPUP != USBPUP_OFF))
#error "Please correctly define USBPUP in Configure.h"
#endif

//=================================================================================
// Choice of input capture pins for SYNC & ACK timing
// This timer is also used for timing delays
//
#define IC_T1CH01 (1) // PTE1/T1CH01 pin
#define IC_T2CH01 (2) // PTE2/T2CH01 pin

#define IC_BDM_TIMING_CHANNEL IC_T2CH01

#if (IC_BDM_TIMING_CHANNEL == IC_T1CH01)
#define IC_BDM_TIMING_TMOD      T1MOD
#define IC_BDM_TIMING_TSC0      T1SC0
#define IC_BDM_TIMING_TSC       T1SC
#define IC_BDM_TIMING_TSC_TOF   T1SC_TOF
#define IC_BDM_TIMING_TCH0      T1CH0
#define IC_BDM_TIMING_TSC0_CH0F T1SC0_CH0F
#elif (IC_BDM_TIMING_CHANNEL == IC_T2CH01)
#define IC_BDM_TIMING_TMOD      T2MOD
#define IC_BDM_TIMING_TSC0      T2SC0
#define IC_BDM_TIMING_TSC       T2SC
#define IC_BDM_TIMING_TSC_TOF   T2SC_TOF
#define IC_BDM_TIMING_TCH0      T2CH0
#define IC_BDM_TIMING_TSC0_CH0F T2SC0_CH0F
#else
#error "Please define timer channel (IC_BDM_TIMING_CHANNEL) in the Configuration file"
#endif

//=================================================================================
//
#if (CAPABILITY&CAP_VDDSENSE)

#define VDD_SENSE       (PTE_PTE1)
#define VDD_SENSE_DDR   (DDRE_DDRE1)

#define IC_VDD_SENSING_CHANNEL IC_T1CH01

// Choice of input capture pins for Target Vdd Sensing
#if (IC_VDD_SENSING_CHANNEL == IC_T1CH01)
#define IC_VDD_SENSING_TMOD      T1MOD
#define IC_VDD_SENSING_TSC0      T1SC0
#define IC_VDD_SENSING_TSC       T1SC
#define IC_VDD_SENSING_TCH0      T1CH0
#define IC_VDD_SENSING_TSC_TOF   T1SC_TOF
#define IC_VDD_SENSING_TSC0_CH0F T1SC0_CH0F
#elif (IC_VDD_SENSING_CHANNEL == IC_T2CH01)
#define IC_VDD_SENSING_TMOD      T2MOD
#define IC_VDD_SENSING_TSC0      T2SC0
#define IC_VDD_SENSING_TSC       T2SC
#define IC_VDD_SENSING_TCH0      T2CH0
#define IC_VDD_SENSING_TSC_TOF   T2SC_TOF
#define IC_VDD_SENSING_TSC0_CH0F T2SC0_CH0F
#else
#error "Please define timer channel (IC_VDD_SENSING_CHANNEL) in the Configuration file"
#endif

#endif // CAP_VDDSENSE

/* SW Changes:

TBDML History
0.1   31/12/2004    Initial coding
0.2   17/02/2005    Fixed bug in ep2_recv routine
0.3   13/06/2005    Changed behaviour during target reset - reset kept flagged until the reset condition disapears 

OSBDM/TBDML History
0.1   16/2/2008     Initial build of combined TBDML/OSBDM code
                    Lots of changes to make the two codes compatible
                    but the only real fix is correcting BDM08_CMD_WRITECONTROL
*/

/* General comments:

Measurements of required stack size indicate 51 bytes. 
The stack is currently set to be 64 bytes and can probably be reduced slightly if needed.

*/

/* Known bugs:

1. device not functional after computer hibernation

When the device is attached and the computer is hibernated
and then woken up again, the computer does not seem to want to talk to the device
anymore. I do not know what to do about this. Other devices (like my mouse) seem to 
survive the same thing OK, so it must be the device, but what is it? Enabling/disabling
the device in device manager brings it into stop and wakes it up again as expected. The 
computer cuts power to the device during hibernation, what is it expecting??

2. detection of target speed does not work on old devices

HC12 and older S12 devices do not support SYNC feature. Trying to guess the speed by
trial and error has proven to be unreliable. I am not sure what algorithm to try here...

*/

#endif
